package cn.sino.mark.common.config.security;

import cn.sino.mark.common.filter.LoginFilter;
import cn.sino.mark.common.filter.TokenFilter;
import cn.sino.mark.common.properties.JwtProperties;
import cn.sino.mark.service.SysUserService;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.dao.DaoAuthenticationProvider;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import javax.annotation.Resource;

/**
 * security配置
 *
 * @author zhangamin
 * @date 2021/12/22
 */
@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    private final SysUserService sysUserService;
    private final UserDetailsService userDetailsService;
    private final PasswordEncoder passwordEncoder;
    private final StringRedisTemplate stringRedisTemplate;
    private final JwtProperties jwtProperties;

    @Resource(name = "markObjectMapper")
    private ObjectMapper objectMapper;


    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.cors().and().csrf().disable()
                .httpBasic().disable()
                // 对请求进行认证
                .authorizeRequests()
                //处理请求嗅探
                .antMatchers(HttpMethod.OPTIONS, "/**").permitAll()
                //SwaggerUI允许匿名访问
                .antMatchers(
                        "/swagger-ui.html",
                        "/swagger-ui/**",
                        "/swagger-resources",
                        "/swagger-resources/**",
                        "/v2/api-docs",
                        "/v3/api-docs",
                        "/configuration/ui",
                        "/webjars/**",
                        "/client/**",
                        "/api/**",
                        "/actuator",
                        "/sys-user/inner-login",
                        "/actuator/**"
                ).permitAll()
                // 所有请求需要身份认证
                .anyRequest().authenticated()
                .and()
                //自定义401/403返回结果
                .exceptionHandling()
                .authenticationEntryPoint(new CusAuthenticationEntryPoint())
                .accessDeniedHandler(new CusAccessDeniedHandler())
                .and()
                //禁止创建session
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                //登录验证
                .addFilterBefore(new LoginFilter("/sys-user/login", authenticationManager(), sysUserService, objectMapper), UsernamePasswordAuthenticationFilter.class)
                //Token验证
                .addFilterBefore(new TokenFilter(jwtProperties, stringRedisTemplate), UsernamePasswordAuthenticationFilter.class);

    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) {
        // 使用自定义身份验证组件
        DaoAuthenticationProvider daoAuthenticationProvider = new DaoAuthenticationProvider();
        daoAuthenticationProvider.setUserDetailsService(userDetailsService);
        daoAuthenticationProvider.setPasswordEncoder(passwordEncoder);
        auth.authenticationProvider(daoAuthenticationProvider);
    }

}
