package cn.sino.mark.common.filter;

import cn.hutool.json.JSONUtil;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.model.dto.LoginUserDTO;
import cn.sino.mark.common.properties.JwtProperties;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.google.common.base.Strings;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

import static cn.hutool.core.text.CharSequenceUtil.isNotBlank;

/**
 * @author zhangamin
 * @description 验证后台授权Token过滤器
 * @className cn.sino.coop.common.filter.TokenFilter
 * @date 2022/3/16
 */
public class TokenFilter extends OncePerRequestFilter {

    private final JwtProperties jwtProperties;
    private final StringRedisTemplate stringRedisTemplate;

    public TokenFilter(JwtProperties jwtProperties, StringRedisTemplate stringRedisTemplate) {
        this.jwtProperties = jwtProperties;
        this.stringRedisTemplate = stringRedisTemplate;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String tokenValue = request.getHeader(Constant.ADMIN_TOKEN_HEADER_KEY);
        if (Strings.isNullOrEmpty(tokenValue)) {
            tokenValue = request.getHeader(Constant.ADMIN_TOKEN_HEADER_KEY.toLowerCase());
        }
        if (isNotBlank(tokenValue)) {
            String token = decodeJwt(tokenValue);
            if (isNotBlank(token)) {
                // 头部信息中存在token
                String loginRedisKey = Constant.ADMIN_LOGIN_KEY + token;
                Object loginUserObj = stringRedisTemplate.opsForHash().get(loginRedisKey, Constant.LOGIN_USER_INFO);
                if (loginUserObj != null) {
                    LoginUserDTO loginUser = JSONUtil.toBean(loginUserObj.toString(), LoginUserDTO.class);
                    //将当前登陆用户信息注入到Authentication
                    UsernamePasswordAuthenticationToken authentication = new UsernamePasswordAuthenticationToken(loginUser, null, new ArrayList<>());
                    SecurityContextHolder.getContext().setAuthentication(authentication);
                    stringRedisTemplate.expire(loginRedisKey, Constant.ADMIN_LOGIN_SESSION_TIME, TimeUnit.HOURS);
                }
            }
        }
        filterChain.doFilter(request, response);
    }

    /**
     * 解析jwt
     *
     * @param jwt 解析的jwt字符串
     * @return token
     */
    private String decodeJwt(String jwt) {
        String token = "";
        try {
            Algorithm algorithm = Algorithm.HMAC256(jwtProperties.getAdminSecret());
            JWTVerifier verifier = JWT.require(algorithm).withIssuer(jwtProperties.getIssuer()).build();
            DecodedJWT decoded = verifier.verify(jwt);
            token = decoded.getClaim("uid").asString();
        } catch (Exception e) {
            logger.error("解析后台登录token jwt失败:", e);
        }
        return token;
    }

}
