package cn.sino.mark.common.processor;

import cn.sino.common.constant.BizStatus;
import cn.sino.common.exception.BizException;
import org.springframework.core.MethodParameter;
import org.springframework.core.convert.ConversionService;
import org.springframework.core.convert.TypeDescriptor;
import org.springframework.http.*;
import org.springframework.http.converter.GenericHttpMessageConverter;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.util.StringUtils;
import org.springframework.validation.DataBinder;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.method.annotation.ModelAttributeMethodProcessor;
import org.springframework.web.servlet.HandlerMapping;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.io.PushbackInputStream;
import java.lang.reflect.Type;
import java.util.Collections;
import java.util.EnumSet;
import java.util.Map;
import java.util.Set;

public class JsonModelAttributeMethodProcessor extends ModelAttributeMethodProcessor {

    private static final Object NO_VALUE = new Object();

    private static final Set<HttpMethod> SUPPORTED_METHODS = EnumSet.of(HttpMethod.POST, HttpMethod.PUT);

    private final HttpMessageConverter<?> converter;

    public JsonModelAttributeMethodProcessor(boolean annotationNotRequired, HttpMessageConverter<?> converter) {
        super(annotationNotRequired);
        this.converter = converter;
    }

    @Override
    protected final Object createAttribute(String attributeName, MethodParameter methodParam,
                                           WebDataBinderFactory binderFactory,
                                           NativeWebRequest request) throws Exception {
        String value = getRequestValueForAttribute(attributeName, request);
        if (value != null) {
            Object attribute = createAttributeFromRequestValue(value, attributeName, methodParam,
                    binderFactory, request);
            if (attribute != null) {
                return attribute;
            }
        }
        Object org = null;
        HttpServletRequest servletRequest = request.getNativeRequest(HttpServletRequest.class);
        if (null == servletRequest) {
            throw new BizException(BizStatus.SYSTEM_ERROR);
        }
        ServletServerHttpRequest inputMessage = new ServletServerHttpRequest(servletRequest);
        try {
            HttpMethod httpMethod = inputMessage.getMethod();
            if (SUPPORTED_METHODS.contains(httpMethod)) {
                MediaType contentType = inputMessage.getHeaders().getContentType();
                if (contentType == null) {
                    throw new BizException(BizStatus.SYSTEM_ERROR);
                }
                if (MediaType.APPLICATION_JSON_VALUE.contains(contentType.getType()) && httpMethod != null) {
                    inputMessage.getBody();
                    org = readWithMessageConverters(inputMessage, methodParam,
                            methodParam.getNestedGenericParameterType(), contentType);
                }
            }
        } catch (InvalidMediaTypeException ex) {
            throw new HttpMediaTypeNotSupportedException(ex.getMessage());
        }
        if (org == null) {
            org = super.createAttribute(attributeName, methodParam, binderFactory, request);
        }

        return org;
    }

    protected String getRequestValueForAttribute(String attributeName, NativeWebRequest request) {
        Map<String, String> variables = getUriTemplateVariables(request);
        if (StringUtils.hasText(variables.get(attributeName))) {
            return variables.get(attributeName);
        } else if (StringUtils.hasText(request.getParameter(attributeName))) {
            return request.getParameter(attributeName);
        } else {
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    protected final Map<String, String> getUriTemplateVariables(NativeWebRequest request) {
        Map<String, String> variables = (Map<String, String>) request
                .getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE, RequestAttributes.SCOPE_REQUEST);
        return (variables != null ? variables : Collections.emptyMap());
    }

    protected Object createAttributeFromRequestValue(String sourceValue, String attributeName,
                                                     MethodParameter methodParam,
                                                     WebDataBinderFactory binderFactory,
                                                     NativeWebRequest request) throws Exception {

        DataBinder binder = binderFactory.createBinder(request, null, attributeName);
        ConversionService conversionService = binder.getConversionService();
        if (conversionService != null) {
            TypeDescriptor source = TypeDescriptor.valueOf(String.class);
            TypeDescriptor target = new TypeDescriptor(methodParam);
            if (conversionService.canConvert(source, target)) {
                return binder.convertIfNecessary(sourceValue, methodParam.getParameterType(), methodParam);
            }
        }
        return null;
    }

    @Override
    protected void bindRequestParameters(WebDataBinder binder, NativeWebRequest request) {
        ServletRequest servletRequest = request.getNativeRequest(ServletRequest.class);
        ServletRequestDataBinder servletBinder = (ServletRequestDataBinder) binder;
        if (null == servletRequest) {
            throw new BizException(BizStatus.SYSTEM_ERROR);
        }
        servletBinder.bind(servletRequest);
    }

    protected Object readWithMessageConverters(HttpInputMessage inputMessage, MethodParameter parameter,
                                               Type targetType, MediaType contentType) {
        Class<?> contextClass = (parameter != null ? parameter.getContainingClass() : null);
        Object body = NO_VALUE;
        try {
            inputMessage = new EmptyBodyCheckingHttpInputMessage(inputMessage);
            GenericHttpMessageConverter<?> genericConverter = (GenericHttpMessageConverter<?>) converter;
            if (genericConverter.canRead(targetType, contextClass, contentType)) {
                inputMessage.getBody();
                body = genericConverter.read(targetType, contextClass, inputMessage);
            }
        } catch (IOException ex) {
            throw new HttpMessageNotReadableException("Could not read document: " + ex.getMessage(), inputMessage);
        }
        if (body == NO_VALUE) {
            return null;
        }
        return body;
    }

    private static class EmptyBodyCheckingHttpInputMessage implements HttpInputMessage {

        private final HttpHeaders headers;

        private final InputStream body;

        public EmptyBodyCheckingHttpInputMessage(HttpInputMessage inputMessage) throws IOException {
            this.headers = inputMessage.getHeaders();
            InputStream inputStream = inputMessage.getBody();
            if (inputStream.markSupported()) {
                inputStream.mark(1);
                this.body = (inputStream.read() != -1 ? inputStream : null);
                inputStream.reset();
            } else {
                PushbackInputStream pushbackInputStream = new PushbackInputStream(inputStream);
                int b = pushbackInputStream.read();
                if (b == -1) {
                    this.body = null;
                } else {
                    this.body = pushbackInputStream;
                    pushbackInputStream.unread(b);
                }
            }
        }

        @Override
        public HttpHeaders getHeaders() {
            return this.headers;
        }

        @Override
        public InputStream getBody() {
            return this.body;
        }
    }

}
