package cn.sino.mark.common.utils;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 集合切分工具类
 *
 * @author Live.InPast
 * @date 2021/12/31
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class ListsUtil {

    /**
     * 对list进行拆分处理,无返回
     *
     * @param list      需要拆分的集合
     * @param batchSize 每批次处理数据行数
     * @param function  拆分后执行
     * @param <T>       入参类型
     */
    public static <T> void splitRun(List<T> list, int batchSize, Consumer<List<T>> function) {
        if (list != null) {
            Stream.iterate(0, n -> n + 1).limit(countBatchNum(list.size(), batchSize))
                    .forEach(n ->
                            function.accept(list.stream()
                                    .skip((long) n * batchSize)
                                    .limit(batchSize)
                                    .collect(Collectors.toList()))
                    );
        }
    }

    /**
     * 对list进行拆分处理并返回集合
     *
     * @param list      需要拆分的集合
     * @param batchSize 每批次处理数据行数
     * @param function  执行函数
     * @param <T>       入参类型
     * @param <R>       出参类型
     * @return List<R>  集合的数量等于传入的集合数量(每一条数据返回一个结果)
     */
    public static <T, R> List<R> splitRun(List<T> list, int batchSize, Function<List<T>, ? extends Stream<R>> function) {
        return splitRunWithStream(list, batchSize, function).collect(Collectors.toList());
    }

    /**
     * 对list进行拆分处理并返回Stream流
     *
     * @param list      需要拆分的集合
     * @param batchSize 每批次处理数据行数
     * @param function  执行函数
     * @param <T>       入参类型
     * @param <R>       出参类型
     * @return Stream<R>
     */
    public static <T, R> Stream<R> splitRunWithStream(List<T> list, int batchSize, Function<List<T>, ? extends Stream<R>> function) {
        if (list == null) {
            return Stream.empty();
        }
        return Stream.iterate(0, n -> n + 1)
                .limit(countBatchNum(list.size(), batchSize))
                .map(n -> list.stream()
                        .skip((long) n * batchSize)
                        .limit(batchSize)
                        .collect(Collectors.toList()))
                .flatMap(function);
    }


    /**
     * 对list进行拆分处理
     * 每批次返回值聚合成一个对象
     *
     * @param list      需要拆分的集合
     * @param batchSize 每批次处理数据行数
     * @param function  执行函数
     * @param <T>       入参类型
     * @param <R>       出参类型
     * @return List<R>  集合的数量等于批次数量(一个批次返回一个结果)
     */
    public static <T, R> List<R> splitRunAgg(List<T> list, int batchSize, Function<List<T>, ? extends R> function) {
        return splitRunAggWithStream(list, batchSize, function).collect(Collectors.toList());
    }


    /**
     * 对list进行拆分处理并返回Stream流
     * 每批次返回值聚合成一个对象
     *
     * @param list      需要拆分的集合
     * @param batchSize 每批次处理数据行数
     * @param function  执行函数
     * @param <T>       入参类型
     * @param <R>       出参类型
     * @return Stream<R> 流的元素数量等于批次数量(一个批次返回一个结果)
     */
    public static <T, R> Stream<R> splitRunAggWithStream(List<T> list, int batchSize, Function<List<T>, ? extends R> function) {
        if (list == null) {
            return Stream.empty();
        }
        return Stream.iterate(0, n -> n + 1)
                .limit(countBatchNum(list.size(), batchSize))
                .map(n -> list.stream()
                        .skip((long) n * batchSize)
                        .limit(batchSize)
                        .collect(Collectors.toList()))
                .map(function);
    }

    /**
     * 计算批次
     *
     * @param size   集合总数量
     * @param maxNum 每批次数量
     * @return 批次数量
     */
    private static Integer countBatchNum(Integer size, Integer maxNum) {
        return (size + maxNum - 1) / maxNum;
    }


}
