package cn.sino.mark.controller.web;

import cn.sino.common.exception.BizException;
import cn.sino.common.vo.RestResponseVO;
import cn.sino.mark.common.model.dto.AddCategoryDTO;
import cn.sino.mark.common.model.dto.ModifyCategoryDTO;
import cn.sino.mark.common.model.vo.CategoryTreeVo;
import cn.sino.mark.service.CategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

import static cn.hutool.core.text.CharSequenceUtil.isBlank;
import static cn.hutool.core.text.CharSequenceUtil.isEmpty;

/**
 * @author zhangamin
 * @description 分类管理控制层
 * @className cn.sino.mark.controller.web.CategoryController
 * @date 2022/3/27
 */
@Api(tags = "分类管理")
@RestController
@RequestMapping("/category")
@RequiredArgsConstructor
public class CategoryController {

    private final CategoryService categoryService;

    @ApiOperation("获取分类树列表")
    @GetMapping("/tree")
    public RestResponseVO<List<CategoryTreeVo>> tree() {
        return RestResponseVO.success(categoryService.getCategoryTree());
    }

    @ApiOperation("获取分类树快照")
    @GetMapping("/tree-snapshot/{taskId}")
    public RestResponseVO<List<Object>> treeSnapshot(@PathVariable("taskId") Long taskId) {
        return RestResponseVO.success(categoryService.treeSnapshot(taskId));
    }

    @ApiOperation("新增分类")
    @PostMapping("/add")
    @PreAuthorize("hasPermission('add-child-category-btn')")
    public RestResponseVO<Void> add(@Valid @RequestBody AddCategoryDTO addCategoryDTO) {
        categoryService.addCategory(addCategoryDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("修改分类")
    @PutMapping("/edit/{categoryId}")
    @PreAuthorize("hasPermission('modify-category-btn')")
    @ApiImplicitParam(name = "categoryId", value = "分类ID", paramType = "path", dataTypeClass = String.class, required = true)
    public RestResponseVO<Void> modify(@Valid @RequestBody ModifyCategoryDTO modifyCategoryDTO, @PathVariable("categoryId") String categoryId) {
        if (isBlank(categoryId) || isEmpty(categoryId)) {
            throw new BizException("请求参数有误");
        }
        categoryService.modifyCategory(modifyCategoryDTO, categoryId.trim());
        return RestResponseVO.success();
    }

    @ApiOperation("删除(作废)分类")
    @ApiImplicitParam(name = "categoryId", value = "分类ID", paramType = "path", dataTypeClass = String.class, required = true)
    @DeleteMapping("/{categoryId}")
    @PreAuthorize("hasPermission('delete-category-btn')")
    public RestResponseVO<Void> remove(@PathVariable("categoryId") String categoryId) {
        if (isBlank(categoryId) || isEmpty(categoryId)) {
            throw new BizException("请求参数有误");
        }
        categoryService.removeCategory(categoryId.trim());
        return RestResponseVO.success();
    }

}