package cn.sino.mark.controller.web;

import cn.sino.common.base.PageResult;
import cn.sino.common.vo.RestResponseVO;
import cn.sino.mark.common.model.dto.*;
import cn.sino.mark.common.model.vo.*;
import cn.sino.mark.service.ProjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @author zhangamin
 * @description 项目管理
 * @className cn.sino.mark.controller.web.ProjectController
 * @date 2022/3/27
 */
@Api(tags = "项目管理")
@RestController
@RequestMapping("/project")
@RequiredArgsConstructor
public class ProjectController {

    private final ProjectService projectService;

    @ApiOperation("资源分配列表")
    @PostMapping("/page")
    public RestResponseVO<PageResult<ProjectPageItemVO>> page(@RequestBody QueryProjectDTO queryProjectDTO) {
        return RestResponseVO.success(projectService.getProjectPage(queryProjectDTO));
    }

    @ApiOperation("新增项目基本信息")
    @PostMapping("/add")
    @PreAuthorize("hasPermission('add-project-btn')")
    public RestResponseVO<Long> add(@Valid @RequestBody AddProjectDTO addProjectDTO) {
        // 返回项目ID
        return RestResponseVO.success(projectService.addProject(addProjectDTO));
    }

    @ApiOperation("新增项目分类配置")
    @PostMapping("/category/add")
    @PreAuthorize("hasPermission('add-project-btn')")
    public RestResponseVO<Void> addCategory(@Valid @RequestBody AddProjectCategoryDTO addProjectCategoryDTO) {
        projectService.addCategory(addProjectCategoryDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("新增项目资源配置")
    @PostMapping("/resource/add")
    @PreAuthorize("hasPermission('add-project-btn')")
    public RestResponseVO<Void> addResource(@Valid @RequestBody AddProjectResourceDTO addProjectResourceDTO) {
        projectService.addResource(addProjectResourceDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("新增项目人员配置")
    @PostMapping("/user/add")
    @PreAuthorize("hasPermission('add-project-btn')")
    public RestResponseVO<List<ProjectTaskAssignVO>> addUser(@Valid @RequestBody AddProjectUserDTO addProjectUserDTO) {
        return RestResponseVO.success(projectService.addUsers(addProjectUserDTO));
    }


    @ApiOperation("项目资源分配完成")
    @PostMapping("/add/complete/{projectId}")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('add-project-btn')")
    public RestResponseVO<Void> assignComplete(@Valid @RequestBody CompleteProjectDTO completeProjectDTO, @PathVariable("projectId") Long projectId) {
        projectService.assignComplete(completeProjectDTO, projectId);
        return RestResponseVO.success();
    }

    @ApiOperation("取消项目")
    @DeleteMapping("/{projectId}")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('delete-project-btn')")
    public RestResponseVO<Void> deleteProject(@PathVariable("projectId") Long projectId) {
        projectService.deleteProject(projectId);
        return RestResponseVO.success();
    }


    @ApiOperation("查看项目任务列表")
    @GetMapping("/task/page/{projectId}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true),
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query", dataTypeClass = Integer.class, required = true)
    })
    public RestResponseVO<PageResult<ProjectTaskPageItemVO>> taskPage(@PathVariable("projectId") Long projectId, @RequestParam("pageNum") Integer pageNum) {
        return RestResponseVO.success(projectService.getTaskPage(projectId, pageNum));
    }

    @ApiOperation("重新分配任务打标人")
    @PutMapping("/reassign/task")
    @PreAuthorize("hasPermission('reassign-btn')")
    public RestResponseVO<Void> reAssignTask(@Valid @RequestBody ReAssignProjectTaskDTO reAssignProjectTaskDTO) {
        projectService.reAssignTask(reAssignProjectTaskDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("查询任务明细")
    @GetMapping("/task-details/{taskId}")
    @ApiImplicitParam(name = "taskId", value = "任务ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<List<ProjectTaskDetailVO>> getTaskDetails(@PathVariable("taskId") Long taskId) {
        return RestResponseVO.success(projectService.getTaskDetails(taskId));
    }


    @ApiOperation("我的项目任务列表")
    @ApiImplicitParam(name = "resourceId", value = "资源ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @GetMapping("/task/my-page")
    public RestResponseVO<PageResult<ProjectTaskPageItemVO>> myTaskPage(@Valid @RequestBody QueryMyProjectTaskDTO queryMyProjectTaskDTO) {
        return RestResponseVO.success(PageResult.of(0, List.of(new ProjectTaskPageItemVO())));
    }

    @ApiOperation("项目打标")
    @PostMapping("/task/mark")
    @PreAuthorize("hasPermission('mark-btn')")
    public RestResponseVO<Void> mark(@Valid @RequestBody MarkProjectTaskDTO markProjectTaskDTO) {
        projectService.mark(markProjectTaskDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("查询项目基本信息")
    @GetMapping("/{projectId}")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<ProjectBaseInfoVO> getProjectBaseInfo(@PathVariable("projectId") Long projectId) {
        return RestResponseVO.success(projectService.getProjectBaseInfo(projectId));
    }

    @ApiOperation("查询任务基本信息")
    @GetMapping("/task/{taskId}")
    @ApiImplicitParam(name = "taskId", value = "任务ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<TaskBaseInfoVO> getTaskBaseInfo(@PathVariable("taskId") Long taskId) {
        return RestResponseVO.success(projectService.getTaskBaseInfo(taskId));
    }

    @ApiOperation("新增抽样验证")
    @PostMapping("/task/sample/add")
    @PreAuthorize("hasPermission('check-btn')")
    public RestResponseVO<List<ProjectTaskDetailVO>> addTaskSample(@Valid @RequestBody AddTaskSampleDTO addTaskSampleDTO) {
        return RestResponseVO.success(projectService.addTaskSample(addTaskSampleDTO));
    }

    @ApiOperation("项目抽查验证")
    @PostMapping("/task/sample-mark")
    @PreAuthorize("hasPermission('check-btn')")
    public RestResponseVO<Void> sampleMark(@Valid @RequestBody MarkProjectTaskDTO markProjectTaskDTO) {
        projectService.sampleMark(markProjectTaskDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("项目验证完成")
    @PutMapping("/verify/{projectId}")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('check-btn')")
    public RestResponseVO<Void> verifyProject(@PathVariable("projectId") Long projectId) {
        projectService.verifyProject(projectId);
        return RestResponseVO.success();
    }

    @ApiOperation("查询我的任务")
    @PostMapping("/my-task/page")
    public RestResponseVO<PageResult<ProjectTaskPageItemVO>> getMyTaskPage(@Valid @RequestBody QueryTaskDTO queryTaskDTO) {
        return RestResponseVO.success(projectService.getMyTaskPage(queryTaskDTO));
    }

    @ApiOperation("查询我的任务完成情况")
    @GetMapping("/my-task/statistics")
    public RestResponseVO<MyTaskStatisticVO> getMyTaskStatistics() {
        return RestResponseVO.success(projectService.getMyTaskStatistics());
    }

    @ApiOperation("标注结果查询")
    @PostMapping("/mark-result/page")
    public RestResponseVO<PageResult<ProjectTaskPageItemVO>> getMarkResultPage(@Valid @RequestBody QueryTaskDTO queryTaskDTO) {
        return RestResponseVO.success(projectService.getMarkResultPage(queryTaskDTO));
    }

    @ApiOperation("导出标注结果")
    @PostMapping(value = "/mark-result/json", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public void exportMarkResultJson(@Valid @RequestBody ExportTaskDTO exportTaskDTO, HttpServletResponse response) {
        projectService.exportMarkResultJson(exportTaskDTO, response);
    }

    @ApiOperation("分页查询任务明细")
    @GetMapping("/task-details/page/{taskId}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "taskId", value = "任务ID", paramType = "path", dataTypeClass = Long.class, required = true),
            @ApiImplicitParam(name = "dataId", value = "数据资源id", paramType = "query", dataTypeClass = Long.class, required = true),
            @ApiImplicitParam(name = "type", value = "类型", paramType = "query", dataTypeClass = Integer.class, required = true)
    })

    public RestResponseVO<List<ProjectTaskDetailVO>> getTaskDetailPage(@PathVariable("taskId") Long taskId,
                                                                       @RequestParam("dataId") Long dataId,
                                                                       @RequestParam("type") Long type) {
        return RestResponseVO.success(projectService.getTaskDetailPage(taskId, dataId, type));
    }

    @ApiOperation("查询项目信息")
    @GetMapping("/detail/{projectId}")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<ProjectVO> getProjectById(@PathVariable("projectId") Long projectId) {
        return RestResponseVO.success(projectService.getProjectById(projectId));
    }

    @ApiOperation("查询标注结果")
    @GetMapping("/task-detail/{detailId}")
    @ApiImplicitParam(name = "detailId", value = "任务明细ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<TaskDetailVO> getTaskDetail(@PathVariable("detailId") Long detailId) {
        return RestResponseVO.success(projectService.getTaskDetail(detailId));
    }

}