package cn.sino.mark.controller.web;

import cn.sino.common.base.PageResult;
import cn.sino.common.vo.RestResponseVO;
import cn.sino.mark.common.model.dto.AddResourceDTO;
import cn.sino.mark.common.model.dto.ImportResourceDTO;
import cn.sino.mark.common.model.dto.QueryResourceDTO;
import cn.sino.mark.common.model.vo.ImportResourceVO;
import cn.sino.mark.common.model.vo.ResourceBaseInfoVO;
import cn.sino.mark.common.model.vo.ResourcePageItemVO;
import cn.sino.mark.service.ResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

/**
 * @author zhangamin
 * @description 资源管理控制层
 * @className cn.sino.mark.controller.web.ResourceController
 * @date 2022/3/27
 */
@Api(tags = "资源管理")
@RestController
@RequestMapping("/resource")
@RequiredArgsConstructor
public class ResourceController {

    private final ResourceService resourceService;

    @ApiOperation("获取资源列表")
    @PostMapping("/page")
    public RestResponseVO<PageResult<ResourcePageItemVO>> page(@RequestBody QueryResourceDTO queryResourceDTO) {
        return RestResponseVO.success(resourceService.getResourcePage(queryResourceDTO));
    }

    @ApiOperation("新增资源")
    @PostMapping("/add")
    @PreAuthorize("hasPermission('add-resource-btn')")
    public RestResponseVO<Long> add(@Valid @RequestBody AddResourceDTO addResourceDTO) {
        return RestResponseVO.success(resourceService.addResource(addResourceDTO));
    }

    @ApiOperation("解析资源并上传")
    @PostMapping("/analyse/{resourceId}")
    @PreAuthorize("hasPermission('analyse-resource-btn')")
    @ApiImplicitParam(name = "resourceId", value = "资源ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<ImportResourceVO> analyseAndImport(MultipartFile uploadFile, @PathVariable("resourceId") Long resourceId) {
        return RestResponseVO.success(resourceService.analyseResourceV2(uploadFile, resourceId));
    }

    @ApiOperation("解析资源")
    @PostMapping("/analyse")
    @PreAuthorize("hasPermission('analyse-resource-btn')")
    @ApiImplicitParam(name = "type", value = "资源类型", paramType = "query", dataTypeClass = Integer.class, required = true)
    public RestResponseVO<ImportResourceVO> analyseResource(MultipartFile uploadFile, @RequestParam("type") Integer type) {
        return RestResponseVO.success(resourceService.analyseResource(uploadFile, type));
    }

    @ApiOperation("更新草稿状态")
    @PostMapping("/edit-draft")
    @PreAuthorize("hasPermission('import-resource-btn')")
    public RestResponseVO<Void> importResourceV2(@Valid @RequestBody ImportResourceDTO importResourceDTO) {
        resourceService.importResourceV2(importResourceDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("上传资源")
    @PostMapping("/import")
    @PreAuthorize("hasPermission('import-resource-btn')")
    public RestResponseVO<Void> importResource(@Valid @RequestBody ImportResourceDTO importResourceDTO) {
        resourceService.importResource(importResourceDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("删除(作废)资源")
    @ApiImplicitParam(name = "resourceId", value = "资源ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @DeleteMapping("/{resourceId}")
    @PreAuthorize("hasPermission('delete-resource-btn')")
    public RestResponseVO<Void> remove(@PathVariable("resourceId") Long resourceId) {
        resourceService.removeResource(resourceId);
        return RestResponseVO.success();
    }

    @ApiOperation("查看资源基本信息")
    @ApiImplicitParam(name = "resourceId", value = "资源ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @GetMapping("/base-info/{resourceId}")
    public RestResponseVO<ResourceBaseInfoVO> getBaseInfo(@PathVariable("resourceId") Long resourceId) {
        return RestResponseVO.success(resourceService.getBaseInfo(resourceId));
    }

    @ApiOperation("查看待选资源列表")
    @GetMapping("/not-assigned-list")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "query", dataTypeClass = Long.class, required = true)
    public RestResponseVO<List<ResourceBaseInfoVO>> getNotAssignedList(@RequestParam("projectId")Long projectId) {
        return RestResponseVO.success(resourceService.getNotAssignedList(projectId));
    }

}