package cn.sino.mark.controller.web;

import cn.sino.common.base.PageResult;
import cn.sino.common.vo.RestResponseVO;
import cn.sino.mark.common.model.dto.*;
import cn.sino.mark.common.model.vo.*;
import cn.sino.mark.service.SysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;

/**
 * @author zhangamin
 * @description 管理用户控制层
 * @className cn.sino.coop.controller.web.SysUserController
 * @date 2022/3/21
 */
@Api(tags = "系统用户管理")
@RestController
@RequestMapping("/sys-user")
@RequiredArgsConstructor
public class SysUserController {

    private final SysUserService userService;

    @ApiOperation("获取当前登录用户信息")
    @GetMapping("/login-info")
    public RestResponseVO<LoginUserInfoVO> getLoginUserInfo() {
        return RestResponseVO.success(userService.getLoginUserInfo());
    }

    @ApiOperation("新增外部用户")
    @PostMapping("/outer")
    @PreAuthorize("hasPermission('add-outer-user-btn')")
    public RestResponseVO<Void> addOuterUser(@Valid @RequestBody AddUserDTO addUserDTO) {
        userService.addOuterUser(addUserDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("查看用户信息")
    @GetMapping("/{userId}")
    @ApiImplicitParam(name = "userId", value = "用户ID", paramType = "path", dataTypeClass = Long.class, required = true)
    public RestResponseVO<UserDetailsVO> getOuterUserDetails(@PathVariable("userId") Long userId) {
        return RestResponseVO.success(userService.getOuterUserDetails(userId));
    }

    @ApiOperation("编辑外部用户")
    @PutMapping("/outer/{userId}")
    @ApiImplicitParam(name = "userId", value = "用户ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('modify-outer-user-btn')")
    public RestResponseVO<Void> editOuterUser(@Valid @RequestBody EditUserDTO editUserDTO, @PathVariable("userId") Long userId) {
        userService.editOuterUser(editUserDTO, userId);
        return RestResponseVO.success();
    }

    @ApiOperation("分页查询用户列表")
    @PostMapping("/page")
    public RestResponseVO<PageResult<UserPageVO>> getUserPage(@RequestBody QueryUserPageDTO queryUserPageDTO) {
        return RestResponseVO.success(userService.getUserPage(queryUserPageDTO));
    }

    @ApiOperation("删除用户")
    @DeleteMapping("/{userId}")
    @ApiImplicitParam(name = "userId", value = "用户ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('delete-user-btn')")
    public RestResponseVO<Void> deleteUser(@PathVariable("userId") Long userId) {
        userService.deleteUser(userId);
        return RestResponseVO.success();
    }

    @ApiOperation("给用户分配角色")
    @PostMapping("/assign-roles")
    public RestResponseVO<Void> assignRoles(@Valid @RequestBody AssignRolesDTO assignRolesDTO) {
        userService.assignUserRoles(assignRolesDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("待选标注人")
    @GetMapping("/mark-person-list")
    @ApiImplicitParam(name = "projectId", value = "项目ID", paramType = "query", dataTypeClass = Long.class, required = true)
    public RestResponseVO<List<UserBaseInfoVO>> getMarkPersonList(@RequestParam("projectId") Long projectId) {
        return RestResponseVO.success(userService.getMarkPersonList(projectId));
    }

    @ApiOperation("退出登录")
    @PostMapping("/logout")
    public RestResponseVO<Void> logout() {
        userService.logout();
        return RestResponseVO.success();
    }

    @ApiOperation("新增内部用户")
    @PostMapping("/inner")
    @PreAuthorize("hasPermission('add-inner-user-btn')")
    public RestResponseVO<Void> addInnerUser(@Valid @RequestBody AddInnerUserDTO addUserDTO) {
        userService.addInnerUser(addUserDTO);
        return RestResponseVO.success();
    }

    @ApiOperation("编辑内部用户")
    @PutMapping("/inner/{userId}")
    @ApiImplicitParam(name = "userId", value = "用户ID", paramType = "path", dataTypeClass = Long.class, required = true)
    @PreAuthorize("hasPermission('modify-inner-user-btn')")
    public RestResponseVO<Void> editInnerUser(@Valid @RequestBody AddInnerUserDTO addUserDTO, @PathVariable("userId") Long userId) {
        userService.editInnerUser(addUserDTO, userId);
        return RestResponseVO.success();
    }

    @ApiOperation("查询内部用户信息")
    @GetMapping("/inner/user-info")
    @ApiImplicitParam(name = "account", value = "账号", paramType = "query", dataTypeClass = Long.class, required = true)
    public RestResponseVO<InnerUserVO> getInnerUserInfo(@RequestParam("account") String account) {
        return RestResponseVO.success(userService.getInnerUserInfo(account));
    }

    @ApiOperation("内部用户登录")
    @PostMapping("/inner-login")
    public RestResponseVO<String> innerLogin(HttpServletRequest request) {
        return RestResponseVO.success(userService.innerLogin(request));
    }

    @ApiOperation("查询所有内部部门")
    @GetMapping("/inner/departments")
    public RestResponseVO<List<DepartmentTreeVO>> allDeptNotTree() {
        return RestResponseVO.success(userService.getAllDeptNotTree());
    }

}