package cn.sino.mark.manager;

import cn.sino.mark.dao.mapper.CategoryMapper;
import cn.sino.mark.dao.model.CategoryDO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author sch
 * @description 分类管理实现层
 * @className cn.sino.mark.manager.CategoryManager
 * @date 2022/4/1
 */
@Component
@RequiredArgsConstructor
public class CategoryManager {
    private final CategoryMapper categoryMapper;

    /**
     * 查询编号最大的一级分类
     *
     * @return CategoryDO
     */
    public CategoryDO queryLastLevelOneCategory() {
        return categoryMapper.selectLastLevelOneCategory();
    }

    /**
     * 根据名称查询
     *
     * @param categoryName 分类名称
     * @return CategoryDO
     */
    public CategoryDO queryByName(String categoryName) {
        QueryWrapper<CategoryDO> wrapper = new QueryWrapper<>();
        wrapper.eq("category_name", categoryName);
        wrapper.eq("is_delete", 0);
        return categoryMapper.selectOne(wrapper);
    }

    /**
     * 根据分类ID查询
     *
     * @param categoryId 分类ID
     * @return CategoryDO
     */
    public CategoryDO queryById(String categoryId) {
        return categoryMapper.selectById(categoryId);
    }

    /**
     * 查询当前层级下的最后一个分类
     *
     * @param parentId 父级id
     * @return CategoryDO
     */
    public CategoryDO queryLastCurLevelCategory(String parentId) {
        return categoryMapper.selectLastCurLevelCategory(parentId);
    }

    /**
     * 新增分类
     *
     * @param categoryDO 分类信息
     */
    public void addCategory(CategoryDO categoryDO) {
        categoryMapper.insert(categoryDO);
    }

    /**
     * 修改分类
     *
     * @param categoryDO 修改分类参数
     */
    public void modifyCategory(CategoryDO categoryDO) {
        categoryMapper.updateById(categoryDO);
    }

    /**
     * 根据父级id查询集合
     *
     * @param parentId 父级分类id
     * @return List<CategoryDO>
     */
    public List<CategoryDO> queryListByParentId(String parentId) {
        QueryWrapper<CategoryDO> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_category_id", parentId);
        wrapper.eq("is_delete", 0);
        return categoryMapper.selectList(wrapper);
    }

    /**
     * 根据id查询
     *
     * @param categoryId 分类id
     * @return CategoryDO
     */
    public CategoryDO queryByPrimary(String categoryId) {
        return categoryMapper.selectByPrimary(categoryId);
    }

    /**
     * 根据父级id查询分类是否存在
     *
     * @param categoryName 分类名称
     * @param parentId     父级id
     * @return CategoryDO
     */
    public CategoryDO queryByNameAndParent(String categoryName, String parentId) {
        QueryWrapper<CategoryDO> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_category_id", parentId);
        wrapper.eq("category_name", categoryName);
        wrapper.eq("is_delete", 0);
        return categoryMapper.selectOne(wrapper);
    }
}
