package cn.sino.mark.manager;

import cn.sino.common.exception.BizException;
import cn.sino.mark.common.model.bo.LabelBO;
import cn.sino.mark.common.model.dto.LabelPageDTO;
import cn.sino.mark.dao.mapper.LabelItemMapper;
import cn.sino.mark.dao.mapper.LabelMapper;
import cn.sino.mark.dao.model.LabelDO;
import cn.sino.mark.dao.model.LabelItemDO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 标签管理层实现
 *
 * @author zhangamin
 * @version 2.0 2022/8/15:添加标签相关功能(lgw)
 * @date 2022/8/8
 */
@Component
@RequiredArgsConstructor
public class LabelManager {

    private final LabelMapper labelMapper;
    private final LabelItemMapper labelItemMapper;


    /**
     * 获取页面
     *
     * @param reqDTO 请求传输对象
     * @return {@link Page}<{@link LabelBO}>
     */
    public Page<LabelBO> getPage(LabelPageDTO reqDTO) {
        return labelMapper.selectLabelPage(reqDTO);
    }


    /**
     * 获取所有
     *
     * @return {@link List}<{@link LabelBO}>
     */
    public List<LabelBO> getAll() {
        return labelMapper.selectAll();
    }

    /**
     * 添加标签
     *
     * @param name           标签名称
     * @param labelItemNames 标签可选项
     */
    @Transactional(rollbackFor = Exception.class)
    public LabelDO add(String name, List<String> labelItemNames) {
        LabelDO label = LabelDO.builder().name(name).build();
        labelMapper.insert(label);
        addLabelItem(label.getId(), labelItemNames);
        return label;
    }

    /**
     * 编辑标签
     *
     * @param label          标签
     * @param labelItemNames 标签可选项
     */
    @Transactional(rollbackFor = Exception.class)
    public void edit(LabelDO label, List<String> labelItemNames) {
        labelMapper.updateById(label);
        deleteLabelItemByLabelId(label.getId());
        addLabelItem(label.getId(), labelItemNames);
    }

    /**
     * 标签可选项添加
     *
     * @param labelId        标签
     * @param labelItemNames 标签可选项
     */
    public void addLabelItem(Long labelId, List<String> labelItemNames) {
        List<LabelItemDO> labelItemList = labelItemNames.stream().map(labelItemName ->
                LabelItemDO.builder().labelId(labelId).name(labelItemName).sort(System.currentTimeMillis()).build()
        ).collect(Collectors.toList());
        labelItemMapper.insertBatch(labelItemList);
    }

    /**
     * 标签删除
     *
     * @param labelId 标签id
     */
    @Transactional(rollbackFor = Exception.class)
    public void deleteById(Long labelId) {
        LabelDO label = checkById(labelId);
        int count = labelMapper.deleteById(label.getId());
        if (count == 0) {
            throw new BizException("标签删除失败");
        }
        deleteLabelItemByLabelId(label.getId());
    }

    /**
     * 标签可选项删除
     *
     * @param labelId 标签id
     */
    public void deleteLabelItemByLabelId(Long labelId) {
        QueryWrapper<LabelItemDO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("label_id", labelId);
        labelItemMapper.delete(queryWrapper);
    }

    /**
     * 根据标签id，校验标签是否存在
     *
     * @param id 标签id
     * @return 标签信息
     */
    public LabelDO checkById(Long id) {
        LabelDO label = labelMapper.selectById(id);
        if (label == null) {
            throw new BizException("当前标签不存在");
        }
        return label;
    }

    /**
     * 根据标签名称查询标签信息
     *
     * @param name 标签名称
     * @return 标签信息
     */
    public LabelDO getByName(String name) {
        QueryWrapper<LabelDO> wrapper = new QueryWrapper<>();
        wrapper.eq("name", name);
        return labelMapper.selectOne(wrapper);
    }

    /**
     * 获取标签业务对象
     *
     * @param labelId 标签ID
     * @return 标签
     */
    public LabelBO getLabelBoById(Long labelId) {
        LabelBO labelBO = labelMapper.selectInfoById(labelId);
        if (labelBO == null) {
            throw new BizException("标签不存在或已删除！");
        }
        return labelBO;
    }

    /**
     * 获取标签选项
     *
     * @param labelId 标签ID
     * @return 选项列表
     */
    public List<LabelItemDO> getItemsByLabelId(Long labelId) {
        QueryWrapper<LabelItemDO> wrapper = new QueryWrapper<>();
        wrapper.eq("label_id", labelId);
        return labelItemMapper.selectList(wrapper);
    }
}