package cn.sino.mark.manager;

import cn.sino.mark.common.model.bo.ExportDataBO;
import cn.sino.mark.common.model.bo.ProjectBO;
import cn.sino.mark.common.model.bo.ProjectTaskBO;
import cn.sino.mark.common.model.bo.ProjectTaskDetailBO;
import cn.sino.mark.common.model.dto.ExportTaskDTO;
import cn.sino.mark.common.model.dto.QueryProjectDTO;
import cn.sino.mark.common.model.dto.QueryTaskDTO;
import cn.sino.mark.dao.mapper.*;
import cn.sino.mark.dao.model.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author sch
 * @description 项目组实现层
 * @className cn.sino.mark.manager.ProjectGroupManager
 * @date 2022/4/3
 */
@Component
@RequiredArgsConstructor
public class ProjectManager {

    private final ProjectGroupMapper groupMapper;
    private final ProjectMapper projectMapper;
    private final ProjectCategoryMapper projectCategoryMapper;
    private final ProjectResourceMapper projectResourceMapper;
    private final ProjectUserMapper projectUserMapper;
    private final ProjectTaskMapper projectTaskMapper;
    private final ProjectTaskDetailMapper taskDetailMapper;
    private static final String DELETED = "is_delete";
    private static final String PROJECT_ID = "project_id";

    /**
     * 根据名称查询项目组
     *
     * @param groupName 组名
     * @return ProjectGroupDO
     */
    public ProjectGroupDO queryGroupByName(String groupName) {
        QueryWrapper<ProjectGroupDO> wrapper = new QueryWrapper<>();
        wrapper.eq("name", groupName);
        wrapper.eq(DELETED, 0);
        return groupMapper.selectOne(wrapper);
    }

    /**
     * 根据项目组ID查询
     *
     * @param groupId 项目组id
     * @return ProjectGroupDO
     */
    public ProjectGroupDO queryGroupById(Long groupId) {
        QueryWrapper<ProjectGroupDO> wrapper = new QueryWrapper<>();
        wrapper.eq("group_id", groupId);
        wrapper.eq(DELETED, 0);
        return groupMapper.selectOne(wrapper);
    }

    /**
     * 新增项目组
     *
     * @param groupDO 项目组信息
     */
    public void addGroup(ProjectGroupDO groupDO) {
        groupMapper.insert(groupDO);
    }

    /**
     * 修改项目组
     *
     * @param groupDO 项目组信息
     * @return int
     */
    public int modifyGroup(ProjectGroupDO groupDO) {
        return groupMapper.updateById(groupDO);
    }

    /**
     * 根据parentId查询
     *
     * @param groupId 组id
     * @return public List<ProjectGroupDO>
     */
    public List<ProjectGroupDO> queryGroupListByParentId(Long groupId) {
        QueryWrapper<ProjectGroupDO> wrapper = new QueryWrapper<>();
        wrapper.eq("parent_id", groupId);
        wrapper.eq(DELETED, 0);
        wrapper.eq("status", 0);
        return groupMapper.selectList(wrapper);
    }

    /**
     * 根据名称查询项目
     *
     * @param name 项目名
     * @return ProjectDO
     */
    public ProjectDO queryByName(String name) {
        QueryWrapper<ProjectDO> wrapper = new QueryWrapper<>();
        wrapper.eq("name", name);
        wrapper.eq(DELETED, 0);
        return projectMapper.selectOne(wrapper);
    }

    /**
     * 新增项目
     *
     * @param projectDO 项目信息
     */
    public void addProject(ProjectDO projectDO) {
        projectMapper.insert(projectDO);
    }

    /**
     * 根据项目id查询项目
     *
     * @param projectId 项目id
     * @return ProjectDO
     */
    public ProjectDO queryById(Long projectId) {
        QueryWrapper<ProjectDO> wrapper = new QueryWrapper<>();
        wrapper.eq(PROJECT_ID, projectId);
        wrapper.eq(DELETED, 0);
        return projectMapper.selectOne(wrapper);
    }

    /**
     * 新增项目-分类关联
     *
     * @param projectCategoryDO 项目-分类关联
     */
    public void addProjectCategory(ProjectCategoryDO projectCategoryDO) {
        projectCategoryMapper.insert(projectCategoryDO);
    }

    /**
     * 根据项目id查询资源集合
     *
     * @param projectId 项目id
     * @return List<ProjectResourceDO>
     */
    public List<ProjectResourceDO> queryProResListByProId(Long projectId) {
        QueryWrapper<ProjectResourceDO> wrapper = new QueryWrapper<>();
        wrapper.eq(PROJECT_ID, projectId);
        wrapper.eq(DELETED, 0);
        return projectResourceMapper.selectList(wrapper);
    }

    /**
     * 批量新增项目-资源
     *
     * @param list 集合
     */
    public void insertBatchRes(List<ProjectResourceDO> list) {
        projectResourceMapper.insertBatch(list);
    }

    /**
     * 批量新增项目-用户
     *
     * @param list 集合
     */
    public void insertBatchUsers(List<ProjectUserDO> list) {
        projectUserMapper.insertBatch(list);
    }

    /**
     * 更新项目
     *
     * @param projectDO 更新项目信息
     * @return int
     */
    public int updateById(ProjectDO projectDO) {
        return projectMapper.updateById(projectDO);
    }

    /**
     * 根据项目id清空项目-资源关联
     *
     * @param projectId 项目id
     */
    public void deleteProResByProId(Long projectId) {
        Map<String, Object> params = new HashMap<>();
        params.put(PROJECT_ID, projectId);
        projectResourceMapper.deleteByMap(params);
    }

    /**
     * 根据项目id删除项目-用户关联
     *
     * @param projectId 项目id
     */
    public void deleteProUsersByProId(Long projectId) {
        Map<String, Object> params = new HashMap<>();
        params.put(PROJECT_ID, projectId);
        projectUserMapper.deleteByMap(params);
    }

    /**
     * 根据项目id删除项目-任务关联
     *
     * @param projectId 项目id
     */
    public void deleteProTasksByProId(Long projectId) {
        Map<String, Object> params = new HashMap<>();
        params.put(PROJECT_ID, projectId);
        projectTaskMapper.deleteByMap(params);
    }

    /**
     * 根据项目id删除项目-任务明细关联
     *
     * @param projectId 项目id
     */
    public void deleteProTaskDetailsByProId(Long projectId) {
        Map<String, Object> params = new HashMap<>();
        params.put(PROJECT_ID, projectId);
        taskDetailMapper.deleteByMap(params);
    }

    /**
     * 根据项目id查询项目-用户数据
     *
     * @param projectId 项目id
     * @return List<ProjectUserDO>
     */
    public List<ProjectUserDO> queryProUseListByProId(Long projectId) {
        QueryWrapper<ProjectUserDO> wrapper = new QueryWrapper<>();
        wrapper.eq(PROJECT_ID, projectId);
        wrapper.eq(DELETED, 0);
        return projectUserMapper.selectList(wrapper);
    }

    /**
     * 批量新增项目-任务
     *
     * @param list 集合
     */
    public void insertBatchProTask(List<ProjectTaskDO> list) {
        projectTaskMapper.insertBatch(list);
    }

    /**
     * 批量新增任务明细
     *
     * @param list 集合
     */
    public void insertBatchTaskDetail(List<ProjectTaskDetailDO> list) {
        taskDetailMapper.insertBatch(list);
    }

    /**
     * 分页查询项目
     *
     * @param queryProjectDTO 查询条件
     * @return Page<ProjectBO>
     */
    public Page<ProjectBO> queryProjectPage(QueryProjectDTO queryProjectDTO) {
        return projectMapper.selectProjectPage(queryProjectDTO);
    }

    /**
     * 根据项目id删除项目-分类关联
     *
     * @param projectId 项目id
     */
    public void deleteProCategory(Long projectId) {
        Map<String, Object> params = new HashMap<>();
        params.put(PROJECT_ID, projectId);
        projectCategoryMapper.deleteByMap(params);
    }

    /**
     * 删除项目
     *
     * @param id 项目id
     */
    public void deleteById(Long id) {
        projectMapper.deleteById(id);
    }

    /**
     * 根据项目id分页查询任务
     *
     * @param projectId 项目id
     * @return Page<ProjectTaskBO>
     */
    public Page<ProjectTaskBO> queryTaskPageByProject(Long projectId) {
        return projectTaskMapper.selectPageByProjectId(projectId);
    }

    /**
     * 根据id查询任务
     *
     * @param taskId 任务id
     * @return ProjectTaskDO
     */
    public ProjectTaskDO queryTaskByTaskId(Long taskId) {
        QueryWrapper<ProjectTaskDO> wrapper = new QueryWrapper<>();
        wrapper.eq("task_id", taskId);
        wrapper.eq(DELETED, 0);
        return projectTaskMapper.selectOne(wrapper);
    }

    /**
     * 更细任务
     *
     * @param taskDO 任务 信息
     * @return int
     */
    public int updateTask(ProjectTaskDO taskDO) {
        return projectTaskMapper.updateById(taskDO);
    }

    /**
     * 查询任务明细
     *
     * @param taskId 任务id
     * @return List<ProjectTaskDetailBO>
     */
    public List<ProjectTaskDetailBO> queryDetailBOsByTaskId(Long taskId) {
        return taskDetailMapper.selectDetailBOsByTaskId(taskId);
    }

    /**
     * 查询任务明细
     *
     * @param taskDetailId 明细id
     * @return ProjectTaskDetailDO
     */
    public ProjectTaskDetailDO queryTaskDetailById(Long taskDetailId) {
        QueryWrapper<ProjectTaskDetailDO> wrapper = new QueryWrapper<>();
        wrapper.eq("detail_id", taskDetailId);
        return taskDetailMapper.selectOne(wrapper);
    }

    /**
     * 更新任务名称
     *
     * @param detailDO 任务明细
     * @return int
     */
    public int updateTaskDetail(ProjectTaskDetailDO detailDO) {
        return taskDetailMapper.updateById(detailDO);
    }

    /**
     * 查询用户的任务数量
     *
     * @param userId     用户id
     * @param isComplete 是否完成
     * @return int
     */
    public Integer queryTaskNumByUserId(Long userId, Integer isComplete) {
        return projectTaskMapper.selectTaskNumByUserId(userId, isComplete);
    }

    /**
     * 查询未完成的任务
     *
     * @param userId 用户id
     * @return List<ProjectTaskBO>
     */
    public List<ProjectTaskBO> queryNotCompTasksByUserId(Long userId) {
        return projectTaskMapper.selectNotCompTasksByUserId(userId);
    }

    /**
     * 分页查询任务
     *
     * @param queryTaskDTO 查询条件
     * @return Page<ProjectTaskBO>
     */
    public Page<ProjectTaskBO> queryTaskPage(QueryTaskDTO queryTaskDTO) {
        return projectTaskMapper.selectTaskPage(queryTaskDTO);
    }

    /**
     * 任务总数
     *
     * @param queryTaskDTO 查询条件
     * @return int
     */
    public Integer queryTaskPageTotal(QueryTaskDTO queryTaskDTO) {
        return projectTaskMapper.selectTaskPageTotal(queryTaskDTO);
    }

    /**
     * 导出标注结果
     *
     * @param exportTaskDTO 导出条件
     * @return List<ExportDataBO>
     */
    public List<ExportDataBO> exportTaskDetails(ExportTaskDTO exportTaskDTO) {
        return taskDetailMapper.exportTaskDetails(exportTaskDTO);
    }

    /**
     * 分页查询任务明细
     *
     * @param taskId 任务id
     * @param dataId 资源数据id
     * @param type   类型
     * @return List<ProjectTaskDetailBO>
     */
    public List<ProjectTaskDetailBO> queryDetailBOsByTaskId(Long taskId, Long dataId, Long type) {
        return taskDetailMapper.selectDetailBOPageByTaskId(taskId, dataId, type);
    }

    /**
     * 查看任务是否全部标注完成
     *
     * @param taskId 任务id
     * @return boolean
     */
    public boolean checkTaskFinish(Long taskId) {
        return taskDetailMapper.checkTaskFinish(taskId) <= 0;
    }

    /**
     * 查询项目-分类关系
     *
     * @param projectId 项目id
     * @return ProjectCategoryDO
     */
    public ProjectCategoryDO queryProCateById(Long projectId) {
        QueryWrapper<ProjectCategoryDO> wrapper = new QueryWrapper<>();
        wrapper.eq(PROJECT_ID, projectId);
        return projectCategoryMapper.selectOne(wrapper);
    }

    /**
     * 查询项目总数
     *
     * @param queryProjectDTO 查询条件
     * @return int
     */
    public Integer queryProjectPageTotal(QueryProjectDTO queryProjectDTO) {
        return projectMapper.selectProjectPageTotal(queryProjectDTO);
    }

    /**
     * 查询项目任务总数
     *
     * @param projectId 项目id
     * @return int
     */
    public Integer queryTaskTotalByProjectId(Long projectId) {
        return projectTaskMapper.selectTotalByProjectId(projectId);
    }

    /**
     * 查询任务标注数量
     *
     * @param taskId     任务id
     * @param type       标注类型
     * @param hasProblem 是否存疑
     * @return int
     */
    public int queryMarkNumByTask(Long taskId, Integer type, Integer hasProblem) {
        return taskDetailMapper.selectMarkNumByTask(taskId, type, hasProblem);
    }

    /**
     * 查询任务验证数量
     *
     * @param taskId     任务id
     * @param sampleType 验证结果
     * @return int
     */
    public int querySampleNumByTask(Long taskId, Integer sampleType) {
        return taskDetailMapper.selectSampleNumByTask(taskId, sampleType);
    }

    /**
     * 查询任务明细id集合
     *
     * @param taskId     任务id
     * @param sampleType 验证类型
     * @return List<Long>
     */
    public List<Long> queryDetailIdsByTask(Long taskId, Integer sampleType) {
        return taskDetailMapper.selectDetailIdsByTask(taskId, sampleType);
    }

    /**
     * 根据id集合查询
     *
     * @param detailIds id集合
     * @return List<ProjectTaskDetailBO>
     */
    public List<ProjectTaskDetailBO> queryDetailBOsByIds(List<Long> detailIds) {
        return taskDetailMapper.selectDetailBOsByIds(detailIds);
    }

    /**
     * 查询分类下是否有项目
     *
     * @param categoryId 分类id
     * @return List<ProjectGroupDO>
     */
    public List<ProjectGroupDO> queryProjectByCategory(String categoryId) {
        QueryWrapper<ProjectGroupDO> wrapper = new QueryWrapper<>();
        wrapper.eq("category_id", categoryId);
        wrapper.eq(DELETED, 0);
        return groupMapper.selectList(wrapper);
    }
}
