package cn.sino.mark.manager;

import cn.sino.mark.common.model.bo.ResourceBO;
import cn.sino.mark.common.model.dto.QueryResourceDTO;
import cn.sino.mark.dao.mapper.ResourceDataMapper;
import cn.sino.mark.dao.mapper.ResourceMapper;
import cn.sino.mark.dao.model.ResourceDO;
import cn.sino.mark.dao.model.ResourceDataDO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

import static cn.hutool.core.collection.CollUtil.isEmpty;
import static cn.hutool.core.collection.CollUtil.isNotEmpty;

/**
 * @author sch
 * @description 资源管理实现层
 * @className cn.sino.mark.manager.ResourceManager
 * @date 2022/4/1
 */
@Component
@RequiredArgsConstructor
public class ResourceManager {
    private final ResourceMapper resourceMapper;
    private final ResourceDataMapper resourceDataMapper;
    private static final String DELETED = "is_delete";

    /**
     * 根据名称查询
     *
     * @param name 名称
     * @return ResourceDO
     */
    public ResourceDO queryByName(String name) {
        QueryWrapper<ResourceDO> wrapper = new QueryWrapper<>();
        wrapper.eq("name", name);
        wrapper.eq(DELETED, 0);
        return resourceMapper.selectOne(wrapper);
    }

    /**
     * 新增资源
     *
     * @param resourceDO 资源信息
     */
    public void addResource(ResourceDO resourceDO) {
        resourceMapper.insert(resourceDO);
    }

    /**
     * 根据资源id查询
     *
     * @param resourceId 资源id
     * @return ResourceDO
     */
    public ResourceDO queryById(Long resourceId) {
        QueryWrapper<ResourceDO> wrapper = new QueryWrapper<>();
        wrapper.eq("resource_id", resourceId);
        wrapper.eq(DELETED, 0);
        return resourceMapper.selectOne(wrapper);
    }

    /**
     * 批量新增资源数据
     *
     * @param list 资源数据集
     */
    public void insertBatch(List<ResourceDataDO> list) {
        resourceDataMapper.batchInsert(list);
    }

    /**
     * 分页查询资源列表
     *
     * @param queryResourceDTO 查询条件
     * @return Page<ResourceDO>
     */
    public Page<ResourceBO> queryResourcePage(QueryResourceDTO queryResourceDTO) {
        return resourceMapper.pageResource(queryResourceDTO);
    }

    /**
     * 修改
     *
     * @param resourceDO 资源信息
     */
    public int modifyResource(ResourceDO resourceDO) {
        return resourceMapper.updateById(resourceDO);
    }

    /**
     * 根据资源id集合查询
     *
     * @param idList 资源id集合
     * @return List<ResourceDO>
     */
    public List<ResourceDO> queryListByIds(List<Long> idList) {
        return resourceMapper.selectListByIds(idList);
    }


    /**
     * 根据资源id集合查询资源数据
     *
     * @param list 资源id集合
     * @return List<ResourceDataDO>
     */
    public List<ResourceDataDO> queryDataListByResIds(List<Long> list) {
        if (isEmpty(list)) {
            return Collections.emptyList();
        } else {
            return resourceDataMapper.selectDataListByResIds(list);
        }
    }

    /**
     * 批量更新
     *
     * @param list 资源id集合
     */
    public void batchUpdate(List<ResourceDO> list) {
        if (isNotEmpty(list)) {
            resourceMapper.batchUpdate(list);
        }
    }

    /**
     * 查询查看待选资源列表
     *
     * @return List<ResourceDO>
     */
    public List<ResourceDO> queryNotAssignedList() {
        QueryWrapper<ResourceDO> wrapper = new QueryWrapper<>();
        wrapper.eq("status", 0);
        wrapper.eq(DELETED, 0);
        wrapper.eq("assign_status", 0);
        return resourceMapper.selectList(wrapper);
    }

    /**
     * 删除资源
     *
     * @param id 主键
     * @return int
     */
    public int removeResource(Long id) {
        return resourceMapper.deleteById(id);
    }

    /**
     * 查询资源数量
     *
     * @param resourceId 资源id
     * @return int
     */
    public Integer queryResourceCount(Long resourceId) {
        return resourceDataMapper.selectCountByResource(resourceId);
    }

    /**
     * 更新资源数据草稿状态
     *
     * @param resourceId 资源id
     */
    public void updateDraftByResourceId(Long resourceId) {
        resourceDataMapper.updateDraftByResourceId(resourceId);
    }

    /**
     * 删除资源数据
     *
     * @param resourceId 资源id
     */
    public void deleteDataByResourceId(Long resourceId) {
        resourceDataMapper.deleteByResourceId(resourceId);
    }
}
