package cn.sino.mark.manager;

import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.model.bo.SysUserBO;
import cn.sino.mark.common.model.dto.QueryUserPageDTO;
import cn.sino.mark.dao.mapper.*;
import cn.sino.mark.dao.model.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.Caching;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author zhangamin
 * @description 系统用户管理层实现
 * @className cn.sino.coop.manager.SysUserManager
 * @date 2022/3/17
 */
@Component
@RequiredArgsConstructor
public class SysUserManager {

    private final SysUserMapper sysUserMapper;
    private final SysRoleMapper sysRoleMapper;
    private final SysPermMapper sysPermMapper;
    private final SysUserRoleMapper userRoleMapper;
    private final SysDeptMapper sysDeptMapper;


    /**
     * 查询系统用户
     *
     * @param account 登录账号
     * @return cn.sino.coop.dao.model.SysUserDO
     */
    @Cacheable(value = Constant.USER_INFO_ACCOUNT, key = "#account", unless = "#result == null")
    public SysUserDO queryByAccount(String account) {
        QueryWrapper<SysUserDO> wrapper = new QueryWrapper<>();
        // 后台管理系统用户登录账户
        wrapper.eq("account", account);
        return sysUserMapper.selectOne(wrapper);
    }

    /**
     * 新增外部用户
     *
     * @param sysUserDO 用户信息
     */
    public void addUser(SysUserDO sysUserDO) {
        sysUserMapper.insert(sysUserDO);
    }

    /**
     * 根据用户ID查询
     *
     * @param userId 用户ID
     * @return SysUserDO
     */
    @Cacheable(value = Constant.USER_INFO_ID, key = "#userId", unless = "#result == null")
    public SysUserDO queryByUserId(Long userId) {
        QueryWrapper<SysUserDO> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id", userId);
        return sysUserMapper.selectOne(wrapper);
    }

    /**
     * 分页查询用户列表
     *
     * @param queryUserPageDTO 查询条件
     * @return Page<SysUserBO>
     */
    public Page<SysUserBO> queryUserPage(QueryUserPageDTO queryUserPageDTO) {
        return sysUserMapper.pageUsers(queryUserPageDTO);
    }

    /**
     * 编辑外部用户
     *
     * @param sysUserDO 用户信息
     */
    @Caching(evict = {
            @CacheEvict(value = Constant.USER_INFO_ACCOUNT, key = "#sysUserDO.account"),
            @CacheEvict(value = Constant.USER_INFO_ID, key = "#sysUserDO.userId")
    })
    public int modifyUserById(SysUserDO sysUserDO) {
        return sysUserMapper.updateById(sysUserDO);
    }

    /**
     * 删除外部用户
     *
     * @param userId 用户ID
     */
    @CacheEvict(value = Constant.USER_INFO_ID, key = "#userId")
    public void removeUserById(Long userId) {
        sysUserMapper.deleteById(userId);
    }

    /**
     * 查询用户角色
     *
     * @param loginUserId 用户ID
     * @return List<SysRoleDO>
     */
    public List<SysRoleDO> queryRolesByUserId(Long loginUserId) {
        return sysRoleMapper.listRolesByUserId(loginUserId);
    }

    /**
     * 查询用户权限
     *
     * @param userId 用户ID
     * @return List<SysPermDO>
     */
    public List<SysPermDO> queryPermByUserId(Long userId) {
        return sysPermMapper.listPermsByUserId(userId);
    }

    /**
     * 查询默认权限
     *
     * @return List<SysPermDO>
     */
    public List<SysPermDO> queryDefaultPerm() {
        QueryWrapper<SysPermDO> wrapper = new QueryWrapper<>();
        wrapper.eq("is_default", 1);
        return sysPermMapper.selectList(wrapper);
    }

    /**
     * 根据用户id集合查询
     *
     * @param userIds 用户id集合
     * @return List<SysUserDO>
     */
    public List<SysUserDO> queryListByUserIds(List<Long> userIds) {
        return sysUserMapper.selectListByUserIds(userIds);
    }

    /**
     * 查询待选标注人员
     *
     * @param projectId 项目id
     * @return List<SysUserBO>
     */
    public List<SysUserBO> queryMarkPersonList(Long projectId) {
        return sysUserMapper.selectMarkPersonList(projectId);
    }

    /**
     * 新增用户-角色
     *
     * @param userRoleDO 用户角色信息
     */
    public void addUserRole(SysUserRoleDO userRoleDO) {
        userRoleMapper.insert(userRoleDO);
    }

    /**
     * 批量新增用户角色
     *
     * @param list 集合
     */
    public void batchInsertUserRoles(List<SysUserRoleDO> list) {
        userRoleMapper.insertBatch(list);
    }

    /**
     * 删除用户角色信息
     *
     * @param userId 用户id
     */
    public void removeUserRoleByUserId(Long userId) {
        Map<String, Object> params = new HashMap<>();
        params.put("user_id", userId);
        userRoleMapper.deleteByMap(params);
    }

    /**
     * 获取所有部门
     *
     * @return {@link List}<{@link SysDeptDO}>
     */
    public List<SysDeptDO> getAllDepts() {
        return sysDeptMapper.selectAllNotTree();
    }
}