package cn.sino.mark.service.impl;

import cn.hutool.json.JSONUtil;
import cn.sino.common.exception.BizException;
import cn.sino.mark.common.enums.DeleteStatusEnum;
import cn.sino.mark.common.enums.IsOrNotEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.dto.AddCategoryDTO;
import cn.sino.mark.common.model.dto.ModifyCategoryDTO;
import cn.sino.mark.common.model.vo.CategoryTreeVo;
import cn.sino.mark.dao.model.CategoryDO;
import cn.sino.mark.dao.model.ProjectCategoryDO;
import cn.sino.mark.dao.model.ProjectTaskDO;
import cn.sino.mark.manager.CategoryManager;
import cn.sino.mark.manager.ProjectManager;
import cn.sino.mark.service.CategoryService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static cn.hutool.core.text.CharSequenceUtil.isBlank;
import static cn.hutool.core.text.CharSequenceUtil.isEmpty;

/**
 * @author sch
 * @description 分类管理实现类
 * @className cn.sino.mark.service.impl.CategoryServiceImpl
 * @date 2022/3/31
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CategoryServiceImpl implements CategoryService {


    private final CategoryManager categoryManager;
    private final SysLogEventPublisher logEventPublisher;
    private final ProjectManager projectManager;

    /**
     * 新增分类
     *
     * @param addCategoryDTO 新增分类参数
     */
    @Override
    @Transactional
    public void addCategory(AddCategoryDTO addCategoryDTO) {
        //分类ID规则
        //第一层为1位数字编码，其后每一层为 3位 数字编码从001 开始增长
        Integer isBaseCategory = addCategoryDTO.getIsBaseCategory();
        CategoryDO categoryDO = new CategoryDO();
        String categoryName = addCategoryDTO.getCategoryName().trim();
        if (IsOrNotEnum.IS.getValue().equals(isBaseCategory)) {
            //根分类
            //限制只有9个一级分类,查出最后一个
            CategoryDO existCategory = categoryManager.queryByNameAndParent(categoryName, "0");
            if (existCategory != null) {
                throw new BizException("分类名称已存在");
            }
            categoryDO.setCategoryName(categoryName);
            CategoryDO lastLevelOneCategory = categoryManager.queryLastLevelOneCategory();
            if (lastLevelOneCategory == null) {
                //此时没有一级分类,分类ID设为1
                categoryDO.setCategoryId("1");
            } else {
                int lastId = Integer.parseInt(lastLevelOneCategory.getCategoryId());
                if (lastId >= 9) {
                    throw new BizException("一级分类限制为9个,无法继续新增");
                } else {
                    lastId++;
                    categoryDO.setCategoryId(lastId + "");
                }
            }
            categoryDO.setParentCategoryId("0");
        } else {
            if (isBlank(addCategoryDTO.getParentId()) || isEmpty(addCategoryDTO.getParentId())) {
                throw new BizException("父级分类id不能为空");
            }
            String parentId = addCategoryDTO.getParentId().trim();
            CategoryDO existCategory = categoryManager.queryByNameAndParent(categoryName, parentId);
            if (existCategory != null) {
                throw new BizException("分类名称已存在");
            }
            categoryDO.setCategoryName(categoryName);
            //非一级分类
            CategoryDO parentCategory = categoryManager.queryById(parentId);
            if (parentCategory == null) {
                throw new BizException("父级分类不存在");
            }
            //根据ID判断层级  一级:1位, 二级:4位, 三级:7位 依次类推
            //查询当前新增分类层级的最后一位
            CategoryDO lastCurLevelCategory = categoryManager.queryLastCurLevelCategory(parentId);
            if (lastCurLevelCategory == null) {
                //当前父级分类下没有子分类
                categoryDO.setCategoryId(parentId + "001");
            } else {
                String lastCategoryId = lastCurLevelCategory.getCategoryId();
                int lastId = Integer.parseInt(lastCategoryId.substring(lastCategoryId.length() - 3));
                if (lastId >= 999) {
                    throw new BizException("分类限制为999,无法继续新增");
                } else {
                    String curLevelId = getCurLevelId(lastId);
                    categoryDO.setCategoryId(parentId + curLevelId);
                }
            }
            categoryDO.setParentCategoryId(parentId);
        }
        categoryDO.setIsDelete(DeleteStatusEnum.NOT_DELETE.getValue());
        categoryDO.setUpdateTime(LocalDateTime.now());

        categoryManager.addCategory(categoryDO);
        logEventPublisher.publish(Log.builder().append("添加分类,ID:" + categoryDO.getCategoryId() + ",名称:" + categoryName).isInsert());
    }

    /**
     * 修改分类
     *
     * @param modifyCategoryDTO 修改分类参数
     * @param categoryId        分类id
     */
    @Override
    @Transactional
    public void modifyCategory(ModifyCategoryDTO modifyCategoryDTO, String categoryId) {
        CategoryDO existCategory = categoryManager.queryById(categoryId);
        if (existCategory == null) {
            throw new BizException("分类不存在");
        }
        String categoryName = modifyCategoryDTO.getCategoryName().trim();
        CategoryDO sameNameCategory = categoryManager.queryByNameAndParent(categoryName,existCategory.getParentCategoryId());
        if (sameNameCategory != null && !sameNameCategory.getCategoryId().equals(categoryId)) {
            throw new BizException("分类名称已存在");
        }
        CategoryDO categoryDO = new CategoryDO();
        categoryDO.setCategoryId(categoryId);
        categoryDO.setCategoryName(categoryName);
        categoryDO.setUpdateTime(LocalDateTime.now());
        categoryManager.modifyCategory(categoryDO);
        logEventPublisher.publish(Log.builder()
                .append("修改分类,ID:" + categoryDO.getCategoryId() + ",原名称:" + existCategory.getCategoryName() + ",新名称:" + categoryName)
                .isUpdate());
    }

    /**
     * 删除(作废)分类
     *
     * @param categoryId 分类id
     */
    @Override
    @Transactional
    public void removeCategory(String categoryId) {
        CategoryDO existCategory = categoryManager.queryById(categoryId);
        if (existCategory == null) {
            throw new BizException("分类不存在");
        }
        //校验是否有子分类
        List<CategoryDO> childCateList = categoryManager.queryListByParentId(categoryId);
        if (isNotEmpty(childCateList)) {
            throw new BizException("当前分类下存在子分类,无法作废");
        }
        CategoryDO categoryDO = new CategoryDO();
        categoryDO.setCategoryId(categoryId);
        categoryDO.setIsDelete(DeleteStatusEnum.DELETED.getValue());
        categoryDO.setUpdateTime(LocalDateTime.now());
        categoryManager.modifyCategory(categoryDO);
        logEventPublisher.publish(Log.builder()
                .append("作废分类,ID:" + categoryDO.getCategoryId() + ",名称:" + existCategory.getCategoryName())
                .isDelete());
    }

    /**
     * 获取分类树列表
     *
     * @return List<CategoryTreeVo>
     */
    @Override
    public List<CategoryTreeVo> getCategoryTree() {
        //查询所有一级分类
        List<CategoryDO> levelOneCategoryList = categoryManager.queryListByParentId("0");
        List<CategoryTreeVo> treeList = new ArrayList<>();
        if (isNotEmpty(levelOneCategoryList)) {
            treeList = buildTree(levelOneCategoryList);
        }
        return treeList;
    }

    /**
     * 获取分类树快照
     *
     * @param taskId 任务id
     * @return List<Object>
     */
    @Override
    public List<Object> treeSnapshot(Long taskId) {
        ProjectTaskDO projectTaskDO = projectManager.queryTaskByTaskId(taskId);
        if (projectTaskDO == null) {
            throw new BizException("任务不存在");
        }
        ProjectCategoryDO projectCategoryDO = projectManager.queryProCateById(projectTaskDO.getProjectId());
        if (projectCategoryDO == null || isBlank(projectCategoryDO.getCategorySnapshot())) {
            throw new BizException("项目关联分类不存");
        }
        return JSONUtil.parseArray(projectCategoryDO.getCategorySnapshot());
    }

    /**
     * 构建分类树
     *
     * @param categoryDOList 分类集合
     * @return List<CategoryTreeVo>
     */
    private List<CategoryTreeVo> buildTree(List<CategoryDO> categoryDOList) {
        List<CategoryTreeVo> treeList = new ArrayList<>();
        categoryDOList.forEach(levelOneCate -> {
            CategoryTreeVo treeVo = new CategoryTreeVo();
            treeVo.setValue(levelOneCate.getCategoryId());
            treeVo.setLabel(levelOneCate.getCategoryName());
            List<CategoryDO> categoryDOS = categoryManager.queryListByParentId(treeVo.getValue());
            if (isNotEmpty(categoryDOS)) {
                List<CategoryTreeVo> categoryTreeVos = buildTree(categoryDOS);
                treeVo.setChildren(categoryTreeVos);
            }
            treeList.add(treeVo);
        });
        return treeList;
    }

    /**
     * 获取当前层级的id
     *
     * @param lastId 最后的id
     * @return curLevelId
     */
    private String getCurLevelId(int lastId) {
        String curLevelId;
        if (lastId < 9) {
            curLevelId = "00" + (lastId + 1);
        } else if (lastId < 99) {
            curLevelId = "0" + (lastId + 1);
        } else {
            curLevelId = lastId + 1 + "";
        }
        return curLevelId;
    }
}
