package cn.sino.mark.service.impl;

import cn.sino.common.base.PageResult;
import cn.sino.common.exception.BizException;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.bo.LabelBO;
import cn.sino.mark.common.model.dto.AddLabelDTO;
import cn.sino.mark.common.model.dto.EditLabelDTO;
import cn.sino.mark.common.model.dto.LabelPageDTO;
import cn.sino.mark.common.model.vo.LabelAllVO;
import cn.sino.mark.common.model.vo.LabelInfoVO;
import cn.sino.mark.common.model.vo.LabelPageVO;
import cn.sino.mark.dao.model.LabelDO;
import cn.sino.mark.manager.LabelManager;
import cn.sino.mark.service.LabelService;
import cn.sino.mark.service.convert.LabelMapStruct;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static com.github.pagehelper.page.PageMethod.startPage;

/**
 * 标签业务接口实现
 *
 * @author zhangamin
 * @date 2022/8/14
 */
@Service
@RequiredArgsConstructor
public class LabelServiceImpl implements LabelService {

    private final LabelManager labelManager;
    private final LabelMapStruct labelMapStruct;

    /**
     * 日志
     */
    private final SysLogEventPublisher logEventPublisher;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addLabel(AddLabelDTO reqDTO) {
        if (labelManager.getByName(reqDTO.getName()) != null) {
            throw new BizException("标签名称已经存在");
        }
        checkLabelItem(reqDTO.getLabelItemNames());
        LabelDO label = labelManager.add(reqDTO.getName(), reqDTO.getLabelItemNames());

        // 系统日志
        logEventPublisher.publish(Log.builder().append("添加标签,名称:" + reqDTO.getName() + ",ID:" + label).isInsert());
    }

    @Override
    public void editLabel(EditLabelDTO reqDTO) {
        LabelDO label = labelManager.checkById(reqDTO.getId());
        LabelDO checkLabel = labelManager.getByName(reqDTO.getName());
        //校验标签可选项是否重复
        if (checkLabel != null && !checkLabel.getId().equals(label.getId())) {
            throw new BizException("标签名称已经存在");
        }
        checkLabelItem(reqDTO.getLabelItemNames());
        label.setName(reqDTO.getName());
        labelManager.edit(label, reqDTO.getLabelItemNames());
        // 系统日志
        logEventPublisher.publish(Log.builder().append("编辑标签,名称:" + reqDTO.getName() + ",ID:" + label).isUpdate());
    }

    @Override
    public void delete(Long id) {
        labelManager.deleteById(id);
        // 系统日志
        logEventPublisher.publish(Log.builder().append("删除标签,ID:" + id).isDelete());
    }

    @Override
    public PageResult<LabelPageVO> getPage(LabelPageDTO reqDTO) {
        startPage(reqDTO.getPageNum(), Constant.DEFAULT_PAGE_SIZE);
        Page<LabelBO> labelBOS = labelManager.getPage(reqDTO);
        Page<LabelPageVO> page = new Page<>();
        labelBOS.forEach(labelBO -> page.add(labelMapStruct.convertToLabelPageVO(labelBO)));
        return PageResult.of(labelBOS.getTotal(), page);
    }

    @Override
    public LabelInfoVO selectInfo(Long id) {
        labelManager.checkById(id);
        LabelBO labelBO = labelManager.getLabelBoById(id);
        return labelMapStruct.convertToLabelInfoVO(labelBO);
    }

    @Override
    public List<LabelAllVO> selectAll() {
        List<LabelBO> labelBOS = labelManager.getAll();
        return labelBOS.stream().map(labelMapStruct::convertToLabelAllResDTO).collect(Collectors.toList());
    }

    /**
     * 校验标签可选项是否重复
     *
     * @param labelItemNames 标签可选项
     */
    private void checkLabelItem(List<String> labelItemNames) {
        //校验标签可选项名称重复
        Set<String> checkNames = new HashSet<>(labelItemNames);
        if (labelItemNames.size() != checkNames.size()) {
            throw new BizException("标签可选项存在重复的名称");
        }
    }
}




