package cn.sino.mark.service.impl;

import cn.sino.common.exception.BizException;
import cn.sino.common.util.SnowflakeIdFactory;
import cn.sino.mark.common.enums.IsOrNotEnum;
import cn.sino.mark.common.enums.SysUserStatusEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.dto.AddProjectGroupDTO;
import cn.sino.mark.common.model.dto.ModifyProjectGroupDTO;
import cn.sino.mark.common.model.vo.ProjectGroupTreeVO;
import cn.sino.mark.dao.model.ProjectGroupDO;
import cn.sino.mark.manager.ProjectManager;
import cn.sino.mark.service.ProjectGroupService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;

/**
 * @author shichanghao
 * @description 针对表【t_project_group(项目分组表)】的数据库操作Service实现
 * @className cn.sino.mark.service.impl.ProjectGroupServiceImpl
 * @date 2022-04-03 20:01:41
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ProjectGroupServiceImpl implements ProjectGroupService {

    private final ProjectManager groupManager;
    private final SnowflakeIdFactory snowflakeIdFactory;
    private final SysLogEventPublisher logEventPublisher;


    /**
     * 新增项目组
     *
     * @param addProjectGroupDTO 新增项目组信息
     */
    @Override
    @Transactional
    public void addGroup(AddProjectGroupDTO addProjectGroupDTO) {
        String groupName = addProjectGroupDTO.getGroupName().trim();
        //校验组名唯一
        ProjectGroupDO existGroup = groupManager.queryGroupByName(groupName);
        if (existGroup != null) {
            throw new BizException("项目组名称已存在");
        }
        ProjectGroupDO groupDO = new ProjectGroupDO();
        groupDO.setGroupId(snowflakeIdFactory.nextId());
        groupDO.setName(groupName);
        groupDO.setStatus(SysUserStatusEnum.NORMAL.getValue());
        if (IsOrNotEnum.IS.getValue().equals(addProjectGroupDTO.getIsBaseGroup())) {
            //一级项目组
            groupDO.setParentId(0L);
        } else {
            //二级项目组  目前只支持2级
            //校验父级项目组
            Long parentId = addProjectGroupDTO.getParentId();
            if (parentId == null) {
                throw new BizException("父级项目组ID不能为空");
            }
            ProjectGroupDO parentGroup = groupManager.queryGroupById(parentId);
            if (parentGroup == null) {
                throw new BizException("父级项目组不存在");
            }
            if (!parentGroup.getParentId().equals(0L)) {
                throw new BizException("项目组当前只能新增2级");
            }
            groupDO.setParentId(parentId);
        }
        groupManager.addGroup(groupDO);
        logEventPublisher.publish(Log.builder().append("新增项目组,ID: " + groupDO.getGroupId() + ",名称: " + groupName).isInsert());

    }

    /**
     * 修改项目组
     *
     * @param modifyDTO 修改项目组信息
     * @param groupId   项目组id
     */
    @Override
    public void modifyGroup(ModifyProjectGroupDTO modifyDTO, Long groupId) {
        ProjectGroupDO existGroup = groupManager.queryGroupById(groupId);
        if (existGroup == null) {
            throw new BizException("项目组不存在");
        }
        String groupName = modifyDTO.getGroupName().trim();
        ProjectGroupDO sameNameGroup = groupManager.queryGroupByName(groupName);
        if (sameNameGroup != null && !sameNameGroup.getGroupId().equals(groupId)) {
            throw new BizException("项目组名称已存在");
        }
        ProjectGroupDO groupDO = new ProjectGroupDO();
        groupDO.setId(existGroup.getId());
        groupDO.setGroupId(groupId);
        groupDO.setName(groupName);
        if (groupManager.modifyGroup(groupDO) < 1) {
            throw new BizException("修改项目组失败");
        }
        logEventPublisher.publish(Log.builder().append("修改项目组,ID: " + groupDO.getGroupId() + ",旧名称: " + existGroup.getName() + ",新名称: " + groupName).isUpdate());

    }

    /**
     * 作废项目组
     *
     * @param groupId 项目组id
     */
    @Override
    @Transactional
    public void removeGroup(Long groupId) {
        ProjectGroupDO existGroup = groupManager.queryGroupById(groupId);
        if (existGroup == null) {
            throw new BizException("项目组不存在");
        }
        List<ProjectGroupDO> projectGroupDOS = groupManager.queryGroupListByParentId(groupId);
        if (isNotEmpty(projectGroupDOS)){
            throw new BizException("存在子项目组,无法删除 ");
        }
        ProjectGroupDO groupDO = new ProjectGroupDO();
        groupDO.setId(existGroup.getId());
        groupDO.setGroupId(groupId);
        groupDO.setStatus(SysUserStatusEnum.FROZEN.getValue());
        if (groupManager.modifyGroup(groupDO) < 1) {
            throw new BizException("删除项目组失败");
        }
        logEventPublisher.publish(Log.builder().append("删除项目组,ID: " + groupDO.getGroupId() + ",名称: " + existGroup.getName()).isUpdate());

    }

    /**
     * 获取分组树列表
     *
     * @return List<ProjectGroupTreeVO>
     */
    @Override
    public List<ProjectGroupTreeVO> getTree() {
        //查询所有一级分类
        List<ProjectGroupDO> levelOneCategoryList = groupManager.queryGroupListByParentId(0L);
        List<ProjectGroupTreeVO> treeList = new ArrayList<>();
        if (isNotEmpty(levelOneCategoryList)) {
            treeList = buildTree(levelOneCategoryList);
        }
        return treeList;
    }

    /**
     * 构建分类树
     *
     * @param groupDOS 分类集合
     * @return List<CategoryTreeVo>
     */
    private List<ProjectGroupTreeVO> buildTree(List<ProjectGroupDO> groupDOS) {
        List<ProjectGroupTreeVO> treeList = new ArrayList<>();
        groupDOS.forEach(group -> {
            ProjectGroupTreeVO treeVo = new ProjectGroupTreeVO();
            treeVo.setValue(group.getGroupId());
            treeVo.setLabel(group.getName());
            List<ProjectGroupDO> groupDOList = groupManager.queryGroupListByParentId(treeVo.getValue());
            if (isNotEmpty(groupDOList)) {
                List<ProjectGroupTreeVO> groupTreeVOS = buildTree(groupDOList);
                treeVo.setChildren(groupTreeVOS);
            }
            treeList.add(treeVo);
        });
        return treeList;
    }
}




