package cn.sino.mark.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.json.JSONUtil;
import cn.sino.common.base.PageResult;
import cn.sino.common.exception.BizException;
import cn.sino.common.util.SnowflakeIdFactory;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.enums.*;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.bo.ExportDataBO;
import cn.sino.mark.common.model.bo.ProjectBO;
import cn.sino.mark.common.model.bo.ProjectTaskBO;
import cn.sino.mark.common.model.bo.ProjectTaskDetailBO;
import cn.sino.mark.common.model.dto.*;
import cn.sino.mark.common.model.vo.*;
import cn.sino.mark.common.utils.ListsUtil;
import cn.sino.mark.common.utils.LoginUserUtil;
import cn.sino.mark.dao.model.*;
import cn.sino.mark.manager.CategoryManager;
import cn.sino.mark.manager.ProjectManager;
import cn.sino.mark.manager.ResourceManager;
import cn.sino.mark.manager.SysUserManager;
import cn.sino.mark.service.ProjectService;
import cn.sino.mark.service.convert.ProjectMapStruct;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.temporal.ChronoField;
import java.util.*;
import java.util.stream.Collectors;

import static cn.hutool.core.collection.CollUtil.isEmpty;
import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static cn.hutool.core.text.CharSequenceUtil.isBlank;
import static cn.hutool.core.text.CharSequenceUtil.isNotBlank;
import static com.github.pagehelper.page.PageMethod.startPage;

/**
 * @author shichanghao
 * @description 针对表【t_project】的数据库操作Service实现
 * @date 2022-04-04 19:41:27
 * @className cn.sino.mark.service.impl.ProjectServiceImpl
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ProjectServiceImpl implements ProjectService {

    private final ProjectManager projectManager;
    private final SnowflakeIdFactory snowflakeIdFactory;
    private final ProjectMapStruct projectMapStruct;
    private final SysLogEventPublisher logEventPublisher;
    private final CategoryManager categoryManager;
    private final ResourceManager resourceManager;
    private final SysUserManager userManager;

    private static final String PROJECT_NULL = "项目不存在";
    private static final String TASK_NULL = "任务不存在";


    /**
     * 新增项目基本信息
     *
     * @param addProjectDTO 新增项目信息
     * @return Long
     */
    @Override
    @Transactional
    public Long addProject(AddProjectDTO addProjectDTO) {
        //校验项目组
        ProjectGroupDO groupDO = projectManager.queryGroupById(addProjectDTO.getGroupId());
        if (groupDO == null) {
            throw new BizException("项目组不存在");
        }
        //校验项目名称唯一
        String name = addProjectDTO.getProjectName().trim();
        ProjectDO sameNameProject = projectManager.queryByName(name);
        String desc = addProjectDTO.getDescription();
        ProjectDO projectDO = projectMapStruct.convertDTOToProjectDO(addProjectDTO);
        projectDO.setName(name);
        if (compareTime(addProjectDTO.getBeginTime(), addProjectDTO.getEndTime())) {
            throw new BizException("有效期开始时间不能在截止时间后面");
        }
        if (addProjectDTO.getIsEdit().equals(IsOrNotEnum.NOT.getValue())) {
            if (sameNameProject != null) {
                throw new BizException("项目名称已存在");
            }
            projectDO.setProjectId(snowflakeIdFactory.nextId());
            if (isNotBlank(desc) && CharSequenceUtil.isNotEmpty(desc)) {
                desc = desc.trim();
                if (desc.length() > 100) {
                    throw new BizException("项目描述长度不能大于100");
                } else {
                    projectDO.setDescription(desc);
                }
            }
            projectDO.setStatus(ProjectStatusEnum.DRAFT.getValue());
            projectManager.addProject(projectDO);
            logEventPublisher.publish(Log.builder().append("新增项目,ID: " + projectDO.getProjectId() + ",名称: " + name).isInsert());
            return projectDO.getProjectId();
        } else {
            Long projectId = addProjectDTO.getProjectId();
            if (projectId == null) {
                throw new BizException("项目id不能为空");
            }
            ProjectDO existProject = projectManager.queryById(projectId);
            if (existProject == null) {
                throw new BizException(PROJECT_NULL);
            }
            if (sameNameProject != null && !sameNameProject.getProjectId().equals(projectId)) {
                throw new BizException("项目名称已存在");
            }
            projectDO.setId(existProject.getId());
            projectDO.setProjectId(projectId);
            if (isNotBlank(desc) && CharSequenceUtil.isNotEmpty(desc)) {
                if (desc.length() > 100) {
                    throw new BizException("项目描述长度不能大于500");
                } else {
                    projectDO.setDescription(desc);
                }
            } else {
                projectDO.setDescription("");
            }
            if (projectManager.updateById(projectDO) < 1) {
                throw new BizException("更新项目失败");
            }
            logEventPublisher.publish(Log.builder().append("编辑项目,ID: " + projectDO.getProjectId() + ",名称: " + name).isUpdate());
            return projectId;
        }


    }

    /**
     * 新增项目分类配置
     *
     * @param addProjectCategoryDTO 新增项目分类信息
     */
    @Override
    @Transactional
    public void addCategory(AddProjectCategoryDTO addProjectCategoryDTO) {
        Long projectId = addProjectCategoryDTO.getProjectId();
        String categoryId = addProjectCategoryDTO.getCategoryId();
        //校验项目
        ProjectDO projectDO = projectManager.queryById(projectId);
        if (projectDO == null) {
            throw new BizException(PROJECT_NULL);
        }
        //校验分类
        CategoryDO categoryDO = categoryManager.queryById(categoryId.trim());
        if (categoryDO == null) {
            throw new BizException("分类不存在");
        }
        //先根据projectId删除数据
        projectManager.deleteProCategory(projectId);
        ProjectCategoryDO projectCategoryDO = new ProjectCategoryDO();
        projectCategoryDO.setCategoryId(categoryId.trim());
        projectCategoryDO.setProjectId(projectId);

        List<CategoryDO> levelOneCategoryList = Collections.singletonList(categoryDO);
        List<CategoryTreeVo> treeList = new ArrayList<>();
        if (isNotEmpty(levelOneCategoryList)) {
            treeList = buildTree(levelOneCategoryList);
        }
        projectCategoryDO.setCategorySnapshot(JSONUtil.toJsonStr(treeList));
        projectManager.addProjectCategory(projectCategoryDO);
        logEventPublisher.publish(Log.builder().append("新增项目-分类关联,项目ID: " + projectId + ",分类ID: " + categoryId).isInsert());
    }

    /**
     * 新增项目资源配置
     *
     * @param addProjectResourceDTO 新增项目-资源信息
     */
    @Override
    @Transactional
    public void addResource(AddProjectResourceDTO addProjectResourceDTO) {
        Long projectId = addProjectResourceDTO.getProjectId();
        //校验项目
        ProjectDO projectDO = projectManager.queryById(projectId);
        if (projectDO == null) {
            throw new BizException(PROJECT_NULL);
        }
        //校验不能重复使用
        List<Long> resourceIds = addProjectResourceDTO.getResourceIds();
        List<ProjectResourceDO> projectResourceDOS = projectManager.queryProResListByProId(projectId);
        List<Long> existResourceIds = projectResourceDOS.stream().map(ProjectResourceDO::getResourceId).collect(Collectors.toList());
        List<ResourceDO> existResourceList = resourceManager.queryListByIds(resourceIds);
        if (isEmpty(existResourceList)) {
            throw new BizException("资源列表不存在");
        }
        if (existResourceList.size() != resourceIds.size()) {
            throw new BizException("部分资源已失效");
        }

        List<String> nameList = existResourceList.stream()
                .filter(item -> item.getAssignStatus().equals(IsOrNotEnum.IS.getValue()) && !existResourceIds.contains(item.getResourceId()))
                .map(ResourceDO::getName).collect(Collectors.toList());
        if (isNotEmpty(nameList)) {
            throw new BizException("以下资源已经分配,请重新选择.资源名称: " + nameList);
        }
        //清空原有的数据
        projectManager.deleteProResByProId(projectId);
        List<ProjectResourceDO> projectResourceDOList = new ArrayList<>();
        List<ResourceDO> resourceDOS = new ArrayList<>();
        resourceIds.forEach(resourceId -> {
            ProjectResourceDO projectResourceDO = new ProjectResourceDO();
            projectResourceDO.setProjectId(projectId);
            projectResourceDO.setResourceId(resourceId);
            projectResourceDOList.add(projectResourceDO);
            ResourceDO resourceDO = new ResourceDO();
            resourceDO.setResourceId(resourceId);
            resourceDO.setAssignStatus(ResourceAssignStatusEnum.ASSIGNED.getValue());
            resourceDOS.add(resourceDO);
        });
        if (isNotEmpty(projectResourceDOList)) {
            ListsUtil.splitRun(projectResourceDOList, 5000, projectManager::insertBatchRes);
            ListsUtil.splitRun(resourceDOS, 5000, resourceManager::batchUpdate);
            logEventPublisher.publish(Log.builder().append("新增项目-资源关联,项目ID: " + projectId + ",资源数量: " + projectResourceDOList.size()).isInsert());
        }
    }

    /**
     * 新增项目人员配置
     *
     * @param addProjectUserDTO 新增项目-用户信息
     */
    @Override
    @Transactional
    public List<ProjectTaskAssignVO> addUsers(AddProjectUserDTO addProjectUserDTO) {
        Long projectId = addProjectUserDTO.getProjectId();
        //校验项目
        ProjectDO projectDO = projectManager.queryById(projectId);
        if (projectDO == null) {
            throw new BizException(PROJECT_NULL);
        }
        //清空原有的数据
        projectManager.deleteProUsersByProId(projectId);
        List<Long> userIds = addProjectUserDTO.getUserIds();
        List<SysUserDO> userList = userManager.queryListByUserIds(userIds);
        if (isEmpty(userList)) {
            throw new BizException("用户列表不存在");
        }
        if (userList.stream().filter(item -> item.getStatus().equals(SysUserStatusEnum.NORMAL.getValue())).count() != userIds.size()) {
            throw new BizException("部分用户不存在");
        }

        List<ProjectUserDO> projectUserDOList = new ArrayList<>();
        userIds.forEach(userId -> {
            ProjectUserDO projectUserDO = new ProjectUserDO();
            projectUserDO.setProjectId(projectId);
            projectUserDO.setUserId(userId);
            projectUserDOList.add(projectUserDO);
        });
        if (isNotEmpty(projectUserDOList)) {
            ListsUtil.splitRun(projectUserDOList, 5000, projectManager::insertBatchUsers);
            logEventPublisher.publish(Log.builder().append("新增项目-用户关联,项目ID: " + projectId + ",用户数量: " + projectUserDOList.size()).isInsert());
        }

        //根据资源类型数量和用户,分配任务  任务总数=资源类型数量 X 用户数量
        List<ProjectResourceDO> proResDOS = projectManager.queryProResListByProId(projectId);
        List<Long> resourceIdList = proResDOS.stream().map(ProjectResourceDO::getResourceId).distinct().collect(Collectors.toList());
        return buildTaskList(projectDO, userIds, resourceIdList);
    }

    /**
     * 项目资源分配完成
     *
     * @param completeProjectDTO 参数
     * @param projectId          项目id
     */
    @Override
    @Transactional
    public void assignComplete(CompleteProjectDTO completeProjectDTO, Long projectId) {
        //校验项目
        ProjectDO existProject = projectManager.queryById(projectId);
        if (existProject == null) {
            throw new BizException(PROJECT_NULL);
        }
        //清空原有的数据
        projectManager.deleteProTasksByProId(projectId);
        projectManager.deleteProTaskDetailsByProId(projectId);
        List<ProjectResourceDO> proResList = projectManager.queryProResListByProId(projectId);
        if (isEmpty(proResList)) {
            throw new BizException("项目没有资源,请检查");
        }
        List<ProjectUserDO> proUserList = projectManager.queryProUseListByProId(projectId);
        if (isEmpty(proUserList)) {
            throw new BizException("项目没有用户,请检查");
        }
        List<Long> resourceIds = proResList.stream().map(ProjectResourceDO::getResourceId).collect(Collectors.toList());
        List<Long> userIds = proUserList.stream().map(ProjectUserDO::getUserId).collect(Collectors.toList());
        List<ResourceDO> resourceList = resourceManager.queryListByIds(resourceIds);
        List<ResourceDO> docList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.DOC.getValue())).collect(Collectors.toList());
        List<ResourceDO> picList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.PICTURE.getValue())).collect(Collectors.toList());
        List<ResourceDO> movList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.MOVIE.getValue())).collect(Collectors.toList());

        List<SysUserDO> userList = userManager.queryListByUserIds(userIds);
        List<Integer> typeList = resourceList.stream().map(ResourceDO::getType).distinct().collect(Collectors.toList());
        int userNum = userList.size();
        int taskNum = userNum * (typeList.size());
        List<Long> taskIds = completeProjectDTO.getTaskIds();
        if (taskNum != taskIds.size()) {
            throw new BizException("提交的数据与实际查询数据不符,请返回上一步重新提交");
        }
        List<String> taskNameList = getTaskNameList(taskNum, existProject.getName());
        int index = 0;
        List<ProjectTaskDO> taskList = new ArrayList<>();
        int docSum = docList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        int picSum = picList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        int movSum = movList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        for (SysUserDO user : userList) {
            for (Integer type : typeList) {
                ProjectTaskDO task = new ProjectTaskDO();
                task.setTaskId(taskIds.get(index));
                task.setName(taskNameList.get(index));
                task.setProjectId(projectId);
                task.setAccount(user.getAccount());
                task.setUserId(user.getUserId());
                task.setType(type);
                task.setTaskCount(0);
                task.setBeginTime(existProject.getBeginTime());
                task.setEndTime(existProject.getEndTime());
                task.setIsComplete(IsOrNotEnum.NOT.getValue());
                if (type.equals(ResourceTypeEnum.DOC.getValue())) {
                    task.setTaskCount(docSum / userNum);
                } else if (type.equals(ResourceTypeEnum.PICTURE.getValue())) {
                    task.setTaskCount(picSum / userNum);
                } else {
                    task.setTaskCount(movSum / userNum);
                }
                index++;
                task.setMarkNum(0);
                task.setProblemNum(0);
                task.setIgnoreNum(0);
                task.setSuccessNum(0);
                task.setFailedNum(0);
                taskList.add(task);
            }
        }
        int docRestNum = docSum - (taskList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.DOC.getValue())).mapToInt(ProjectTaskDO::getTaskCount).sum());
        int picRestNum = picSum - (taskList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.PICTURE.getValue())).mapToInt(ProjectTaskDO::getTaskCount).sum());
        int movRestNum = movSum - (taskList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.MOVIE.getValue())).mapToInt(ProjectTaskDO::getTaskCount).sum());
        //把除不尽的余数,继续分配下去
        for (int i = 0; i < docRestNum; i++) {
            SysUserDO user = userList.get(i);
            solveRestNum(taskList, user, ResourceTypeEnum.DOC.getValue());
        }
        for (int i = 0; i < picRestNum; i++) {
            SysUserDO user = userList.get(i);
            solveRestNum(taskList, user, ResourceTypeEnum.PICTURE.getValue());

        }
        for (int i = 0; i < movRestNum; i++) {
            SysUserDO user = userList.get(i);
            solveRestNum(taskList, user, ResourceTypeEnum.MOVIE.getValue());
        }

        List<ResourceDataDO> docDataList = resourceManager.queryDataListByResIds(docList.stream().map(ResourceDO::getResourceId).collect(Collectors.toList()));
        List<ResourceDataDO> picDataList = resourceManager.queryDataListByResIds(picList.stream().map(ResourceDO::getResourceId).collect(Collectors.toList()));
        List<ResourceDataDO> movDataList = resourceManager.queryDataListByResIds(movList.stream().map(ResourceDO::getResourceId).collect(Collectors.toList()));
        List<ProjectTaskDetailDO> taskDetailDOList = new ArrayList<>();
        LocalDateTime now = LocalDateTime.now();
        if (isNotEmpty(docDataList)) {
            taskDetailDOList = buildTaskDetails(projectId, taskList, docDataList, taskDetailDOList, now, ResourceTypeEnum.DOC.getValue());
        }
        if (isNotEmpty(picDataList)) {
            taskDetailDOList = buildTaskDetails(projectId, taskList, picDataList, taskDetailDOList, now, ResourceTypeEnum.PICTURE.getValue());
        }
        if (isNotEmpty(movDataList)) {
            taskDetailDOList = buildTaskDetails(projectId, taskList, movDataList, taskDetailDOList, now, ResourceTypeEnum.MOVIE.getValue());
        }
        //新增项目任务
        if (isNotEmpty(taskList)) {
            ListsUtil.splitRun(taskList, 5000, projectManager::insertBatchProTask);
        }
        //新增任务明细
        if (isNotEmpty(taskDetailDOList)) {
            ListsUtil.splitRun(taskDetailDOList, 5000, projectManager::insertBatchTaskDetail);
        }
        ProjectDO projectDO = new ProjectDO();
        projectDO.setId(existProject.getId());
        projectDO.setStatus(ProjectStatusEnum.OFFICIAL.getValue());
        projectManager.updateById(projectDO);
        logEventPublisher.publish(Log.builder().append("完成资源任务分配,项目ID: " + projectId + ",名称: " + existProject.getName()).isUpdate());
    }

    /**
     * 分页查询项目
     *
     * @param queryProjectDTO 查询条件
     * @return PageResult<ProjectPageItemVO>
     */
    @Override
    public PageResult<ProjectPageItemVO> getProjectPage(QueryProjectDTO queryProjectDTO) {
        startPage(queryProjectDTO.getPageNum(), queryProjectDTO.getPageSize());
        long start = System.currentTimeMillis();
        Page<ProjectBO> projectBOS = projectManager.queryProjectPage(queryProjectDTO);
        long queryEnd = System.currentTimeMillis();
        log.info("查询耗时: {}ms", (queryEnd - start));
        Page<ProjectPageItemVO> page = new Page<>();
        LocalDateTime now = LocalDateTime.now();
        if (isNotEmpty(projectBOS)) {
            projectBOS.forEach(item -> {
                ProjectPageItemVO result = new ProjectPageItemVO();
                result.setProjectId(item.getProjectId());
                result.setName(item.getName());
                if (isNotBlank(item.getFirstGroup()) && CharSequenceUtil.isNotEmpty(item.getFirstGroup())) {
                    result.setGroupName(item.getFirstGroup() + "/" + item.getSecondGroup());
                } else {
                    result.setGroupName(item.getSecondGroup());
                }
                if (queryProjectDTO.getStatus() != null && !queryProjectDTO.getStatus().equals(100)) {
                    result.setStatusText(ProjectStatusEnum.getName(queryProjectDTO.getStatus()));
                    result.setStatus(queryProjectDTO.getStatus());
                } else {
                    if (now.isBefore(item.getPushTime())) {//待分配：在推送时间前
                        result.setStatusText(ProjectStatusEnum.WAIT_ASSIGN.getName());
                        result.setStatus(ProjectStatusEnum.WAIT_ASSIGN.getValue());
                    } else if (now.isAfter(item.getPushTime())) {//已分配: 在推送时间后
                        result.setStatusText(ProjectStatusEnum.ASSIGNED.getName());
                        result.setStatus(ProjectStatusEnum.ASSIGNED.getValue());
                        if (now.isAfter(item.getBeginTime())) {
                            result.setStatusText(ProjectStatusEnum.STARTED.getName());
                            result.setStatus(ProjectStatusEnum.STARTED.getValue());
                        }
                        if (item.getTotalTaskNum() > 0 && item.getCompleteTaskNum() > 0 && item.getTotalTaskNum().equals(item.getCompleteTaskNum())) {
                            result.setStatusText(ProjectStatusEnum.FINISHED.getName());
                            result.setStatus(ProjectStatusEnum.FINISHED.getValue());
                        }
                        if (item.getExpiredNum() > 0) {
                            result.setStatusText(ProjectStatusEnum.PART_EXPIRED.getName());
                            result.setStatus(ProjectStatusEnum.PART_EXPIRED.getValue());
                        }
                        if (item.getSampledNum() > 0) {
                            result.setStatusText(ProjectStatusEnum.ACCESS.getName());
                            result.setStatus(ProjectStatusEnum.ACCESS.getValue());
                        }
                    }
                }
                result.setAssignTimeStr(DateUtil.format(item.getPushTime(), Constant.DATETIME_FORMAT));
                result.setCompleteTimeStr(item.getCompleteTime() != null ? DateUtil.format(item.getCompleteTime(), Constant.DATETIME_FORMAT) : "");
                result.setValidTimeStr(DateUtil.format(item.getBeginTime(), Constant.DATETIME_FORMAT)
                        + "~" + DateUtil.format(item.getEndTime(), Constant.DATETIME_FORMAT));
                result.setProblemNum(item.getProblemNum());
                page.add(result);
            });
        }
        return PageResult.of(projectBOS.getTotal(), page);
    }

    /**
     * 取消项目
     *
     * @param projectId 项目id
     */
    @Override
    @Transactional
    public void deleteProject(Long projectId) {
        //校验项目
        ProjectDO existProject = projectManager.queryById(projectId);
        if (existProject == null) {
            throw new BizException(PROJECT_NULL);
        }
        projectManager.deleteById(existProject.getId());

        //修改资源分配状态
        List<Long> resourceIds = projectManager.queryProResListByProId(projectId).stream().map(ProjectResourceDO::getResourceId).collect(Collectors.toList());
        if (isNotEmpty(resourceIds)) {
            List<ResourceDO> collect = resourceIds.stream().map(resourceId -> {
                ResourceDO resourceDO = new ResourceDO();
                resourceDO.setResourceId(resourceId);
                resourceDO.setAssignStatus(ResourceAssignStatusEnum.NOT_ASSIGNED.getValue());
                return resourceDO;
            }).collect(Collectors.toList());
            resourceManager.batchUpdate(collect);
        }

        //清空原有的数据
        projectManager.deleteProCategory(projectId);
        projectManager.deleteProResByProId(projectId);
        projectManager.deleteProUsersByProId(projectId);
        projectManager.deleteProTasksByProId(projectId);
        projectManager.deleteProTaskDetailsByProId(projectId);
        logEventPublisher.publish(Log.builder().append("删除项目,项目ID: " + projectId + ",名称: " + existProject.getName()).isDelete());

    }

    /**
     * 查看项目的任务列表
     *
     * @param projectId 项目id
     * @param pageNum   页码
     * @return PageResult<ProjectTaskPageItemVO>
     */
    @Override
    public PageResult<ProjectTaskPageItemVO> getTaskPage(Long projectId, Integer pageNum) {
        startPage(pageNum, Constant.DEFAULT_PAGE_SIZE);
        Page<ProjectTaskBO> projectTaskBOS = projectManager.queryTaskPageByProject(projectId);
        Page<ProjectTaskPageItemVO> page = new Page<>();
        LocalDateTime now = LocalDateTime.now();
        if (isNotEmpty(projectTaskBOS)) {
            projectTaskBOS.forEach(item -> {
                ProjectTaskPageItemVO pageItemVO = projectMapStruct.convertTOTaskPageItemVO(item);
                pageItemVO.setTypeName(ResourceTypeEnum.getName(item.getType()));
                if (item.getIsComplete().equals(IsOrNotEnum.IS.getValue())) {
                    pageItemVO.setStatusName(TaskRunningEnums.COMPLETED.getName());
                    pageItemVO.setStatus(TaskRunningEnums.COMPLETED.getValue());
                } else {
                    if (now.isAfter(item.getEndTime())) {
                        pageItemVO.setStatusName(TaskRunningEnums.EXPIRED.getName());
                        pageItemVO.setStatus(TaskRunningEnums.EXPIRED.getValue());
                    } else if (now.isBefore(item.getPushTime())) {
                        pageItemVO.setStatusName(TaskRunningEnums.WAIT.getName());
                        pageItemVO.setStatus(TaskRunningEnums.WAIT.getValue());
                    } else {
                        pageItemVO.setStatusName(TaskRunningEnums.RUNNING.getName());
                        pageItemVO.setStatus(TaskRunningEnums.RUNNING.getValue());
                    }
                }
                pageItemVO.setMarkedNum(item.getMarkNum() + item.getIgnoreNum());
                pageItemVO.setTotalNum(item.getTaskCount());
                pageItemVO.setBeginTimeStr(DateUtil.format(item.getBeginTime(), Constant.DATETIME_FORMAT));
                pageItemVO.setEndTimeStr(DateUtil.format(item.getEndTime(), Constant.DATETIME_FORMAT));
                pageItemVO.setCompleteTimeStr(item.getCompleteTime() != null ?
                        DateUtil.format(item.getCompleteTime(), Constant.DATETIME_FORMAT) : "");
                pageItemVO.setValidTimeStr(DateUtil.format(item.getBeginTime(), Constant.DATETIME_FORMAT)
                        + " ~ " + DateUtil.format(item.getEndTime(), Constant.DATETIME_FORMAT));
                int totalSampleNum = item.getSuccessNum() + item.getFailedNum();
                pageItemVO.setTotalSampleNum(totalSampleNum);
                if (totalSampleNum > 0) {
                    String successRate = BigDecimal.valueOf(item.getSuccessNum())
                            .divide(BigDecimal.valueOf(totalSampleNum), 2, RoundingMode.HALF_UP)
                            .multiply(BigDecimal.valueOf(100)) + "%";
                    pageItemVO.setSuccessRate(successRate);
                }
                page.add(pageItemVO);
            });
        }
        return PageResult.of(projectTaskBOS.getTotal(), page);
    }

    /**
     * 重新分配任务打标人
     *
     * @param reAssignProjectTaskDTO 参数
     */
    @Override
    @Transactional
    public void reAssignTask(ReAssignProjectTaskDTO reAssignProjectTaskDTO) {
        ProjectDO existProject = projectManager.queryById(reAssignProjectTaskDTO.getProjectId());
        if (existProject == null) {
            throw new BizException("项目不存");
        }
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(reAssignProjectTaskDTO.getTaskId());
        if (existTask == null) {
            throw new BizException(TASK_NULL);
        }
        SysUserDO oldUser = userManager.queryByUserId(existTask.getUserId());
        SysUserDO sysUserDO = userManager.queryByUserId(reAssignProjectTaskDTO.getMarkUserId());
        if (sysUserDO == null) {
            throw new BizException("用户不存在");
        }
        if (compareTime(reAssignProjectTaskDTO.getBeginTime(), reAssignProjectTaskDTO.getEndTime())) {
            throw new BizException("有效期开始时间不能在截止时间后面");
        }
        ProjectTaskDO taskDO = new ProjectTaskDO();
        taskDO.setId(existTask.getId());
        taskDO.setTaskId(existTask.getTaskId());
        taskDO.setUserId(reAssignProjectTaskDTO.getMarkUserId());
        taskDO.setAccount(sysUserDO.getAccount());
        taskDO.setBeginTime(reAssignProjectTaskDTO.getBeginTime());
        taskDO.setEndTime(reAssignProjectTaskDTO.getEndTime());
        if (projectManager.updateTask(taskDO) < 1) {
            throw new BizException("重新指派失败");
        }
        logEventPublisher.publish(Log.builder()
                .append("重新指派打标人,任务名称: " + existTask.getName() + " ,原标注员: " + oldUser.getUserName() + " ,新标注员: " + sysUserDO.getUserName())
                .isUpdate());

    }

    /**
     * 查询任务明细
     *
     * @param taskId 任务id
     * @return List<ProjectTaskDetailVO>
     */
    @Override
    public List<ProjectTaskDetailVO> getTaskDetails(Long taskId) {
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(taskId);
        if (existTask == null) {
            throw new BizException(TASK_NULL);
        }
        List<ProjectTaskDetailBO> taskDetailBOS = projectManager.queryDetailBOsByTaskId(taskId);
        List<ProjectTaskDetailVO> list = new ArrayList<>();
        if (isNotEmpty(taskDetailBOS)) {
            for (ProjectTaskDetailBO item : taskDetailBOS) {
                ProjectTaskDetailVO detailVO = projectMapStruct.convertTOTaskDetailVO(item);
                list.add(detailVO);
            }
        }
        return list;
    }

    /**
     * 打标
     *
     * @param markProjectTaskDTO 打标参数
     */
    @Override
    @Transactional
    public void mark(MarkProjectTaskDTO markProjectTaskDTO) {
        ProjectTaskDetailDO existDetail = projectManager.queryTaskDetailById(markProjectTaskDTO.getTaskDetailId());
        if (existDetail == null) {
            throw new BizException("任务明细不存在");
        }
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(existDetail.getTaskId());
        ProjectTaskDetailDO detailDO = new ProjectTaskDetailDO();
        detailDO.setId(existDetail.getId());
        detailDO.setTaskId(existDetail.getTaskId());
        detailDO.setType(markProjectTaskDTO.getType());
        detailDO.setMarkUserId(LoginUserUtil.getLoginUserId());
        detailDO.setIsMark(IsOrNotEnum.IS.getValue());
        detailDO.setMarkTime(LocalDateTime.now());
        if (markProjectTaskDTO.getType().equals(TaskMarkEnums.MARK.getValue()) && isBlank(markProjectTaskDTO.getCategoryId())) {
            throw new BizException("如果打标,请选择分类");
        }
        if (markProjectTaskDTO.isHasProblem()) {
            if (!markProjectTaskDTO.getType().equals(TaskMarkEnums.MARK.getValue())) {
                throw new BizException("只有打标的任务才能存疑");
            }
            detailDO.setHasProblem(IsOrNotEnum.IS.getValue());
        } else {
            detailDO.setHasProblem(IsOrNotEnum.NOT.getValue());
        }
        if (isNotBlank(markProjectTaskDTO.getCategoryId()) && CharSequenceUtil.isNotEmpty(markProjectTaskDTO.getCategoryId())) {
            List<CategoryDO> categoryDOS = categoryManager.queryListByParentId(markProjectTaskDTO.getCategoryId());
            if (isNotEmpty(categoryDOS)) {
                throw new BizException("只能选择最后一级分类");
            }
            detailDO.setCategoryId(markProjectTaskDTO.getCategoryId());
        } else {
            detailDO.setCategoryId("");
        }
        if (isNotEmpty(markProjectTaskDTO.getCategoryIds())) {
            detailDO.setCategoryIds(markProjectTaskDTO.getCategoryIds().toString());
        }
        if (projectManager.updateTaskDetail(detailDO) < 1) {
            throw new BizException("任务明细打标失败");
        }

        //查询任务下明细是否都已标注  全部标注  任务即为完成
        boolean completeFlag = projectManager.checkTaskFinish(existTask.getTaskId());
        ProjectTaskDO taskDO = new ProjectTaskDO();
        taskDO.setId(existTask.getId());
        taskDO.setTaskId(existTask.getTaskId());
        if (completeFlag) {
            taskDO.setIsComplete(IsOrNotEnum.IS.getValue());
            taskDO.setCompleteTime(LocalDateTime.now());
        }
        //查询已标注数量,存疑数量,忽略数量
        int markNum = projectManager.queryMarkNumByTask(existTask.getTaskId(), TaskMarkEnums.MARK.getValue(), null);
        int problemNum = projectManager.queryMarkNumByTask(existTask.getTaskId(), TaskMarkEnums.MARK.getValue(), IsOrNotEnum.IS.getValue());
        int ignoreNum = projectManager.queryMarkNumByTask(existTask.getTaskId(), TaskMarkEnums.IGNORE.getValue(), null);
        taskDO.setMarkNum(markNum);
        taskDO.setProblemNum(problemNum);
        taskDO.setIgnoreNum(ignoreNum);
        if (projectManager.updateTask(taskDO) < 0) {
            throw new BizException("修改任务失败,任务名称: " + existTask.getName());
        }
        logEventPublisher.publish(Log.builder()
                .append("任务明细已打标,明细id: " + detailDO.getDetailId() + " ,标注员: " + LoginUserUtil.getLoginUser().getUserName())
                .isUpdate());
    }

    /**
     * 查询项目基本信息
     *
     * @param projectId 项目id
     * @return ProjectBaseInfoVO
     */
    @Override
    public ProjectBaseInfoVO getProjectBaseInfo(Long projectId) {
        ProjectDO projectDO = projectManager.queryById(projectId);
        ProjectBaseInfoVO baseInfoVO = new ProjectBaseInfoVO();
        baseInfoVO.setProjectId(projectId);
        baseInfoVO.setProjectName(projectDO.getName());
        return baseInfoVO;
    }

    /**
     * 查询任务基本信息
     *
     * @param taskId 任务id
     * @return TaskBaseInfoVO
     */
    @Override
    public TaskBaseInfoVO getTaskBaseInfo(Long taskId) {
        ProjectTaskDO projectTaskDO = projectManager.queryTaskByTaskId(taskId);
        TaskBaseInfoVO baseInfoVO = new TaskBaseInfoVO();
        baseInfoVO.setTaskId(taskId);
        baseInfoVO.setTaskName(projectTaskDO.getName());
        baseInfoVO.setResourceType(projectTaskDO.getType());
        return baseInfoVO;
    }

    /**
     * 新增抽样验证
     *
     * @param addTaskSampleDTO 新增抽样参数
     * @return List<ProjectTaskDetailVO>
     */
    @Override
    public List<ProjectTaskDetailVO> addTaskSample(AddTaskSampleDTO addTaskSampleDTO) {
        String reg = "^[1-9]\\d*$";
        if (!addTaskSampleDTO.getSampleNum().toString().matches(reg)) {
            throw new BizException("抽样数量必须是正整数");
        }
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(addTaskSampleDTO.getTaskId());
        if (existTask == null) {
            throw new BizException(TASK_NULL);
        }
        List<ProjectTaskDetailVO> list = new ArrayList<>();
        long start = System.currentTimeMillis();
        List<Long> detailIds = projectManager.queryDetailIdsByTask(addTaskSampleDTO.getTaskId(), SampleTypeEnums.NONE.getValue());
        long end = System.currentTimeMillis();
        log.info("查询任务明细耗时: {}ms", (end - start));
        if (isEmpty(detailIds)) {
            throw new BizException("任务明细都已抽样验证完毕");
        }

        Integer sampleNum = addTaskSampleDTO.getSampleNum();
        if (sampleNum > detailIds.size()) {
            //抽样数量大于任务明细数量,全部抽样
            List<ProjectTaskDetailBO> detailBOS = projectManager.queryDetailBOsByIds(detailIds);
            for (ProjectTaskDetailBO item : detailBOS) {
                ProjectTaskDetailVO detailVO = projectMapStruct.convertTOTaskDetailVO(item);
                list.add(detailVO);
            }
        } else {
            //随机
            Map<Integer, Integer> rdMap = new HashMap<>();
            SecureRandom random;
            try {
                random = SecureRandom.getInstanceStrong();
                while (sampleNum > rdMap.keySet().size()) {
                    int idx = random.nextInt(detailIds.size());
                    rdMap.putIfAbsent(idx, 1);
                }
                List<Long> rdDetailIds = rdMap.keySet().stream().map(detailIds::get).collect(Collectors.toList());
                List<ProjectTaskDetailBO> detailBOS = projectManager.queryDetailBOsByIds(rdDetailIds);
                for (ProjectTaskDetailBO item : detailBOS) {
                    ProjectTaskDetailVO detailVO = projectMapStruct.convertTOTaskDetailVO(item);
                    list.add(detailVO);
                }
            } catch (NoSuchAlgorithmException e) {
                log.error(e.getMessage());
                throw new BizException(e.getMessage());
            }

        }
        log.info("随机抽样耗时: {}ms", (System.currentTimeMillis() - end));
        return list;
    }

    /**
     * 项目抽查验证
     *
     * @param markProjectTaskDTO 抽查验证参数
     */
    @Override
    @Transactional
    public void sampleMark(MarkProjectTaskDTO markProjectTaskDTO) {
        ProjectTaskDetailDO existDetail = projectManager.queryTaskDetailById(markProjectTaskDTO.getTaskDetailId());
        if (existDetail == null) {
            throw new BizException("任务明细不存在");
        }
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(existDetail.getTaskId());
        if (existTask == null) {
            throw new BizException(TASK_NULL);
        }
        Integer sampleType = markProjectTaskDTO.getSampleType();
        if (sampleType == null || sampleType.equals(IsOrNotEnum.NOT.getValue())) {
            throw new BizException("验证结果不能为空");
        }
        if (sampleType.equals(SampleTypeEnums.FAIL.getValue()) && isBlank(markProjectTaskDTO.getSampleDesc())) {
            throw new BizException("如果验证失败,请填写验证说明");
        }
        ProjectTaskDetailDO detailDO = new ProjectTaskDetailDO();
        detailDO.setId(existDetail.getId());
        detailDO.setTaskId(existDetail.getTaskId());
        detailDO.setSampleTime(LocalDateTime.now());
        detailDO.setSampleType(sampleType);
        String sampleDesc = markProjectTaskDTO.getSampleDesc();
        if (isNotBlank(sampleDesc) && CharSequenceUtil.isNotEmpty(sampleDesc)) {
            sampleDesc = sampleDesc.trim();
            if (sampleDesc.length() > 100) {
                throw new BizException("验证说明长度不能超过100字");
            } else {
                detailDO.setSampleDesc(sampleDesc);
            }
        }
        if (projectManager.updateTaskDetail(detailDO) < 1) {
            throw new BizException("任务明细验证失败");
        }
        //查询任务验证成功和失败数量
        int successNum = projectManager.querySampleNumByTask(existTask.getTaskId(), SampleTypeEnums.SUCCESS.getValue());
        int failedNum = projectManager.querySampleNumByTask(existTask.getTaskId(), SampleTypeEnums.FAIL.getValue());
        ProjectTaskDO taskDO = new ProjectTaskDO();
        taskDO.setId(existTask.getId());
        taskDO.setTaskId(existTask.getTaskId());
        taskDO.setSuccessNum(successNum);
        taskDO.setFailedNum(failedNum);
        projectManager.updateTask(taskDO);
        logEventPublisher.publish(Log.builder()
                .append("任务明细验证,明细id: " + detailDO.getDetailId() + " ,验证人: " + LoginUserUtil.getLoginUser().getUserName())
                .isUpdate());
    }

    /**
     * 项目验证完成
     *
     * @param projectId 项目id
     */
    @Override
    @Transactional
    public void verifyProject(Long projectId) {
        ProjectDO existProject = projectManager.queryById(projectId);
        if (existProject == null) {
            throw new BizException(PROJECT_NULL);
        }
        Page<ProjectTaskBO> list = projectManager.queryTaskPageByProject(projectId);
        if (list.stream().anyMatch(item -> item.getIsComplete().equals(IsOrNotEnum.NOT.getValue()))) {
            throw new BizException("项目中有任务没有打标完成");
        }
        ProjectDO projectDO = new ProjectDO();
        projectDO.setId(existProject.getId());
        projectDO.setCompleteTime(LocalDateTime.now());
        if (projectManager.updateById(projectDO) < 1) {
            throw new BizException("验证完成项目报错");
        }
        logEventPublisher.publish(Log.builder()
                .append("验证完成项目,项目id: " + projectId + " ,验证人: " + LoginUserUtil.getLoginUser().getUserName())
                .isUpdate());
    }

    /**
     * 分页查询我的任务
     *
     * @param queryTaskDTO 查询条件
     * @return PageResult<ProjectTaskPageItemVO>
     */
    @Override
    public PageResult<ProjectTaskPageItemVO> getMyTaskPage(QueryTaskDTO queryTaskDTO) {
        startPage(queryTaskDTO.getPageNum(), queryTaskDTO.getPageSize());
        queryTaskDTO.setUserId(LoginUserUtil.getLoginUserId());
        long start = System.currentTimeMillis();
        Page<ProjectTaskBO> projectTaskBOS = projectManager.queryTaskPage(queryTaskDTO);
        long end = System.currentTimeMillis();
        log.info("查询我的任务耗时: {}ms", (end - start));
        Page<ProjectTaskPageItemVO> page = new Page<>();
        LocalDateTime now = LocalDateTime.now();
        if (isNotEmpty(projectTaskBOS)) {
            projectTaskBOS.forEach(item -> {
                ProjectTaskPageItemVO pageItemVO = projectMapStruct.convertTOTaskPageItemVO(item);
                pageItemVO.setTypeName(ResourceTypeEnum.getName(item.getType()));
                if (item.getIsComplete().equals(IsOrNotEnum.IS.getValue())) {
                    pageItemVO.setStatusName(TaskRunningEnums.COMPLETED.getName());
                    pageItemVO.setStatus(TaskRunningEnums.COMPLETED.getValue());
                } else {
                    if (now.isAfter(item.getEndTime())) {
                        pageItemVO.setStatusName(TaskRunningEnums.EXPIRED.getName());
                        pageItemVO.setStatus(TaskRunningEnums.EXPIRED.getValue());
                    } else if (now.isBefore(item.getPushTime())) {
                        pageItemVO.setStatusName(TaskRunningEnums.WAIT.getName());
                        pageItemVO.setStatus(TaskRunningEnums.WAIT.getValue());
                    } else {
                        pageItemVO.setStatusName(TaskRunningEnums.RUNNING.getName());
                        pageItemVO.setStatus(TaskRunningEnums.RUNNING.getValue());
                    }
                }
                pageItemVO.setMarkedNum(item.getMarkNum() + item.getIgnoreNum());
                pageItemVO.setTotalNum(item.getTaskCount());
                pageItemVO.setBeginTimeStr(DateUtil.format(item.getBeginTime(), Constant.DATETIME_FORMAT));
                pageItemVO.setEndTimeStr(DateUtil.format(item.getEndTime(), Constant.DATETIME_FORMAT));
                pageItemVO.setCompleteTimeStr(item.getCompleteTime() != null ?
                        DateUtil.format(item.getCompleteTime(), Constant.DATETIME_FORMAT) : "");
                pageItemVO.setValidTimeStr(DateUtil.format(item.getBeginTime(), Constant.DATETIME_FORMAT)
                        + " ~ " + DateUtil.format(item.getEndTime(), Constant.DATETIME_FORMAT));
                int totalSampleNum = item.getSuccessNum() + item.getFailedNum();
                if (totalSampleNum > 0) {
                    String successRate = BigDecimal.valueOf(item.getSuccessNum())
                            .divide(BigDecimal.valueOf(totalSampleNum), 2, RoundingMode.HALF_UP)
                            .multiply(BigDecimal.valueOf(100)) + "%";
                    pageItemVO.setSuccessRate(successRate);
                }
                page.add(pageItemVO);
            });
        }
        return PageResult.of(projectTaskBOS.getTotal(), page);
    }

    /**
     * 查询我的任务完成情况
     *
     * @return MyTaskStatisticVO
     */
    @Override
    public MyTaskStatisticVO getMyTaskStatistics() {
        //查询未完成的任务
        List<ProjectTaskBO> notCompTasks = projectManager.queryNotCompTasksByUserId(LoginUserUtil.getLoginUserId());
        MyTaskStatisticVO statisticVO = new MyTaskStatisticVO();
        statisticVO.setNotCompleteNum(0);
        statisticVO.setExpiredSoonNum(0);
        if (isNotEmpty(notCompTasks)) {
            statisticVO.setNotCompleteNum(notCompTasks.size());
            int count = (int) notCompTasks.stream().filter(
                    item -> (item.getEndTime().get(ChronoField.DAY_OF_YEAR) - LocalDateTime.now().get(ChronoField.DAY_OF_YEAR)) < 5
                            && (item.getEndTime().get(ChronoField.DAY_OF_YEAR) - LocalDateTime.now().get(ChronoField.DAY_OF_YEAR)) >= 0).count();
            statisticVO.setExpiredSoonNum(count);
        }

        return statisticVO;
    }

    /**
     * 标注结果查询
     *
     * @param queryTaskDTO 查询条件
     * @return PageResult<ProjectTaskPageItemVO>
     */
    @Override
    public PageResult<ProjectTaskPageItemVO> getMarkResultPage(QueryTaskDTO queryTaskDTO) {
        queryTaskDTO.setIsFinish(IsOrNotEnum.IS.getValue());
        Page<ProjectTaskBO> projectTaskBOS = projectManager.queryTaskPage(queryTaskDTO);
        Page<ProjectTaskPageItemVO> page = new Page<>();
        if (isNotEmpty(projectTaskBOS)) {
            projectTaskBOS.forEach(item -> {
                ProjectTaskPageItemVO pageItemVO = projectMapStruct.convertTOTaskPageItemVO(item);
                pageItemVO.setTotalNum(item.getTaskCount());
                pageItemVO.setTypeName(ResourceTypeEnum.getName(item.getType()));
                pageItemVO.setTotalNum(item.getTaskCount());
                pageItemVO.setCompleteTimeStr(item.getCompleteTime() != null ?
                        DateUtil.format(item.getCompleteTime(), Constant.DATETIME_FORMAT) : "");
                if (isNotBlank(item.getFirstGroup()) && CharSequenceUtil.isNotEmpty(item.getFirstGroup())) {
                    pageItemVO.setProjectGroup(item.getFirstGroup() + "/" + item.getSecondGroup());
                } else {
                    pageItemVO.setProjectGroup(item.getSecondGroup());
                }
                page.add(pageItemVO);
            });
        }
        return PageResult.of(projectTaskBOS.getTotal(), page);
    }

    @Override
    public void exportMarkResultJson(ExportTaskDTO exportTaskDTO, HttpServletResponse response) {

        //构建分类层级
        Map<String, String> categoryMap = new HashMap<>();
        List<CategoryDO> levelOneCategoryList = categoryManager.queryListByParentId("0");
        if (isNotEmpty(levelOneCategoryList)) {
            buildCateNameMap(categoryMap, levelOneCategoryList, "");
        }

        List<ExportDataBO> dataBOS = projectManager.exportTaskDetails(exportTaskDTO);
        StringBuilder jsonStr = new StringBuilder();
        if (isNotEmpty(dataBOS)) {
            if (dataBOS.size() > 100000) {
                throw new BizException("导出行数超过10w,请筛选后再导出");
            }
            dataBOS.forEach(item -> {
                item.setTypeName(ResourceTypeEnum.getName(item.getType()));
                if (item.getType().equals(ResourceTypeEnum.PICTURE.getValue())) {
                    item.setContent(item.getPicUrl());
                } else if (item.getType().equals(ResourceTypeEnum.MOVIE.getValue())) {
                    item.setContent(item.getMovUrl());
                }
                item.setCategoryName(categoryMap.get(item.getCategoryId()) == null ? "" : categoryMap.get(item.getCategoryId()));
                Map<String, Object> singleMap = new HashMap<>();
                Map<String, String> dataMap = new HashMap<>();
                dataMap.put("projectId", item.getProjectId());
                dataMap.put("projectName", item.getProjectName());
                dataMap.put("taskId", item.getTaskId());
                dataMap.put("taskName", item.getTaskName());
                dataMap.put("resourceType", item.getTypeName());
                dataMap.put("resourceContent", item.getContent());
                dataMap.put("categoryId", item.getCategoryId());
                dataMap.put("categoryPath", item.getCategoryName());
                singleMap.put("result", dataMap);
                String singleData = JSONUtil.toJsonStr(singleMap);
                jsonStr.append(singleData).append(System.getProperty("line.separator"));
            });

            try (ServletOutputStream os = response.getOutputStream()) {
                String fileName = "标注结果" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".json";
                response.setCharacterEncoding("utf-8");
                response.setContentType("text/plain");
                response.setHeader("fileName", URLEncoder.encode(fileName, StandardCharsets.UTF_8));
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                response.setHeader("Access-Control-Expose-Headers", "fileName");
                os.write(jsonStr.toString().getBytes(StandardCharsets.UTF_8));
                os.flush();
            } catch (IOException e) {
                log.error("导出标注结果失败: ", e);
                throw new BizException("导出标注结果失败");
            }
        }
    }

    /**
     * 分页查询任务明细
     *
     * @param taskId 任务id
     * @param dataId 资源数据id
     * @param type   类型
     * @return List<ProjectTaskDetailVO>
     */
    @Override
    public List<ProjectTaskDetailVO> getTaskDetailPage(Long taskId, Long dataId, Long type) {
        ProjectTaskDO existTask = projectManager.queryTaskByTaskId(taskId);
        if (existTask == null) {
            throw new BizException(TASK_NULL);
        }
        List<ProjectTaskDetailBO> taskDetailBOS = projectManager.queryDetailBOsByTaskId(taskId, dataId, type);
        List<ProjectTaskDetailVO> list = new ArrayList<>();
        if (isNotEmpty(taskDetailBOS)) {
            for (ProjectTaskDetailBO item : taskDetailBOS) {
                ProjectTaskDetailVO detailVO = projectMapStruct.convertTOTaskDetailVO(item);
                list.add(detailVO);
            }
        }
        return list;
    }

    /**
     * 查询项目信息
     *
     * @param projectId 项目id
     * @return ProjectVO
     */
    @Override
    public ProjectVO getProjectById(Long projectId) {
        ProjectDO existProject = projectManager.queryById(projectId);
        if (existProject == null) {
            throw new BizException(PROJECT_NULL);
        }
        ProjectVO projectVO = projectMapStruct.convertDOToProjectVO(existProject);
        ProjectGroupDO groupDO = projectManager.queryGroupById(projectVO.getGroupId());
        List<Long> groupIds = new ArrayList<>();
        if (groupDO.getParentId() != 0) {
            groupIds.add(groupDO.getParentId());
            groupIds.add(groupDO.getGroupId());
        } else {
            groupIds.add(groupDO.getGroupId());
        }
        projectVO.setGroupIds(groupIds);
        projectVO.setBeginTime(DateUtil.format(existProject.getBeginTime(), Constant.DATETIME_FORMAT));
        projectVO.setEndTime(DateUtil.format(existProject.getEndTime(), Constant.DATETIME_FORMAT));
        projectVO.setPushTime(DateUtil.format(existProject.getPushTime(), Constant.DATETIME_FORMAT));
        ProjectCategoryDO projectCategoryDO = projectManager.queryProCateById(projectId);
        if (projectCategoryDO != null) {
            projectVO.setCategoryId(projectCategoryDO.getCategoryId());
            CategoryDO categoryDO = categoryManager.queryByPrimary(projectCategoryDO.getCategoryId());
            if (categoryDO == null) {
                throw new BizException("分类不存在");
            }
            List<String> cateIds = new ArrayList<>();
            cateIds.add(categoryDO.getCategoryId());
            if (!categoryDO.getParentCategoryId().equals("0")) {
                getCategoryIds(categoryDO, cateIds);
            }
            Collections.reverse(cateIds);
            projectVO.setCategoryIds(cateIds);
        }
        List<ProjectResourceDO> proResDOS = projectManager.queryProResListByProId(projectId);
        if (isNotEmpty(proResDOS)) {
            List<Long> resourceIds = proResDOS.stream().map(ProjectResourceDO::getResourceId).collect(Collectors.toList());
            projectVO.setResourceIds(resourceIds);
            List<ProjectUserDO> projectUserDOS = projectManager.queryProUseListByProId(projectId);
            if (isNotEmpty(projectUserDOS)) {
                List<Long> userIds = projectUserDOS.stream().map(ProjectUserDO::getUserId).collect(Collectors.toList());
                projectVO.setUserIds(userIds);

                List<Long> resourceIdList = proResDOS.stream().map(ProjectResourceDO::getResourceId).distinct().collect(Collectors.toList());
                List<ProjectTaskAssignVO> taskList = buildTaskList(existProject, userIds, resourceIdList);
                projectVO.setTaskList(taskList);
            } else {
                projectVO.setUserIds(Collections.emptyList());
                projectVO.setTaskList(Collections.emptyList());
            }
        } else {
            projectVO.setResourceIds(Collections.emptyList());
            projectVO.setUserIds(Collections.emptyList());
            projectVO.setTaskList(Collections.emptyList());
        }
        return projectVO;
    }

    /**
     * 查询标注结果
     *
     * @param detailId 明细id
     * @return TaskDetailVO
     */
    @Override
    public TaskDetailVO getTaskDetail(Long detailId) {
        ProjectTaskDetailDO detailDO = projectManager.queryTaskDetailById(detailId);
        TaskDetailVO detailVO = projectMapStruct.convertDOTOTaskDetailVO(detailDO);
        detailVO.setHasProblem(detailDO.getHasProblem() == 1);
        if (isNotBlank(detailDO.getCategoryIds()) && CharSequenceUtil.isNotEmpty(detailDO.getCategoryIds())) {
            List<String> cateIds = new ArrayList<>();
            String[] split = detailDO.getCategoryIds().substring(1, detailDO.getCategoryIds().length() - 1).split(",");
            for (String str : split) {
                cateIds.add(str.trim());
            }
            detailVO.setCategoryIds(cateIds);
        }
        if (detailDO.getIsMark().equals(IsOrNotEnum.NOT.getValue())) {
            detailVO.setType(null);
        }
        return detailVO;
    }

    private void getCategoryIds(CategoryDO categoryDO, List<String> cateIds) {
        if (!categoryDO.getParentCategoryId().equals("0")) {
            cateIds.add(categoryDO.getParentCategoryId());
            CategoryDO parent = categoryManager.queryByPrimary(categoryDO.getParentCategoryId());
            getCategoryIds(parent, cateIds);
        }
    }

    /**
     * 构建任务列表
     *
     * @param project        项目
     * @param userIds        用户id集合
     * @param resourceIdList 资源id集合
     * @return List<ProjectTaskAssignVO>
     */
    private List<ProjectTaskAssignVO> buildTaskList(ProjectDO project, List<Long> userIds, List<Long> resourceIdList) {
        List<ResourceDO> resourceList = resourceManager.queryListByIds(resourceIdList);
        List<ResourceDO> docList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.DOC.getValue())).collect(Collectors.toList());
        List<ResourceDO> picList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.PICTURE.getValue())).collect(Collectors.toList());
        List<ResourceDO> movList = resourceList.stream().filter(item -> item.getType().equals(ResourceTypeEnum.MOVIE.getValue())).collect(Collectors.toList());

        //类型数量
        List<SysUserDO> userList = userManager.queryListByUserIds(userIds);
        List<Integer> typeList = resourceList.stream().map(ResourceDO::getType).distinct().collect(Collectors.toList());
        int userNum = userList.size();
        List<String> taskNameList = getTaskNameList(userNum * (typeList.size()), project.getName());
        int index = 0;
        List<ProjectTaskAssignVO> taskList = new ArrayList<>();
        int docSum = docList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        int picSum = picList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        int movSum = movList.stream().mapToInt(ResourceDO::getResourceCount).sum();
        for (SysUserDO user : userList) {
            for (Integer type : typeList) {
                ProjectTaskAssignVO result = new ProjectTaskAssignVO();
                result.setTaskId(snowflakeIdFactory.nextId());
                result.setTaskName(taskNameList.get(index));
                result.setTypeName(ResourceTypeEnum.getName(type));
                result.setAccount(user.getAccount());
                result.setUserName(user.getUserName());
                result.setDataNum(0);
                if (type.equals(ResourceTypeEnum.DOC.getValue())) {
                    result.setDataNum(docSum / userNum);
                } else if (type.equals(ResourceTypeEnum.PICTURE.getValue())) {
                    result.setDataNum(picSum / userNum);
                } else {
                    result.setDataNum(movSum / userNum);
                }
                index++;
                taskList.add(result);
            }
        }
        int docRestNum = docSum - (taskList.stream().filter(item -> item.getTypeName().equals(ResourceTypeEnum.DOC.getName())).mapToInt(ProjectTaskAssignVO::getDataNum).sum());
        int picRestNum = picSum - (taskList.stream().filter(item -> item.getTypeName().equals(ResourceTypeEnum.PICTURE.getName())).mapToInt(ProjectTaskAssignVO::getDataNum).sum());
        int movRestNum = movSum - (taskList.stream().filter(item -> item.getTypeName().equals(ResourceTypeEnum.MOVIE.getName())).mapToInt(ProjectTaskAssignVO::getDataNum).sum());
        //把除不尽的余数,继续分配下去
        for (int i = 0; i < docRestNum; i++) {
            SysUserDO user = userList.get(i);
            taskList.stream()
                    .filter(item -> item.getTypeName().equals(ResourceTypeEnum.DOC.getName())
                            && item.getAccount().equals(user.getAccount()))
                    .findFirst()
                    .ifPresent(projectTaskAssignVO -> projectTaskAssignVO.setDataNum(projectTaskAssignVO.getDataNum() + 1));
        }
        for (int i = 0; i < picRestNum; i++) {
            SysUserDO user = userList.get(i);
            taskList.stream()
                    .filter(item -> item.getTypeName().equals(ResourceTypeEnum.PICTURE.getName())
                            && item.getAccount().equals(user.getAccount()))
                    .findFirst()
                    .ifPresent(projectTaskAssignVO -> projectTaskAssignVO.setDataNum(projectTaskAssignVO.getDataNum() + 1));
        }
        for (int i = 0; i < movRestNum; i++) {
            SysUserDO user = userList.get(i);
            taskList.stream()
                    .filter(item -> item.getTypeName().equals(ResourceTypeEnum.MOVIE.getName())
                            && item.getAccount().equals(user.getAccount()))
                    .findFirst()
                    .ifPresent(projectTaskAssignVO -> projectTaskAssignVO.setDataNum(projectTaskAssignVO.getDataNum() + 1));
        }
        return taskList;
    }

    /**
     * 构建分类名称map
     *
     * @param categoryMap    分类map
     * @param categoryDOList 分类集合
     * @param parentName     父级分类名称
     */
    private void buildCateNameMap(Map<String, String> categoryMap, List<CategoryDO> categoryDOList, String parentName) {
        for (CategoryDO item : categoryDOList) {
            List<CategoryDO> categoryDOS = categoryManager.queryListByParentId(item.getCategoryId());
            String cateName = parentName + "/" + item.getCategoryName();
            item.setCategoryName(cateName);
            categoryMap.put(item.getCategoryId(), cateName);
            if (isNotEmpty(categoryDOS)) {
                buildCateNameMap(categoryMap, categoryDOS, item.getCategoryName());
            }
        }
    }


    /**
     * 解决余数问题
     *
     * @param taskList 任务集合
     * @param user     用户
     * @param type     资源类型
     */
    private void solveRestNum(List<ProjectTaskDO> taskList, SysUserDO user, Integer type) {
        taskList.stream()
                .filter(item -> item.getType().equals(type)
                        && item.getAccount().equals(user.getAccount()))
                .findFirst()
                .ifPresent(projectTaskDO -> projectTaskDO.setTaskCount(projectTaskDO.getTaskCount() + 1));
    }

    /**
     * 构造任务明细数据
     *
     * @param projectId        项目id
     * @param taskList         任务集合
     * @param dataList         数据资源集合
     * @param taskDetailDOList 任务明细集合
     * @param now              创建时间
     * @param type             资源类型
     * @return List<ProjectTaskDetailDO>
     */
    private List<ProjectTaskDetailDO> buildTaskDetails(Long projectId,
                                                       List<ProjectTaskDO> taskList,
                                                       List<ResourceDataDO> dataList,
                                                       List<ProjectTaskDetailDO> taskDetailDOList,
                                                       LocalDateTime now,
                                                       int type) {
        List<ProjectTaskDO> projectTaskDOS = taskList.stream().filter(item -> item.getType().equals(type)).collect(Collectors.toList());
        int idx = 0;
        for (int i = 0; i < projectTaskDOS.size(); ) {
            ProjectTaskDO taskDO = projectTaskDOS.get(i);
            Long dataId = dataList.get(idx).getDataId();
            ProjectTaskDetailDO detailDO = new ProjectTaskDetailDO();
            detailDO.setDetailId(snowflakeIdFactory.nextId());
            detailDO.setProjectId(projectId);
            detailDO.setTaskId(taskDO.getTaskId());
            detailDO.setDataId(dataId);
            detailDO.setIsMark(IsOrNotEnum.NOT.getValue());
            detailDO.setCreateTime(now);
            detailDO.setType(TaskMarkEnums.MARK.getValue());
            detailDO.setSampleType(SampleTypeEnums.NONE.getValue());
            detailDO.setHasProblem(IsOrNotEnum.NOT.getValue());
            taskDetailDOList.add(detailDO);
            idx++;
            if ((i + 1) % projectTaskDOS.size() == 0) {
                i = 0;
            } else {
                i++;
            }
            if (idx == dataList.size()) {
                break;
            }
        }
        return taskDetailDOList;
    }

    /**
     * 获取任务名称列表
     *
     * @param size 任务数量
     * @param name 项目名称
     * @return List<String>
     */
    private List<String> getTaskNameList(int size, String name) {
        List<String> taskNames = new ArrayList<>();
        if (size > 999) {
            throw new BizException("任务数量不能超过999");
        } else if (size <= 0) {
            throw new BizException("任务数量小于1,请检查");
        } else {
            for (int i = 1; i < size + 1; i++) {
                if (i <= 9) {
                    taskNames.add(name + "00" + i);
                } else if (i <= 99) {
                    taskNames.add(name + "0" + i);
                } else {
                    taskNames.add(name + i);
                }
            }
        }
        return taskNames;
    }

    /**
     * 对比时间大小
     *
     * @param beginTime 开始时间
     * @param endTime   结束时间
     * @return boolean
     */
    private boolean compareTime(LocalDateTime beginTime, LocalDateTime endTime) {
        if (beginTime == null || endTime == null) {
            throw new BizException("请输入正确时间");
        }
        return beginTime.isAfter(endTime);
    }

    /**
     * 构建分类树
     *
     * @param categoryDOList 分类集合
     * @return List<CategoryTreeVo>
     */
    private List<CategoryTreeVo> buildTree(List<CategoryDO> categoryDOList) {
        List<CategoryTreeVo> treeList = new ArrayList<>();
        categoryDOList.forEach(levelOneCate -> {
            CategoryTreeVo treeVo = new CategoryTreeVo();
            treeVo.setValue(levelOneCate.getCategoryId());
            treeVo.setLabel(levelOneCate.getCategoryName());
            List<CategoryDO> categoryDOS = categoryManager.queryListByParentId(treeVo.getValue());
            if (isNotEmpty(categoryDOS)) {
                List<CategoryTreeVo> categoryTreeVos = buildTree(categoryDOS);
                treeVo.setChildren(categoryTreeVos);
            }
            treeList.add(treeVo);
        });
        return treeList;
    }
}




