package cn.sino.mark.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import cn.sino.common.base.PageResult;
import cn.sino.common.exception.BizException;
import cn.sino.common.util.SnowflakeIdFactory;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.enums.IsOrNotEnum;
import cn.sino.mark.common.enums.ResourceAssignStatusEnum;
import cn.sino.mark.common.enums.ResourceTypeEnum;
import cn.sino.mark.common.enums.SysUserStatusEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.bo.ResourceBO;
import cn.sino.mark.common.model.bo.ResourceDataBO;
import cn.sino.mark.common.model.dto.AddResourceDTO;
import cn.sino.mark.common.model.dto.ImportResourceDTO;
import cn.sino.mark.common.model.dto.QueryResourceDTO;
import cn.sino.mark.common.model.vo.ImportResourceVO;
import cn.sino.mark.common.model.vo.ResourceBaseInfoVO;
import cn.sino.mark.common.model.vo.ResourcePageItemVO;
import cn.sino.mark.common.utils.ListsUtil;
import cn.sino.mark.dao.model.ProjectResourceDO;
import cn.sino.mark.dao.model.ResourceDO;
import cn.sino.mark.dao.model.ResourceDataDO;
import cn.sino.mark.manager.ProjectManager;
import cn.sino.mark.manager.ResourceManager;
import cn.sino.mark.service.ResourceService;
import cn.sino.mark.service.convert.ResourceMapStruct;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static com.github.pagehelper.page.PageMethod.startPage;

/**
 * @author sch
 * @description 资源管理实现类
 * @className cn.sino.mark.service.impl.ResourceServiceImpl
 * @date 2022/4/1
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ResourceServiceImpl implements ResourceService {

    private final ResourceManager resourceManager;
    private final SnowflakeIdFactory snowflakeIdFactory;
    private final ResourceMapStruct resourceMapStruct;
    private final SysLogEventPublisher logEventPublisher;
    private final ProjectManager projectManager;

    /**
     * 新增资源
     *
     * @param addResourceDTO 新增资源信息
     */
    @Override
    @Transactional
    public Long addResource(AddResourceDTO addResourceDTO) {
        String name = addResourceDTO.getName().trim();
        //校验名称重复
        ResourceDO sameNameResource = resourceManager.queryByName(name);
        if (sameNameResource != null) {
            throw new BizException("数据集名称已存在名称重复请重新设置");
        }
        ResourceDO resourceDO = new ResourceDO();
        resourceDO.setResourceId(snowflakeIdFactory.nextId());
        resourceDO.setName(name);
        resourceDO.setType(addResourceDTO.getType());
        resourceDO.setAssignStatus(ResourceAssignStatusEnum.NOT_ASSIGNED.getValue());
        resourceDO.setStatus(SysUserStatusEnum.NORMAL.getValue());
        resourceManager.addResource(resourceDO);
        logEventPublisher.publish(Log.builder().append("新增资源,ID:" + resourceDO.getResourceId() + ",名称:" + name).isInsert());
        return resourceDO.getResourceId();
    }

    /**
     * 解析资源
     *
     * @param uploadFile 文件
     * @param type       资源类型
     * @return ImportResourceVO
     */
    @Override
    public ImportResourceVO analyseResource(MultipartFile uploadFile, Integer type) {
        long start = System.currentTimeMillis();
        log.info("开始解析资源...");
        String suffix = "." + getUploadSuffix(Objects.requireNonNull(uploadFile.getOriginalFilename()));
        if (!isJson(suffix)) {
            throw new BizException("只能上传json或txt后缀的文本文件");
        }
        List<String> effectiveJsonList = new ArrayList<>();
        BufferedReader br = null;
        StringBuilder finalSourceStr = new StringBuilder();
        List<String> dataList = new ArrayList<>();
        int totalCount = 0;
        try {
            InputStreamReader read = new InputStreamReader(new ByteArrayInputStream(uploadFile.getBytes()));
            br = new BufferedReader(read);//BufferedReader读取文件
            String s;
            while ((s = br.readLine()) != null) {
                totalCount++;
                try {
                    Map map = (Map) JSONUtil.parse(s.trim());
                    Map dataMap = (Map) map.get("data");
                    if (dataMap != null) {
                        if (ResourceTypeEnum.DOC.getValue().equals(type)) {
                            Object txtSource = dataMap.get("txtsource");
                            if (txtSource != null) {
                                effectiveJsonList.add(s.trim());
                                finalSourceStr.append(s.trim()).append("&");
                                dataList.add(s.trim());
                            }
                        } else if (ResourceTypeEnum.PICTURE.getValue().equals(type)) {
                            Object picSource = dataMap.get("picsource");
                            if (picSource != null) {
                                effectiveJsonList.add(s.trim());
                                finalSourceStr.append(s.trim()).append("&");
                                dataList.add(s.trim());
                            }
                        } else {
                            Object movSource = dataMap.get("movsource");
                            if (movSource != null) {
                                effectiveJsonList.add(s.trim());
                                finalSourceStr.append(s.trim()).append("&");
                                dataList.add(s.trim());
                            }
                        }
                    }
                } catch (JSONException jsonException) {
                    //解析失败不用理会
                    log.error("json解析失败: ", jsonException);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        ImportResourceVO importResourceVO = new ImportResourceVO();
        importResourceVO.setResourceCount(effectiveJsonList.size());
        importResourceVO.setEffectiveResourceStr(finalSourceStr.toString());
        importResourceVO.setDataList(dataList);
        importResourceVO.setFailedCount(totalCount - effectiveJsonList.size());
        log.info("资源解析结束,耗时: {}ms", (System.currentTimeMillis() - start));
        return importResourceVO;
    }

    /**
     * 解析资源并上传
     *
     * @param uploadFile 文件
     * @param resourceId 资源id
     */
    @Override
    @Transactional
    public ImportResourceVO analyseResourceV2(MultipartFile uploadFile, Long resourceId) {
        long start = System.currentTimeMillis();
        ResourceDO resourceDO = resourceManager.queryById(resourceId);
        if (resourceDO == null) {
            throw new BizException("资源不存在");
        }
        resourceManager.deleteDataByResourceId(resourceId);
        Integer type = resourceDO.getType();

        List<ResourceDataBO> resourceDataList;
        if (type.equals(ResourceTypeEnum.LINK.getValue())) {
            resourceDataList = parseExcelContent(uploadFile, resourceId, type);
        } else {
            resourceDataList = parseJsonContent(uploadFile, resourceId, type);
        }
        List<ResourceDataDO> resourceDataDOList = resourceDataList.stream()
                .filter(ResourceDataBO::getIsValid).map(ResourceDataBO::getResourceData).collect(Collectors.toList());
        if (isNotEmpty(resourceDataDOList)) {
            ListsUtil.splitRun(resourceDataDOList, 5000, resourceManager::insertBatch);
            logEventPublisher.publish(Log.builder().append("上传资源数据,ID:" + resourceId + ",数据量:" + resourceDataDOList.size()).isInsert());
        }

        ImportResourceVO importResourceVO = new ImportResourceVO();
        importResourceVO.setResourceCount(resourceDataDOList.size());
        importResourceVO.setFailedCount(resourceDataList.size() - resourceDataDOList.size());
        log.info("资源解析结束,耗时: {}ms", (System.currentTimeMillis() - start));
        return importResourceVO;
    }

    /**
     * 上传资源
     *
     * @param importResourceDTO 上传参数
     */
    @Override
    @Transactional
    public void importResource(ImportResourceDTO importResourceDTO) {
        ResourceDO existResource = resourceManager.queryById(importResourceDTO.getResourceId());
        List<String> dataList = importResourceDTO.getDataList();
        if (isNotEmpty(dataList)) {
            //校验资源是否存在
            if (existResource == null || SysUserStatusEnum.FROZEN.getValue() == existResource.getStatus()) {
                throw new BizException("资源包不存在或者已作废");
            }
            List<ResourceDataDO> resourceDataDOS = new ArrayList<>();
            LocalDateTime now = LocalDateTime.now();
            for (String singleData : dataList) {
                ResourceDataDO resourceDataDO = new ResourceDataDO();
                resourceDataDO.setDataId(snowflakeIdFactory.nextId());
                resourceDataDO.setResourceId(existResource.getResourceId());
                resourceDataDO.setCreateTime(now);
                try {
                    Map map = (Map) JSONUtil.parse(singleData.trim());
                    Map dataMap = (Map) map.get("data");
                    Object hint = dataMap.get("hint");
                    if (hint != null) {
                        resourceDataDO.setHint(hint.toString().trim());
                    }
                    Object source;
                    if (existResource.getType().equals(ResourceTypeEnum.DOC.getValue())) {
                        source = dataMap.get("txtsource");
                        if (source != null) {
                            resourceDataDO.setContent(source.toString().trim());
                            resourceDataDOS.add(resourceDataDO);
                        }
                    } else if (existResource.getType().equals(ResourceTypeEnum.PICTURE.getValue())) {
                        source = dataMap.get("picsource");
                        if (source != null) {
                            resourceDataDO.setPicUrl(source.toString().trim());
                            resourceDataDOS.add(resourceDataDO);
                        }
                    } else {
                        source = dataMap.get("movsource");
                        if (source != null) {
                            resourceDataDO.setMovUrl(source.toString().trim());
                            resourceDataDOS.add(resourceDataDO);
                        }
                    }

                } catch (JSONException jsonException) {
                    log.error(jsonException.getMessage());
                }
            }
            if (isNotEmpty(resourceDataDOS)) {
                ListsUtil.splitRun(resourceDataDOS, 5000, resourceManager::insertBatch);
                logEventPublisher.publish(Log.builder().append("上传资源数据,ID:" + existResource.getResourceId() + ",数据量:" + resourceDataDOS.size()).isInsert());
            }
            ResourceDO resourceDO = new ResourceDO();
            resourceDO.setId(existResource.getId());
            resourceDO.setResourceId(existResource.getResourceId());
            resourceDO.setResourceCount(existResource.getResourceCount() + resourceDataDOS.size());
            if (resourceManager.modifyResource(resourceDO) < 1) {
                throw new BizException("上传资源数据失败");
            }
        } else {
            log.info("上传文件为空,数据集Id: {}, 名称: {}", existResource.getResourceId(), existResource.getName());
        }

    }

    /**
     * 修改资源状态
     *
     * @param importResourceDTO 参数
     */
    @Override
    @Transactional
    public void importResourceV2(ImportResourceDTO importResourceDTO) {
        ResourceDO existResource = resourceManager.queryById(importResourceDTO.getResourceId());
        //校验资源是否存在
        if (existResource == null || SysUserStatusEnum.FROZEN.getValue() == existResource.getStatus()) {
            throw new BizException("资源包不存在或者已作废");
        }
        Integer resourceCount = resourceManager.queryResourceCount(importResourceDTO.getResourceId());
        ResourceDO resourceDO = new ResourceDO();
        resourceDO.setId(existResource.getId());
        resourceDO.setResourceId(existResource.getResourceId());
        resourceDO.setResourceCount(resourceCount);
        if (resourceManager.modifyResource(resourceDO) < 1) {
            throw new BizException("上传资源数据失败");
        }
        resourceManager.updateDraftByResourceId(importResourceDTO.getResourceId());

    }

    /**
     * 分页查询资源列表
     *
     * @param queryResourceDTO 查询条件
     * @return PageResult<ResourcePageItemDTO>
     */
    @Override
    public PageResult<ResourcePageItemVO> getResourcePage(QueryResourceDTO queryResourceDTO) {
        // 查询当前用户信息
        startPage(queryResourceDTO.getPageNum(), queryResourceDTO.getPageSize());
        Page<ResourceBO> resourceBOS = resourceManager.queryResourcePage(queryResourceDTO);
        Page<ResourcePageItemVO> page = new Page<>();
        if (isNotEmpty(resourceBOS)) {
            resourceBOS.forEach(item -> {
                ResourcePageItemVO result = resourceMapStruct.convertToPageItem(item);
                result.setTypeName(ResourceTypeEnum.getName(item.getType()));
                result.setStatusName(SysUserStatusEnum.getName(item.getStatus()));
                result.setCreateTimeStr(DateUtil.format(item.getCreateTime(), Constant.DATETIME_FORMAT));
                result.setAssignStatusName(ResourceAssignStatusEnum.getName(item.getAssignStatus()));
                page.add(result);
            });
        }
        return PageResult.of(resourceBOS.getTotal(), page);
    }

    /**
     * 查看资源基本信息
     *
     * @param resourceId 资源id
     * @return ResourceBaseInfoVO
     */
    @Override
    public ResourceBaseInfoVO getBaseInfo(Long resourceId) {
        ResourceDO resourceDO = resourceManager.queryById(resourceId);
        ResourceBaseInfoVO baseInfoVO = new ResourceBaseInfoVO();
        baseInfoVO.setResourceId(resourceId);
        baseInfoVO.setName(resourceDO.getName());
        baseInfoVO.setType(resourceDO.getType());
        baseInfoVO.setTypeName(ResourceTypeEnum.getName(resourceDO.getType()));
        return baseInfoVO;
    }

    /**
     * 删除(作废)资源
     *
     * @param resourceId 资源id
     */
    @Override
    @Transactional
    public void removeResource(Long resourceId) {
        ResourceDO existResource = resourceManager.queryById(resourceId);
        if (existResource == null) {
            throw new BizException("资源不存在");
        }
        if (resourceManager.removeResource(existResource.getId()) < 1) {
            throw new BizException("删除资源失败,数据集名称: " + existResource.getName());
        }
        logEventPublisher.publish(Log.builder().append("删除资源数据集,ID:" + existResource.getResourceId() + ",名称:" + existResource.getName()).isUpdate());


    }

    /**
     * 查询查看待选资源列表
     *
     * @param projectId 项目id
     * @return List<ResourceBaseInfoVO>
     */
    @Override
    public List<ResourceBaseInfoVO> getNotAssignedList(Long projectId) {
        List<ResourceDO> resourceDOS = resourceManager.queryNotAssignedList();
        List<ResourceBaseInfoVO> list = new ArrayList<>();
        if (isNotEmpty(resourceDOS)) {
            list = resourceDOS.stream().filter(item -> item.getResourceCount() > 0).map(item -> {
                ResourceBaseInfoVO baseInfoVO = new ResourceBaseInfoVO();
                baseInfoVO.setResourceId(item.getResourceId());
                baseInfoVO.setName(item.getName());
                baseInfoVO.setType(item.getType());
                baseInfoVO.setTypeName(ResourceTypeEnum.getName(item.getType()));
                return baseInfoVO;
            }).collect(Collectors.toList());
        }
        if (projectId != null) {
            List<Long> ids = projectManager.queryProResListByProId(projectId).stream().map(ProjectResourceDO::getResourceId).collect(Collectors.toList());
            if (isNotEmpty(ids)) {
                List<ResourceDO> selectedResources = resourceManager.queryListByIds(ids);
                for (ResourceDO item : selectedResources) {
                    ResourceBaseInfoVO baseInfoVO = new ResourceBaseInfoVO();
                    baseInfoVO.setResourceId(item.getResourceId());
                    baseInfoVO.setName(item.getName());
                    baseInfoVO.setType(item.getType());
                    baseInfoVO.setTypeName(ResourceTypeEnum.getName(item.getType()));
                    list.add(baseInfoVO);
                }
            }
        }
        return list;
    }

    /**
     * 获取上传文件后缀名
     *
     * @param uploadName 文件名
     * @return String
     */
    public String getUploadSuffix(String uploadName) {
        return uploadName.substring(uploadName.lastIndexOf(".") + 1);
    }

    /**
     * 判断是否文本
     *
     * @param suffix 后缀
     * @return boolean
     */
    private boolean isJson(String suffix) {
        Pattern p = Pattern.compile("\\.(json|txt)");
        Matcher m = p.matcher(suffix);
        return m.find();
    }

    /**
     * 判断是否文本
     *
     * @param suffix 后缀
     * @return boolean
     */
    private boolean isExcel(String suffix) {
        Pattern p = Pattern.compile("\\.(xls|xlsx)");
        Matcher m = p.matcher(suffix);
        return m.find();
    }

    /**
     * 解析json内容(文本，图片，视频)
     *
     * @param uploadFile 上传文件
     * @param resourceId 资源id
     * @param type       类型
     * @return {@link List}<{@link ResourceDataDO}>
     */
    private List<ResourceDataBO> parseJsonContent(MultipartFile uploadFile, Long resourceId, Integer type) {
        String suffix = "." + getUploadSuffix(Objects.requireNonNull(uploadFile.getOriginalFilename()));
        if (!isJson(suffix)) {
            throw new BizException("只能上传json或txt后缀的文本文件");
        }
        List<ResourceDataBO> resourceDataList = new ArrayList<>();
        LocalDateTime now = LocalDateTime.now();
        try (InputStreamReader read = new InputStreamReader(new ByteArrayInputStream(uploadFile.getBytes()));
             BufferedReader br = new BufferedReader(read)) {
            String s;
            while ((s = br.readLine()) != null) {
                ResourceDataDO dataDO = new ResourceDataDO();
                dataDO.setResourceId(resourceId);
                dataDO.setDataId(snowflakeIdFactory.nextId());
                dataDO.setCreateTime(now);
                dataDO.setIsDraft(IsOrNotEnum.IS.getValue());
                ResourceDataBO resourceDataBO = ResourceDataBO.builder().resourceData(dataDO).isValid(false).build();
                try {
                    Map map = (Map) JSONUtil.parse(s.trim());
                    Map dataMap = (Map) map.get("data");
                    if (dataMap != null) {
                        Object hint = dataMap.get("hint");
                        if (hint != null) {
                            dataDO.setHint(hint.toString().trim());
                        }
                        if (ResourceTypeEnum.DOC.getValue().equals(type)) {
                            Object txtSource = dataMap.get("txtsource");
                            if (txtSource != null) {
                                dataDO.setContent(txtSource.toString().trim());
                            }
                        } else if (ResourceTypeEnum.PICTURE.getValue().equals(type)) {
                            Object picSource = dataMap.get("picsource");
                            if (picSource != null) {
                                dataDO.setPicUrl(picSource.toString().trim());
                            }
                        } else {
                            Object movSource = dataMap.get("movsource");
                            if (movSource != null) {
                                dataDO.setMovUrl(movSource.toString().trim());
                            }
                        }
                    }
                    resourceDataBO.setIsValid(true);
                    resourceDataList.add(resourceDataBO);
                } catch (JSONException jsonException) {
                    //解析失败不用理会
                    log.error("资源文件解析失败: ", jsonException);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return resourceDataList;
    }

    /**
     * 解析excel内容(链接)
     *
     * @param uploadFile 上传文件
     * @param resourceId 资源id
     * @param type       类型
     * @return {@link List}<{@link ResourceDataDO}>
     */
    private List<ResourceDataBO> parseExcelContent(MultipartFile uploadFile, Long resourceId, Integer type) {
        String suffix = "." + getUploadSuffix(Objects.requireNonNull(uploadFile.getOriginalFilename()));
        if (!isExcel(suffix)) {
            throw new BizException("只能上传excel文件");
        }
        List<ResourceDataBO> resourceDataList = new ArrayList<>();
        LocalDateTime now = LocalDateTime.now();

        try (XSSFWorkbook excel = new XSSFWorkbook(uploadFile.getInputStream())) {
            // 读取且仅仅读取第一个sheet
            XSSFSheet sheet = excel.getSheetAt(0);
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                ResourceDataDO dataDO = new ResourceDataDO();
                dataDO.setResourceId(resourceId);
                dataDO.setDataId(snowflakeIdFactory.nextId());
                dataDO.setCreateTime(now);
                dataDO.setIsDraft(IsOrNotEnum.IS.getValue());
                ResourceDataBO resourceDataBO = ResourceDataBO.builder().resourceData(dataDO).isValid(false).build();
                // 从第二行开始读取，第一行是头部
                XSSFRow row = sheet.getRow(i);
                // 读取第1列
                XSSFCell cell = row.getCell(0);
                String rawValue = cell.getStringCellValue();
                if (StringUtils.isNotBlank(rawValue)) {
                    dataDO.setContent(rawValue);
                    resourceDataBO.setIsValid(true);
                }
                resourceDataList.add(resourceDataBO);
            }
        } catch (IOException e) {
            // 解析失败不用理会
            log.error("资源文件解析失败: ", e);
        }
        return resourceDataList;
    }

}
