package cn.sino.mark.service.impl;

import cn.sino.common.exception.BizException;
import cn.sino.mark.common.enums.DeleteStatusEnum;
import cn.sino.mark.common.enums.MoveNodeTypeEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.bo.SysPermBO;
import cn.sino.mark.common.model.dto.AddPermReqDTO;
import cn.sino.mark.common.model.dto.DeletePermReqDTO;
import cn.sino.mark.common.model.dto.EditPermReqDTO;
import cn.sino.mark.common.model.vo.PermResDTO;
import cn.sino.mark.common.model.vo.SysPermInfoResDTO;
import cn.sino.mark.common.utils.CompareHelper;
import cn.sino.mark.common.utils.LoginUserUtil;
import cn.sino.mark.common.utils.UUIDHelper;
import cn.sino.mark.dao.mapper.SysPermMapper;
import cn.sino.mark.dao.mapper.SysRolePermMapper;
import cn.sino.mark.dao.model.SysPermDO;
import cn.sino.mark.dao.model.SysRolePermDO;
import cn.sino.mark.service.SysPermService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author shichanghao
 * @description 针对表【t_sys_perm(系统权限表)】的数据库操作Service实现
 * @createDate 2022-03-29 23:56:35
 */
@Service
public class SysPermServiceImpl implements SysPermService {
    private final SysPermMapper sysPermMapper;
    private final SysRolePermMapper sysRolePermMapper;
    private final SysLogEventPublisher logEventPublisher;

    public SysPermServiceImpl(SysPermMapper sysPermMapper,
                              SysRolePermMapper sysRolePermMapper,
                              SysLogEventPublisher logEventPublisher) {
        this.sysPermMapper = sysPermMapper;
        this.sysRolePermMapper = sysRolePermMapper;
        this.logEventPublisher = logEventPublisher;
    }

    @Override
    @Transactional
    public void addPerm(AddPermReqDTO addPermReqDTO) {
        //1、根据权限名称进行查询
        addPermReqDTO.setPermCode(addPermReqDTO.getPermCode().trim());
        SysPermDO sysPerm = sysPermMapper.selectByPermCode(addPermReqDTO.getPermCode());
        if (sysPerm != null) {
            throw new BizException("权限已存在");
        }
        Date currentDate = new Date();
        //2、向数据库中加入新权限
        sysPerm = new SysPermDO();
        BeanUtils.copyProperties(addPermReqDTO, sysPerm);
        sysPerm.setPermId(UUIDHelper.getInstance().nextId());
        sysPerm.setIsDelete(DeleteStatusEnum.NOT_DELETE.getValue());
        sysPerm.setCreateTime(LocalDateTime.now());
        sysPerm.setSort(currentDate.getTime());//用时间戳当排序
        sysPerm.setCreateBy(LoginUserUtil.getLoginUserId());
        sysPerm.setUpdateBy(LoginUserUtil.getLoginUserId());
        sysPermMapper.insert(sysPerm);
        logEventPublisher.publish(Log.builder().append("新增权限,ID:" + sysPerm.getPermId() + ",名称:" + sysPerm.getPermName()).isInsert());
    }

    @Override
    public SysPermInfoResDTO getSysPermInfo(Long permId) {
        SysPermInfoResDTO sysPermInfoResDTO = new SysPermInfoResDTO();
        SysPermDO sysPerm = sysPermMapper.selectById(permId);
        if (sysPerm == null) {
            throw new BizException("权限不存在");
        }
        BeanUtils.copyProperties(sysPerm, sysPermInfoResDTO);
        return sysPermInfoResDTO;
    }

    @Override
    @Transactional
    public void deleteById(Long permId) {
        SysPermDO sysPerm = sysPermMapper.selectById(permId);
        if (sysPerm == null) {
            throw new BizException("权限不存在");
        }
        //如果子与角色有关联，父肯定与角色有关联
        //1、根据权限id查询角色权限关系
        List<SysRolePermDO> sysRolePermList = sysRolePermMapper.selectByPermId(permId);
        if (!CollectionUtils.isEmpty(sysRolePermList)) {
            throw new BizException("有角色正在使用该权限,不能删除");
        }
        List<Long> permIdList = sysPermMapper.selectPermIdsByParentId(permId);
        if (!CollectionUtils.isEmpty(permIdList)) {
            throw new BizException("此权限下拥有子权限,不能删除");
        }

        //2、做逻辑删除
        if (sysPermMapper.deleteById(permId) <= 0) {
            throw new BizException("删除权限失败");
        }

        //3、写入日志
        logEventPublisher.publish(Log.builder().append("删除权限,ID:" + permId + ",名称:" + sysPerm.getPermName()).isDelete());
    }

    @Override
    @Transactional
    public void editPerm(EditPermReqDTO editPermReqDTO) {
        //根据权限名称查询
        SysPermDO sysPerm = sysPermMapper.selectByPermCode(editPermReqDTO.getPermCode());
        if (sysPerm != null && sysPerm.getPermId() != editPermReqDTO.getPermId().longValue()) {
            //权限已存在
            throw new BizException("权限已存在");
        }
        SysPermDO perm = sysPermMapper.selectById(editPermReqDTO.getPermId());
        if (perm == null) {
            //权限不存在
            throw new BizException("权限不存在");
        }
        sysPerm = new SysPermDO();
        BeanUtils.copyProperties(editPermReqDTO, sysPerm);
        sysPerm.setSort(perm.getSort());
        if (sysPermMapper.update(sysPerm) <= 0) {
            throw new BizException("修改失败");
        }
        logEventPublisher.publish(Log.builder().append("修改权限,ID:" + editPermReqDTO.getPermId() + ",名称:" + perm.getPermName() + "\n")
                .append(new CompareHelper<SysPermDO>().compareObject(perm, sysPerm)).isUpdate());
    }

    @Override
    public PermResDTO getPermById(Long permId) {
        PermResDTO permResDTO = new PermResDTO();
        SysPermDO sysPerm = sysPermMapper.selectById(permId);
        if (sysPerm != null) {
            BeanUtils.copyProperties(sysPerm, permResDTO);
        }
        return permResDTO;
    }

    @Override
    @Transactional
    public void movePermNode(Long id, int type) {
        SysPermDO perm = sysPermMapper.selectById(id);
        if (perm == null) {
            throw new BizException("权限不存在");
        }
        Long parentId = perm.getParentId();
        List<SysPermDO> childrenList = sysPermMapper.selectChildrenPermsById(parentId);
        Log.Builder builder = Log.builder();
        //上移
        if (type == MoveNodeTypeEnum.UP_NODE.getValue()) {
            for (int i = 0; i < childrenList.size(); i++) {
                if (childrenList.get(i).getPermId().equals(perm.getPermId())) {
                    if (i == 0) {
                        throw new BizException("菜单上移失败,已经在最上面");
                    }
                    //上一个 对象
                    SysPermDO prve = childrenList.get(i - 1);
                    //交换sort字段，完成移动操作
                    Long prveosrt = prve.getSort();
                    Long currentSort = perm.getSort();
                    perm.setSort(prveosrt);
                    prve.setSort(currentSort);
                    sysPermMapper.update(perm);
                    SysPermDO sysPerm = new SysPermDO();
                    BeanUtils.copyProperties(prve, sysPerm);
                    sysPermMapper.update(sysPerm);
                    builder.append(String.format("上移位置,ID:%s,名称:%s", id, perm.getPermName()));
                }
            }
        } else if (type == MoveNodeTypeEnum.DOWN_NODE.getValue()) {//下移
            for (int i = 0; i < childrenList.size(); i++) {
                if (childrenList.get(i).getPermId().equals(perm.getPermId())) {
                    if (i == childrenList.size() - 1) {
                        throw new BizException("菜单下移失败,已经在最下面");
                    }
                    SysPermDO nextObj = childrenList.get(i + 1);
                    Long currentSort = perm.getSort();
                    Long nextObjSort = nextObj.getSort();
                    perm.setSort(nextObjSort);
                    nextObj.setSort(currentSort);
                    sysPermMapper.update(perm);
                    SysPermDO sysPerm = new SysPermDO();
                    BeanUtils.copyProperties(nextObj, sysPerm);
                    sysPermMapper.update(sysPerm);
                    builder.append(String.format("下移位置,ID:%s,名称:%s", id, perm.getPermName()));
                }
            }
        } else {
            throw new BizException("无效的菜单移动方向参数" + ":" + type);
        }
        logEventPublisher.publish(builder.isUpdate());
    }

    /**
     * 构建树
     *
     * @param permBOList
     */
    private List<PermResDTO> buildPermTree(List<SysPermBO> permBOList) {
        List<PermResDTO> permResDTOS = new ArrayList<>();
        if (!CollectionUtils.isEmpty(permBOList)) {
            for (SysPermBO permBO : permBOList) {
                PermResDTO permResDTO = new PermResDTO();
                BeanUtils.copyProperties(permBO, permResDTO);
                permResDTO.setChildren(new ArrayList<>());
                permResDTO.setChildren(buildPermTree(permBO.getChildren()));
                permResDTOS.add(permResDTO);
            }
        }
        return permResDTOS;

    }

    @Override
    public List<PermResDTO> getAllMenuList() {
        List<SysPermBO> permList = sysPermMapper.selectMenuTreeData();
        return buildPermTree(permList);
    }

    @Override
    public List<PermResDTO> getPermsByMenuId(Long menuId) {
        List<PermResDTO> permResDTOList = new ArrayList<>();
        List<SysPermDO> permList = sysPermMapper.selectPermsByMenuId(menuId);
        buildDTO(permList, permResDTOList);
        return permResDTOList;
    }

    @Override
    @Transactional
    public void deleteMoreByIds(DeletePermReqDTO permIds) {
        //判断list  里的权限是否有 角色关联
        for (Long permId : permIds.getPermIds()) {
            List<SysRolePermDO> sysRolePermList = sysRolePermMapper.selectByPermId(permId);
            if (!CollectionUtils.isEmpty(sysRolePermList)) {
                PermResDTO permResDTO = getPermById(sysRolePermList.get(0).getPermId());
                throw new BizException(String.format("有角色正在使用【%s】权限,不能删除", permResDTO.getPermName()));
            }
        }
        //判断是否有子权限
        for (Long permId : permIds.getPermIds()) {
            List<Long> permIdList = sysPermMapper.selectPermIdsByParentId(permId);
            if (!CollectionUtils.isEmpty(permIdList)) {
                PermResDTO permResDTO = getPermById(permId);
                throw new BizException(String.format("【%s】权限下有子权限,不能删除", permResDTO.getPermName()));
            }
        }

        Log.Builder delete = Log.builder();
        delete.append("删除权限,");
        permIds.getPermIds().forEach(permId -> {
            SysPermDO sysPerm = sysPermMapper.selectById(permId);
            if (sysPerm != null) {
                delete.append("ID:" + sysPerm.getPermId() + ",名称:" + sysPerm.getPermName() + "\n");
            }
        });
        //批量删除
        sysPermMapper.deleteMoreByPermIds(permIds.getPermIds());
        logEventPublisher.publish(delete.isDelete());

    }

    @Override
    public List<PermResDTO> getNotDefaultPerm() {
        List<SysPermDO> permList = sysPermMapper.selectAllNotDefaultPerm();
        List<PermResDTO> list = new ArrayList<>();
        permList.stream().forEach(per -> {
            PermResDTO permResDTO = new PermResDTO();
            BeanUtils.copyProperties(per, permResDTO);
            list.add(permResDTO);
        });
        return list;
    }


    /**
     * 将数据封装到DTO中
     *
     * @param permList
     * @param permResDTOList
     */
    private void buildDTO(List<SysPermDO> permList, List<PermResDTO> permResDTOList) {
        if (!CollectionUtils.isEmpty(permList)) {
            permList.stream().forEach(sysPerm -> {
                PermResDTO permResDTO = new PermResDTO();
                BeanUtils.copyProperties(sysPerm, permResDTO);
                permResDTOList.add(permResDTO);
            });
        }
    }

}




