package cn.sino.mark.service.impl;

import cn.sino.common.exception.BizException;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.enums.DeleteStatusEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.dto.AddRoleReqDTO;
import cn.sino.mark.common.model.dto.AssignPermsReqDTO;
import cn.sino.mark.common.model.dto.EditRoleReqDTO;
import cn.sino.mark.common.model.dto.SysRolePageQueryReqDTO;
import cn.sino.mark.common.model.vo.PermResDTO;
import cn.sino.mark.common.model.vo.RoleResDTO;
import cn.sino.mark.common.model.vo.SysRoleInfoVO;
import cn.sino.mark.common.utils.CompareHelper;
import cn.sino.mark.common.utils.UUIDHelper;
import cn.sino.mark.dao.mapper.SysPermMapper;
import cn.sino.mark.dao.mapper.SysRoleMapper;
import cn.sino.mark.dao.mapper.SysRolePermMapper;
import cn.sino.mark.dao.mapper.SysUserRoleMapper;
import cn.sino.mark.dao.model.SysPermDO;
import cn.sino.mark.dao.model.SysRoleDO;
import cn.sino.mark.dao.model.SysRolePermDO;
import cn.sino.mark.dao.model.SysUserRoleDO;
import cn.sino.mark.service.SysPermService;
import cn.sino.mark.service.SysRoleService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author shichanghao
 * @description 针对表【t_sys_role(系统角色表)】的数据库操作Service实现
 * @createDate 2022-03-28 22:55:10
 */
@Service
public class SysRoleServiceImpl implements SysRoleService {
    private final SysRoleMapper sysRoleMapper;
    private final SysRolePermMapper sysRolePermMapper;
    private final SysPermMapper sysPermMapper;
    private final SysUserRoleMapper sysUserRoleMapper;
    private final SysPermService sysPermService;
    /**
     * 国际化
     */
    private final SysLogEventPublisher logEventPublisher;

    public SysRoleServiceImpl(SysRoleMapper sysRoleMapper,
                              SysRolePermMapper sysRolePermMapper,
                              SysPermMapper sysPermMapper,
                              SysUserRoleMapper sysUserRoleMapper,
                              SysPermService sysPermService,
                              SysLogEventPublisher logEventPublisher) {
        this.sysRoleMapper = sysRoleMapper;
        this.sysRolePermMapper = sysRolePermMapper;
        this.sysPermMapper = sysPermMapper;
        this.sysUserRoleMapper = sysUserRoleMapper;
        this.sysPermService = sysPermService;
        this.logEventPublisher = logEventPublisher;
    }


    @Override
    public PageInfo<RoleResDTO> getRolePage(SysRolePageQueryReqDTO roleListReqDTO) {
        PageHelper.startPage(roleListReqDTO.getPageNum() == null || roleListReqDTO.getPageNum() <= 0 ? 1 : roleListReqDTO.getPageNum(), Constant.DEFAULT_PAGE_SIZE);
        Page<SysRoleDO> sysRolePage = (Page<SysRoleDO>) sysRoleMapper.selectPage(roleListReqDTO);
        Page<RoleResDTO> rolePage = new Page<>();
        BeanUtils.copyProperties(sysRolePage, rolePage);
        sysRolePage.stream().forEach(sysRole -> {
            RoleResDTO roleResDTO = new RoleResDTO();
            BeanUtils.copyProperties(sysRole, roleResDTO);
            rolePage.add(roleResDTO);
        });
        return new PageInfo<>(rolePage);
    }

    @Override
    @Transactional
    public void addRole(AddRoleReqDTO addRoleReqDTO) {
        //1、根据角色名称查询
        SysRoleDO sysRole = sysRoleMapper.selectByRoleName(addRoleReqDTO.getRoleName());
        if (null != sysRole) {
            throw new BizException("角色已存在");
        }
        //2、向数据库中加入新角色
        sysRole = new SysRoleDO();
        Long roleId = UUIDHelper.getInstance().nextId();
        sysRole.setRoleId(roleId);
        sysRole.setRoleName(addRoleReqDTO.getRoleName());
        sysRole.setRoleCode(UUID.randomUUID().toString().replace("-", ""));
        sysRole.setDescription(addRoleReqDTO.getDescription());
        sysRole.setIsDelete(DeleteStatusEnum.NOT_DELETE.getValue());
        sysRole.setCreateTime(LocalDateTime.now());
        sysRole.setUpdateTime(LocalDateTime.now());
        sysRoleMapper.insert(sysRole);
        logEventPublisher.publish(Log.builder().append("添加角色,ID:" + roleId + ",名称:" + addRoleReqDTO.getRoleName()).isInsert());
    }

    @Override
    @Transactional
    public void editRole(EditRoleReqDTO editRoleReqDTO) {
        //查询名字是否重复
        SysRoleDO role = sysRoleMapper.selectByRoleName(editRoleReqDTO.getRoleName());
        if (null != role && !role.getRoleId().equals(editRoleReqDTO.getRoleId())) {
            throw new BizException("角色已存在");
        }
        //查询就得角色
        SysRoleDO oldRole = sysRoleMapper.selectById(editRoleReqDTO.getRoleId());

        SysRoleDO sysRole = new SysRoleDO();
        sysRole.setRoleId(editRoleReqDTO.getRoleId());
        sysRole.setRoleName(editRoleReqDTO.getRoleName());
        sysRole.setDescription(editRoleReqDTO.getDescription());
        if (sysRoleMapper.updateById(sysRole) <= 0) {
            throw new BizException("编辑失败");
        }

        Log.Builder builder = Log.builder();
        builder.append("更新角色,ID:" + editRoleReqDTO.getRoleId() + ",名称:" + oldRole.getRoleName() + "\n");
        builder.append(new CompareHelper<SysRoleDO>().compareObject(oldRole, sysRole));
        logEventPublisher.publish(builder.isUpdate());
    }

    @Override
    @Transactional
    public void deleteById(Long roleId) {
        //1、如果该角色是超级管理员，则不能删除
        SysRoleDO sysRole = sysRoleMapper.selectById(roleId);
        if (sysRole == null) {
            throw new BizException("角色不存在");
        }
        if (sysRole.getRoleCode().equals(Constant.ADMIN_ROLE)) {
            throw new BizException("不能删除超级管理员角色");
        }

        //2、判断该角色下是否有用户
        List<SysUserRoleDO> sysUserRoleList = sysUserRoleMapper.selectByRoleId(roleId);
        if (!CollectionUtils.isEmpty(sysUserRoleList)) {
            throw new BizException("有用户正在使用该角色,不能删除.");
        }

        //3、逻辑删除角色
        if (sysRoleMapper.deleteById(roleId) <= 0) {
            throw new BizException("删除失败");
        }

        // 记录日志
        logEventPublisher.publish(Log.builder().append("删除角色,ID:" + roleId + ",名称:" + sysRole.getRoleName()).isDelete());
    }

    @Override
    public void addRolePerms(AssignPermsReqDTO assignPermsReqDTO) {
        SysRoleDO sysRole = sysRoleMapper.selectById(assignPermsReqDTO.getRoleId());
        if (sysRole == null) {
            throw new BizException("角色不存在");
        }
        //1、先删除该角色的角色权限关系
        sysRolePermMapper.deleteByRoleId(sysRole.getRoleId());

        //根据角色id，查询名称 用于写入日志
        Log.Builder builder = Log.builder();
        builder.append("给角色：");
        builder.append(sysRole.getRoleName());
        builder.append("分配了权限: \n");
        //2、批量插入新的角色权限关系
        Long[] permIds = assignPermsReqDTO.getPermIds();
        if (permIds == null || permIds.length == 0) {
            return;
        }
        List<SysRolePermDO> sysRolePermList = new ArrayList<>();
        for (Long permId : permIds) {
            SysRolePermDO sysRolePerm = new SysRolePermDO();
            sysRolePerm.setRoleId(sysRole.getRoleId());
            sysRolePerm.setPermId(permId);
            sysRolePermList.add(sysRolePerm);
            PermResDTO permResDTO = sysPermService.getPermById(permId);
            builder.append(permResDTO.getPermName() + ",");
        }
        if (!sysRolePermList.isEmpty()) {
            sysRolePermMapper.insertBatch(sysRolePermList);
        }

        // 记录日志
        logEventPublisher.publish(builder.isUpdate());
    }


    @Override
    public List<PermResDTO> getPermsByRoleId(Long roleId) {
        List<PermResDTO> permResDTOList = new ArrayList<>();
        SysRoleDO sysRole = sysRoleMapper.selectById(roleId);
        if (sysRole == null) {
            throw new BizException("角色不存在");
        }
        List<SysPermDO> perms = sysPermMapper.selectPermsByRoleId(roleId);
        perms.stream().forEach(p -> {
            PermResDTO permResDTO = new PermResDTO();
            BeanUtils.copyProperties(p, permResDTO);
            permResDTOList.add(permResDTO);
        });
        return permResDTOList;
    }

    @Override
    public RoleResDTO getRoleById(Long roleId) {
        RoleResDTO roleResDTO = new RoleResDTO();
        SysRoleDO sysRole = sysRoleMapper.selectById(roleId);
        if (sysRole == null) {
            throw new BizException("角色不存在");
        }
        BeanUtils.copyProperties(sysRole, roleResDTO);
        return roleResDTO;
    }

    @Override
    public List<SysRoleInfoVO> getAllRole() {
        //只有超级管理员用户才能看见超级管理员角色
        List<SysRoleDO> sysRoleList = sysRoleMapper.selectAllRole();
        List<SysRoleInfoVO> listResDTOS = new ArrayList<>();
        sysRoleList.forEach(role -> {
            SysRoleInfoVO sysRoleInfo = new SysRoleInfoVO();
            sysRoleInfo.setRoleId(role.getRoleId());
            sysRoleInfo.setRoleName(role.getRoleName());
            listResDTOS.add(sysRoleInfo);
        });
        return listResDTOS;
    }

}




