package cn.sino.mark.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.json.JSONUtil;
import cn.sino.common.base.PageResult;
import cn.sino.common.exception.BizException;
import cn.sino.common.util.SnowflakeIdFactory;
import cn.sino.mark.common.constants.Constant;
import cn.sino.mark.common.enums.IsOrNotEnum;
import cn.sino.mark.common.enums.SysRoleEnums;
import cn.sino.mark.common.enums.SysUserStatusEnum;
import cn.sino.mark.common.enums.SysUserTypeEnum;
import cn.sino.mark.common.log.Log;
import cn.sino.mark.common.log.SysLogEventPublisher;
import cn.sino.mark.common.model.bo.SysUserBO;
import cn.sino.mark.common.model.dto.*;
import cn.sino.mark.common.model.vo.*;
import cn.sino.mark.common.properties.InnerUserProperties;
import cn.sino.mark.common.properties.JwtProperties;
import cn.sino.mark.common.utils.HttpUtil;
import cn.sino.mark.common.utils.LoginUserUtil;
import cn.sino.mark.dao.mapper.SysRoleMapper;
import cn.sino.mark.dao.mapper.SysUserRoleMapper;
import cn.sino.mark.dao.model.*;
import cn.sino.mark.manager.ProjectManager;
import cn.sino.mark.manager.SysUserManager;
import cn.sino.mark.service.SysUserService;
import cn.sino.mark.service.convert.SysUserMapStruct;
import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.github.pagehelper.Page;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static cn.hutool.core.text.CharSequenceUtil.*;
import static com.github.pagehelper.page.PageMethod.startPage;

/**
 * @author zhangamin
 * @description 后台管理系统用户业务接口实现
 * @className cn.sino.mark.service.impl.SysUserServiceImpl
 * @date 2022/3/17
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysUserServiceImpl implements SysUserService, UserDetailsService {

    private final SysUserMapStruct sysUserMapStruct;
    private final SysUserManager sysUserManager;
    private final StringRedisTemplate stringRedisTemplate;
    private final JwtProperties jwtProperties;
    private final SnowflakeIdFactory snowflakeIdFactory;
    private final PasswordEncoder passwordEncoder;
    private final SysLogEventPublisher logEventPublisher;
    private final SysUserRoleMapper sysUserRoleMapper;
    private final SysRoleMapper sysRoleMapper;
    private final ProjectManager projectManager;
    private final InnerUserProperties innerUserProperties;

    private static final String USER_NULL = "用户不存在";


    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        // 后台管理系统用户登录账户
        SysUserDO sysUser = sysUserManager.queryByAccount(username);
        if (sysUser == null) {
            throw new UsernameNotFoundException(username);
        }
        if (sysUser.getUserType().equals(SysUserTypeEnum.INNER_EMPLOYEE.getValue())) {
            throw new BizException("内部用户不能使用外部登录");
        }
        return sysUserMapStruct.convertToSysUserDetails(sysUser);
    }

    @Override
    public String createUserLoginToken(String account) {
        // 后台管理系统用户登录账户
        SysUserDO sysUser = sysUserManager.queryByAccount(account);
        if (sysUser == null) {
            throw new BizException("用户账户不存在");
        }

        // 查询用户角色
        List<String> sysRoles = List.of(Constant.ADMIN_ROLE);
        //查询用户权限
        List<String> sysPerms = List.of();

        // 利用随机字符串来生成token
        String token = UUID.randomUUID().toString();
        LoginUserDTO loginUser = sysUserMapStruct.convertToLoginUserDTO(sysUser);
        loginUser.setLoginTime(Instant.now().getEpochSecond());
        loginUser.setUuid(token);
        try {
            String jwt = createJwt(token);
            String loginKey = Constant.ADMIN_LOGIN_KEY + token;
            stringRedisTemplate.opsForHash().put(loginKey, Constant.LOGIN_USER_INFO, JSONUtil.toJsonStr(loginUser));
            stringRedisTemplate.opsForHash().put(loginKey, Constant.LOGIN_USER_ROLES, JSONUtil.toJsonStr(sysRoles));
            stringRedisTemplate.opsForHash().put(loginKey, Constant.LOGIN_USER_PERMS, JSONUtil.toJsonStr(sysPerms));
            stringRedisTemplate.expire(loginKey, 8, TimeUnit.HOURS);
            //记录日志
            log.info(String.format("%s(%s)登录成功", sysUser.getUserName(), sysUser.getId()));
            return jwt;
        } catch (Exception e) {
            log.error("生成jwt失败:", e);
            throw new BizException("登录失败");
        }
    }

    @Override
    @Transactional
    public void addOuterUser(AddUserDTO addUserDTO) {
        SysUserDO existUser = sysUserManager.queryByAccount(addUserDTO.getAccount().trim());
        if (existUser != null) {
            throw new BizException("用户账户已存在");
        }
        if (!addUserDTO.getPassword().trim().equals(addUserDTO.getConfirmedPassword().trim())) {
            throw new BizException("“密码”和“确认密码”不一致");
        }

        SysUserDO sysUserDO = new SysUserDO();
        sysUserDO.setUserId(snowflakeIdFactory.nextId());
        sysUserDO.setAccount(addUserDTO.getAccount().trim());
        sysUserDO.setPassword(passwordEncoder.encode(addUserDTO.getPassword().trim()));
        sysUserDO.setUserName(addUserDTO.getUserName().trim());
        sysUserDO.setStatus(SysUserStatusEnum.NORMAL.getValue());
        sysUserDO.setUserType(SysUserTypeEnum.OUTER_EMPLOYEE.getValue());
        sysUserManager.addUser(sysUserDO);
        SysUserRoleDO userRoleDO = new SysUserRoleDO();
        userRoleDO.setRoleId(Long.valueOf(SysRoleEnums.OUTER_MARKER.getValue()));
        userRoleDO.setUserId(sysUserDO.getUserId());
        userRoleDO.setCreateTime(LocalDateTime.now());
        sysUserManager.addUserRole(userRoleDO);
        logEventPublisher.publish(Log.builder().append("添加外部用户,ID:" + sysUserDO.getUserId() + ",账号:" + sysUserDO.getAccount()).isInsert());

    }

    @Override
    public UserDetailsVO getOuterUserDetails(Long userId) {
        SysUserDO sysUserDO = sysUserManager.queryByUserId(userId);
        if (sysUserDO == null) {
            throw new BizException("外部用户不存在");
        }
        List<SysRoleDO> sysRoleDOS = sysUserManager.queryRolesByUserId(userId);
        UserDetailsVO userDetailsVO = sysUserMapStruct.convertToUserDetailsVO(sysUserDO);
        if (isNotEmpty(sysRoleDOS)) {
            userDetailsVO.setRoleIds(
                    sysRoleDOS.stream().map(SysRoleDO::getRoleId).collect(Collectors.toList())
            );
        } else {
            userDetailsVO.setRoleIds(Collections.emptyList());
        }
        return userDetailsVO;
    }

    @Override
    @Transactional
    public void editOuterUser(EditUserDTO editUserDTO, Long userId) {
        SysUserDO existUser = sysUserManager.queryByUserId(userId);
        if (existUser == null) {
            throw new BizException("外部用户不存在");
        }
        List<SysRoleDO> sysRoleDOS = sysUserManager.queryRolesByUserId(userId);
        if (sysRoleDOS.stream().anyMatch(item -> item.getRoleId().intValue() == SysRoleEnums.ADMIN.getValue())) {
            throw new BizException("超级管理员不可修改");
        }
        SysUserDO sysUserDO = new SysUserDO();
        sysUserDO.setAccount(existUser.getAccount());
        sysUserDO.setUserId(userId);
        sysUserDO.setUserName(editUserDTO.getUserName().trim());
        sysUserDO.setId(existUser.getId());
        String password = editUserDTO.getPassword().trim();
        String confirmPassword = editUserDTO.getConfirmedPassword().trim();
        //只有密码
        boolean justPsw = (isBlank(confirmPassword) || isEmpty(confirmPassword))
                && isNotBlank(password) && CharSequenceUtil.isNotEmpty(password);
        //只有确认密码
        boolean justConfPsw = (isBlank(password) || isEmpty(password))
                && isNotBlank(confirmPassword) && CharSequenceUtil.isNotEmpty(confirmPassword);
        if (justPsw || justConfPsw) {
            throw new BizException("如需修改密码，请将“密码”和“确认密码”填写完整");
        } else {
            //密码和确认密码都有
            boolean allPassword = isNotBlank(password) && CharSequenceUtil.isNotEmpty(password)
                    && isNotBlank(confirmPassword.trim()) && CharSequenceUtil.isNotEmpty(confirmPassword);
            if (allPassword) {
                String reg = "^[A-Za-z0-9]+$";
                if (!password.matches(reg) || !confirmPassword.matches(reg)) {
                    throw new BizException("密码只能是字母或数字");
                }
                if (password.length() < 9 || password.length() > 16) {
                    throw new BizException("密码长度在9~16之间");
                }
                if (confirmPassword.length() < 9 || confirmPassword.length() > 16) {
                    throw new BizException("密码长度在9~16之间");
                }
                if (!password.equals(confirmPassword)) {
                    throw new BizException("“密码”和“确认密码”不一致");
                } else {
                    sysUserDO.setPassword(passwordEncoder.encode(password));
                }
            }
        }

        sysUserDO.setStatus(editUserDTO.getStatus());
        if (sysUserManager.modifyUserById(sysUserDO) < 1) {
            throw new BizException("修改用户失败");
        }
        logEventPublisher.publish(Log.builder().append("修改外部用户,ID:" + sysUserDO.getUserId() + ",账号:" + sysUserDO.getAccount()).isUpdate());
    }

    @Override
    public PageResult<UserPageVO> getUserPage(QueryUserPageDTO queryUserPageDTO) {
        startPage(queryUserPageDTO.getPageNum(), Constant.DEFAULT_PAGE_SIZE);
        Page<SysUserBO> sysUserBOS = sysUserManager.queryUserPage(queryUserPageDTO);
        Page<UserPageVO> page = new Page<>();
        if (isNotEmpty(sysUserBOS)) {
            sysUserBOS.forEach(item -> {
                item.setStatusName(SysUserStatusEnum.getName(item.getStatus()));
                item.setUserTypeName(SysUserTypeEnum.getName(item.getUserType()));
                item.setCreateTimeStr(DateUtil.format(item.getCreateTime(), Constant.DATETIME_FORMAT));
                UserPageVO userPageVO = sysUserMapStruct.convertToUserPageVO(item);
                userPageVO.setIsAdmin(IsOrNotEnum.NOT.getValue());
                if (userPageVO.getUserRoleNames().contains(SysRoleEnums.ADMIN.getName())) {
                    userPageVO.setIsAdmin(IsOrNotEnum.IS.getValue());
                }
                page.add(userPageVO);
            });
        }
        return PageResult.of(page.size(), page);
    }

    @Override
    @Transactional
    public void deleteUser(Long userId) {
        SysUserDO existUser = sysUserManager.queryByUserId(userId);
        if (existUser == null) {
            throw new BizException(USER_NULL);
        }
        List<SysRoleDO> sysRoleDOS = sysUserManager.queryRolesByUserId(userId);
        if (sysRoleDOS.stream().anyMatch(item -> item.getRoleId().intValue() == SysRoleEnums.ADMIN.getValue())) {
            throw new BizException("超级管理员不可删除");
        }
        Integer taskNum = projectManager.queryTaskNumByUserId(userId, IsOrNotEnum.NOT.getValue());
        if (taskNum > 0) {
            throw new BizException("当前用户存在未完成的任务,无法删除");
        }
        sysUserManager.removeUserById(existUser.getId());
        logEventPublisher.publish(Log.builder().append("删除用户,ID:" + userId + ",账号:" + existUser.getAccount()).isUpdate());
    }

    @Override
    public LoginUserInfoVO getLoginUserInfo() {
        Long loginUserId = LoginUserUtil.getLoginUserId();
        SysUserDO sysUser = sysUserManager.queryByUserId(loginUserId);
        if (sysUser == null) {
            throw new BizException(USER_NULL);
        }
        LoginUserInfoVO loginUserInfoVO = sysUserMapStruct.convertToLoginUserVO(sysUser);
        //查询角色
        List<SysRoleDO> sysRoles = sysUserManager.queryRolesByUserId(loginUserId);
        List<String> roles = sysRoles.stream().map(SysRoleDO::getRoleCode).distinct().collect(Collectors.toList());
        loginUserInfoVO.setRoles(roles);

        //如果包含超级管理员角色就不需要查权限了
        if (!roles.contains(Constant.ADMIN_ROLE)) {
            //查询分配权限
            List<SysPermDO> sysPerms = sysUserManager.queryPermByUserId(loginUserId);
            //查询默认权限
            List<SysPermDO> defaultSysPerms = sysUserManager.queryDefaultPerm();
            List<String> permissions = Stream.of(sysPerms, defaultSysPerms)
                    .flatMap(Collection::stream)
                    .map(SysPermDO::getPermCode)
                    .distinct()
                    .collect(Collectors.toList());
            loginUserInfoVO.setPermissions(permissions);
        } else {
            //查询默认权限
            List<SysPermDO> defaultSysPerms = sysUserManager.queryDefaultPerm();
            List<String> permissions = defaultSysPerms.stream().map(SysPermDO::getPermCode).distinct().collect(Collectors.toList());
            loginUserInfoVO.setPermissions(permissions);
        }
        return loginUserInfoVO;
    }

    @Override
    @Transactional
    public void assignUserRoles(AssignRolesDTO assignRolesDTO) {
        Long loginUser = LoginUserUtil.getLoginUserId();
        Long userId = assignRolesDTO.getUserId();
        SysUserDO sysUser = sysUserManager.queryByUserId(userId);
        if (sysUser == null) {
            throw new BizException(USER_NULL);
        }
        //不能给自己分配角色
        if (loginUser.equals(userId)) {
            throw new BizException("自己不能修改自己的角色");
        }
        //1、先删除该用户的用户角色关系
        sysUserRoleMapper.deleteByUserId(userId);
        //2、批量插入新的用户角色关系
        Long[] roleIds = assignRolesDTO.getRoleIds();
        if (roleIds == null || roleIds.length == 0) {
            return;
        }
        Log.Builder builder = Log.builder();
        builder.append("给用户,ID:" + userId + ",名称:" + sysUser.getUserName() + ",分配角色\n");
        List<SysUserRoleDO> sysUserRoleList = new ArrayList<>();
        for (Long roleId : roleIds) {
            SysUserRoleDO sysUserRole = new SysUserRoleDO();
            sysUserRole.setUserId(userId);
            sysUserRole.setRoleId(roleId);
            sysUserRole.setCreateTime(LocalDateTime.now());
            sysUserRoleList.add(sysUserRole);
            SysRoleDO sysRole = sysRoleMapper.selectById(roleId);
            if (sysRole != null) {
                builder.append("ID:" + sysRole.getRoleId() + ",角色:" + sysRole.getRoleName() + "\n");
            }
        }
        sysUserRoleMapper.insertBatch(sysUserRoleList);
        logEventPublisher.publish(builder.isUpdate());
    }

    /**
     * 查询待选标注人员
     *
     * @param projectId 项目id
     * @return List<UserBaseInfoVO>
     */
    @Override
    public List<UserBaseInfoVO> getMarkPersonList(Long projectId) {
        List<SysUserBO> userBOS = sysUserManager.queryMarkPersonList(projectId);
        if (isNotEmpty(userBOS)) {
            return userBOS.stream().map(user -> {
                UserBaseInfoVO baseInfoVO = new UserBaseInfoVO();
                baseInfoVO.setUserId(user.getUserId());
                baseInfoVO.setUserName(user.getUserName());
                baseInfoVO.setAccount(user.getAccount());
                baseInfoVO.setUserTypeName(SysUserTypeEnum.getName(user.getUserType()));
                baseInfoVO.setTaskNum(user.getTaskNum());
                return baseInfoVO;
            }).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 退出登录
     */
    @Override
    public void logout() {
        String loginKey = String.format(Constant.ADMIN_LOGIN_KEY, LoginUserUtil.getLoginUserId());
        stringRedisTemplate.delete(loginKey);
    }

    /**
     * 新增内部用户
     *
     * @param addUserDTO 用户信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addInnerUser(AddInnerUserDTO addUserDTO) {
        SysUserDO existUser = sysUserManager.queryByAccount(addUserDTO.getAccount().trim());
        if (existUser != null) {
            throw new BizException("用户账户已存在");
        }
        SysUserDO sysUserDO = new SysUserDO();
        sysUserDO.setUserId(snowflakeIdFactory.nextId());
        sysUserDO.setAccount(addUserDTO.getAccount().trim());
        sysUserDO.setPassword(passwordEncoder.encode(Constant.DEFAULT_PASSWORD));
        sysUserDO.setUserName(addUserDTO.getUserName().trim());
        sysUserDO.setStatus(SysUserStatusEnum.NORMAL.getValue());
        sysUserDO.setUserType(SysUserTypeEnum.INNER_EMPLOYEE.getValue());
        sysUserDO.setDeptId(addUserDTO.getDeptId());
        sysUserManager.addUser(sysUserDO);
        List<Long> roleIds = addUserDTO.getRoleIds();
        LocalDateTime now = LocalDateTime.now();
        List<SysUserRoleDO> sysUserRoleDOS = roleIds.stream().map(roleId -> {
            SysUserRoleDO userRoleDO = new SysUserRoleDO();
            userRoleDO.setUserId(sysUserDO.getUserId());
            userRoleDO.setRoleId(roleId);
            userRoleDO.setCreateTime(now);
            return userRoleDO;
        }).collect(Collectors.toList());
        sysUserManager.batchInsertUserRoles(sysUserRoleDOS);
        logEventPublisher.publish(Log.builder().append("添加内部用户,ID:" + sysUserDO.getUserId() + ",账号:" + sysUserDO.getAccount()).isInsert());
    }

    /**
     * 编辑内部用户
     *
     * @param addUserDTO 用户信息
     */
    @Override
    @Transactional
    public void editInnerUser(AddInnerUserDTO addUserDTO, Long userId) {
        SysUserDO existUser = sysUserManager.queryByUserId(userId);
        if (existUser == null) {
            throw new BizException("内部用户不存在");
        }
        List<SysRoleDO> sysRoleDOS = sysUserManager.queryRolesByUserId(userId);
        if (sysRoleDOS.stream().anyMatch(item -> item.getRoleId().intValue() == SysRoleEnums.ADMIN.getValue())) {
            throw new BizException("超级管理员不可修改");
        }
        SysUserDO sysUserDO = new SysUserDO();
        sysUserDO.setUserId(userId);
        sysUserDO.setAccount(existUser.getAccount());
        sysUserDO.setId(existUser.getId());
        sysUserDO.setStatus(addUserDTO.getStatus());
        sysUserDO.setDeptId(addUserDTO.getDeptId());
        if (sysUserManager.modifyUserById(sysUserDO) < 1) {
            throw new BizException("修改用户失败");
        }
        sysUserManager.removeUserRoleByUserId(userId);
        List<Long> roleIds = addUserDTO.getRoleIds();
        LocalDateTime now = LocalDateTime.now();
        List<SysUserRoleDO> sysUserRoleDOS = roleIds.stream().map(roleId -> {
            SysUserRoleDO userRoleDO = new SysUserRoleDO();
            userRoleDO.setUserId(userId);
            userRoleDO.setRoleId(roleId);
            userRoleDO.setCreateTime(now);
            return userRoleDO;
        }).collect(Collectors.toList());
        sysUserManager.batchInsertUserRoles(sysUserRoleDOS);
        logEventPublisher.publish(Log.builder().append("修改内部用户,ID:" + sysUserDO.getUserId() + ",账号:" + sysUserDO.getAccount()).isUpdate());

    }

    /**
     * 查询内部用户信息
     *
     * @param account 账号
     * @return InnerUserVO
     */
    @Override
    public InnerUserVO getInnerUserInfo(String account) {
        String queryUserUrl = innerUserProperties.getQueryUserUrl();
        account = account.trim();
        InnerUserVO innerUserVO = new InnerUserVO();
        innerUserVO.setDisplayName(UUID.randomUUID().toString().substring(0,20));
        return innerUserVO;
//        return HttpUtil.getInnerUserInfo(queryUserUrl, JSONUtil.toJsonStr(Collections.singletonList(account)));
    }

    /**
     * 内部用户登录
     *
     * @param request 登录信息
     * @return String
     */
    @Override
    public String innerLogin(HttpServletRequest request) {
        // 调用http接口,验证内部用户是否已登录,查询到信息即为已登录
        String tokenKey = innerUserProperties.getTokenKey();
        Cookie cookie = Arrays.stream(request.getCookies()).filter(item -> item.getName().equals(tokenKey)).findFirst().orElse(null);
        if (cookie == null) {
            throw new BizException("获取cookie失败");
        }
        InnerLoginDTO innerLoginDTO = new InnerLoginDTO();
        innerLoginDTO.setUserSource(16);
        innerLoginDTO.setToken(cookie.getValue());
        String checkUserUrl = innerUserProperties.getCheckUserUrl();
        InnerLoginUserVO innerLoginUserInfo = HttpUtil.getInnerLoginUserInfo(checkUserUrl, JSONUtil.toJsonStr(innerLoginDTO));
        return createUserLoginToken(innerLoginUserInfo.getLoginName());
    }

    @Override
    public List<DepartmentTreeVO> getAllDeptNotTree() {
        List<SysDeptDO> allDepts = sysUserManager.getAllDepts();
        return allDepts.stream()
                .map(sysUserMapStruct::convertToDepartmentTreeVO)
                .collect(Collectors.toList());
    }

    /**
     * 生成jwt
     *
     * @param token 授权token
     * @return jwt
     * @throws UnsupportedEncodingException 不支持编码异常
     */
    private String createJwt(String token) throws UnsupportedEncodingException {
        Algorithm algorithm = Algorithm.HMAC256(jwtProperties.getAdminSecret());
        return JWT.create()
                .withIssuer(jwtProperties.getIssuer())
                .withIssuedAt(new Date())
                .withClaim("uid", token)
                .sign(algorithm);
    }

}